/**
 * Created by sasjkp on 4/2/15.
 */
jQuery.sap.declare("sas.modelfactory.control.rangeinput.RangeInput");
jQuery.sap.require("sas.hc.m.Input");

sas.hc.m.Input.extend("sas.modelfactory.control.rangeinput.RangeInput", {
    metadata : {

        properties : {
            min: {type: "float", defaultValue: 0},
            max: {type: "float", defaultValue: 0},
            hasMin: {type: "boolean", defaultValue: false},
            hasMax: {type: "boolean", defaultValue: false},
            excludeMin: {type: "boolean", defaultValue: false},
            excludeMax: {type: "boolean", defaultValue: false},
            isInteger: {type: "boolean", defaultValue: false}
        }
    },

    setIsInteger: function(value) {
        this.setProperty("isInteger", value);
        this.setAllowIntegerOnly(value);
    },

    rangeValidation: function(valueEntered) {
        var valid = true;
        var r = null;
        var numberFormat = sap.ui.core.format.NumberFormat.getFloatInstance({
                groupingEnabled: false,
                groupingSeparator: null
            },
            sap.ui.getCore().getConfiguration().getLocale());
        var parsedNum = numericParse(valueEntered, this);
        if(parsedNum === -1 || parsedNum === -2){
            return displayErrorMessage(parsedNum, this);
        }

        if (this.getIsInteger()){
            // Integer number
            var value = parseInt(parsedNum);
            if (this.getMin() >= 0){
                r = new RegExp("^[\d]+$", "g");
            } else {
                r = new RegExp("^[+-]?[\d]+$", "g");
            }
        } else  {
            // Float number
            var value = parseFloat(parsedNum);
            if (this.getMin() >= 0){
                r = new RegExp("^\d{0,8}(\.\d{1,2})?$", "g");
            } else {
                r = new RegExp("^[+-]?\d{0,8}(\.\d{1,2})?$", "g");
            }
        }

        if (parsedNum === "") {
            valid = false;
            this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.empty.txt"));
        } else {
            var errVal = liveNumericValidation(valueEntered, this);
            displayErrorMessage(errVal, this);
        }

        if (parsedNum.match(r) != null) {
            valid = false;
            this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.invalidNumber.txt"));
        }

        if (this.getHasMin()){
            if (this.getExcludeMin()){
                if (value <= this.getMin()) {
                    valid = false;
                    this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.greaterMinExclude.txt") + this.getMin());
                }
            } else {
                if (value < this.getMin()) {
                    valid = false;
                    this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.greaterMinInclude.txt") + this.getMin());
                }
            }
        }

        if (this.getHasMax()){
            if (this.getExcludeMax()){
                if (value >= this.getMax()) {
                    valid = false;
                    this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.lessMaxExclude.txt")+ this.getMax());
                }
            } else {
                if (value > this.getMax()) {
                    valid = false;
                    this.setValueStateText(sas.modelfactory.RB.getText("rangeinput.validation.lessMaxInclude.txt") + this.getMax());
                }
            }
        }

        if (!valid) {
            this.setValueState(sap.ui.core.ValueState.Error);
        } else {
            this.setValueState(sap.ui.core.ValueState.None);
        }
        return valid;

    }
});

sas.modelfactory.control.rangeinput.RangeInput.prototype._performLiveValidation = function(){
    //logUtil.log("RangeInput: _performLiveValidation: " + this.getValue() + "  last:" + this._lastValue);
    this.setValueState(sap.ui.core.ValueState.None);
    numericValidation( this.getValue(), this);
    if (this.getValueState() === sap.ui.core.ValueState.None) {
       this.rangeValidation(this.getValue(), this);
    }
};

sas.modelfactory.control.rangeinput.RangeInput.prototype._performValidation = function(){
    //    logUtil.log("RangeInput: _performValidation: " + this.getValue() + "  last:" + this._lastValue);
    // ** IMPORTANT ** This empty function overrides the builtin validation which breaks range checking

};

// This event is called on completion of an edit (focus exit, enter key, or escape key)
sas.modelfactory.control.rangeinput.RangeInput.prototype.onChange=function(e){
    if(!this.getEditable()||!this.getEnabled()) {return;}

    if (this.getValueState() === sap.ui.core.ValueState.None) {
        var v=this._getInputValue();
        if(v!==this._lastValue){
            this.setValue(v);
            v=this.getValue();
            this._lastValue=v;
            this.fireChangeEvent(v);
            return true;
        }
    } else {
        this.setValue(this._lastValue);
        this.setValueState(sap.ui.core.ValueState.None);
    }
};


