jQuery.sap.require(
    "sas.modelfactory.proxies.DatasourceProxy",
    "sas.modelfactory.proxies.DataSourceMetaProxy",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.controller("sas.modelfactory.dialog.DefineMetadataDialog", {

    datasourceVarModel: null,
    roleMap: null,
    environmentId: null,
    metadata: null,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf testproject2.testView
     */
	onInit: function() {

        this.datasourceVarModel = new sap.ui.model.json.JSONModel();

        sas.eventBus.subscribe("defineMetadataEvent", "selectedDatasource", this.initDialog, this);

        var enumModel = sap.ui.getCore().getModel("enums");
        this.roleMap = {};
        if (enumModel){
            this.roleMap = enumModel.getProperty("/roles");
        }
	},

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf testproject2.testView
     */
	onBeforeRendering: function() {

	},

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf testproject2.testView
     */
//	onAfterRendering: function() {
//
//	},

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf testproject2.testView
     */
	onExit: function() {
        sas.eventBus.unsubscribe("defineMetadataEvent", "selectedDatasource", this.initDialog, this);
	},

    handleFilter: function(oEvent) {
        logUtil.log("filter hit: ", this.filter.getValue());
        this.table.filter(this.nameCol, this.filter.getValue());
    },

    handleRowSelectionChange: function(oEvent, view) {
        logUtil.log(view.table.getSelectedIndices());

        this.processControls(view);
    },

    processControls: function(view) {
        // Multiple variables selected
        var selected = view.table.getSelectedIndices();
        if (selected.length > 1) {
            view.keyBtn.setEnabled(false);
            view.freqBtn.setEnabled(false);
            if (selected.length > 3) {
                view.segBtn.setEnabled(false);
            }
        } else {
            view.keyBtn.setEnabled(true);
            view.freqBtn.setEnabled(true);
            view.segBtn.setEnabled(true);
        }

        // Single row selected
        if (selected.length > 0 && selected.length <= 1) {
            var model = view.table.getModel();
            var type = model.getProperty("/variables/" + selected[0] + "/type");

            // Frequency rules
            if ((type.key != undefined && type.key != "NUMERIC") || this.hasRole(view, "FREQ")) {
                view.freqBtn.setEnabled(false);
            } else {
                view.freqBtn.setEnabled(true);
            }

            // Key rules
            if (this.hasRole(view, "KEY")) {
                view.keyBtn.setEnabled(false);
            } else {
                view.keyBtn.setEnabled(true);
            }

            // Segment rules
            var segCount = this.roleCount(view, "SEGMENT");
            if (segCount >= 3){
                view.segBtn.setEnabled(false);
            } else {
                view.segBtn.setEnabled(true);
            }

            // Target rules
            var targetCount = this.roleCount(view, "TARGET");
            if (segCount >= 1 && targetCount >= 1){
                view.targetBtn.setEnabled(false);
            } else {
                view.targetBtn.setEnabled(true);
            }
        }
    },

    hasRole: function(view, role) {
        var model = view.table.getModel();
        var num = model.oData.variables.length;
        for (var i=0;i<num;i++) {
            var varRole = model.getProperty("/variables/" + i + "/role/key");
            if (varRole == role) {
                return true;
            }
        }

        return false;
    },

    roleCount: function(view, role) {
        var model = view.table.getModel();
        var num = model.oData.variables.length;
        var count = 0;
        for (var i=0;i<num;i++) {
            var varRole = model.getProperty("/variables/" + i + "/role/key");
            if (varRole == role) {
                count++;
            }
        }

        return count;
    },

    onRolePress: function(evt, view) {
        logUtil.log(evt.getParameter("id"));
        logUtil.log(view.table.getSelectedIndices());
        var controller = this;
        var selected = view.table.getSelectedIndices();
        var roleKey;
        switch (evt.getParameter("id")) {
            case "DefineMetadataDialogView--freqBtn":
                roleKey = "FREQ";
                break;
            case "DefineMetadataDialogView--idBtn":
                roleKey = "ID";
                break;
            case "DefineMetadataDialogView--inputBtn":
                roleKey = "INPUT";
                break;
            case "DefineMetadataDialogView--keyBtn":
                roleKey = "KEY";
                break;
            case "DefineMetadataDialogView--rejectBtn":
                roleKey = "REJECTED";
                break;
            case "DefineMetadataDialogView--segBtn":
                roleKey = "SEGMENT";
                break;
            case "DefineMetadataDialogView--targetBtn":
                roleKey = "TARGET";
                break;
            case "DefineMetadataDialogView--resetBtn":
                roleKey = "RESET";
                break;
        }

        if (roleKey !== "RESET"){
            var role = this.roleMap[roleKey];
            for (var i=0;i<selected.length; i++ ) {
                var context = view.table.getContextByIndex(selected[i]);
                if (context && role) {
                    var path = context.getPath();
                    var model = view.table.getModel();
                    model.setProperty(path + "/role", role);
                }
            }
        } else {
            // Reset the role
            var origData = controller.getView().clonedArray.variables;
            for (var i=0;i<selected.length; i++ ) {
                var context = view.table.getContextByIndex(selected[i]);
                if (context) {
                    var path = context.getPath();
                    var model = view.table.getModel();
                    var name =  model.getProperty(path + "/name");
                    // Find the original value passed in
                    for (var j=0;j<origData.length;j++){
                        if (origData[j].name === name){
                            // Reset the role
                            model.setProperty(path + "/role", origData[j].role);
                        }
                    }
                }
            }
        }

        this.processControls(view);
    },

    getDatasourceVariables: function(links) {
        var view = this.oView;
        var controller = this;
        var model = this.datasourceVarModel;

        var onSuccess = function(oJson){
            model.setData({variables: oJson.items});

            // Apply the dataset metadata
            //controller.getDatasetMeta();
            controller.applyMetadataOverLay();
        };

        var onComplete = function(oJson){
            busySpinner.show(false);
        };

        //datasourceProxy.getDatasourceVariables(id, onSuccess);
        // Get the datasource variables
        if (links) {
            messageProxy.request(links, "variables", "GET", "json", null, onSuccess, null, onComplete);
            busySpinner.show(true,
                sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.title.txt"),
                sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.varMsg.txt"));
        } else {
            logUtil.error("Link Error: List of links is null.");
        }
    },

    applyMetadataOverLay: function() {
        var controller = this;
        var view = controller.getView();
        var metaDataItems = controller.metadata;

        // Overlay any metadata values on top of the data variables
        if (metaDataItems){
            for (var i=0;i<metaDataItems.length;i++){
                var name = metaDataItems[i].name;
                var type = metaDataItems[i].type;
                var role = metaDataItems[i].role;
                var variables = controller.datasourceVarModel.getProperty("/variables");
                if (variables){
                    for (var j=0;j<variables.length;j++){
                        if (variables[j].name === name && variables[j].type.key === type.key){
                            // Found a matching variable.  Overlay the role.
                            var prop = "/variables/" + j + "/role";
                            controller.datasourceVarModel.setProperty(prop, role);
                        }
                    }
                }
            }
        }

        view.setAndBindModel(controller.datasourceVarModel , "/variables");

        controller.processControls(view);
    },

    /*getDatasetMeta: function() {
        var view = this.oView;
        var controller = this;
        var model = this.metadataModel;

        var onEnvirSuccess = function(oJson){
            var envirData = oJson.items[0];

            var onSuccess = function(oJson){
                var metaDataItems = oJson.items;

                // Overlay any metadata values on top of the data variables
                if (metaDataItems){
                    for (var i=0;i<metaDataItems.length;i++){
                        var name = metaDataItems[i].name;
                        var type = metaDataItems[i].type;
                        var role = metaDataItems[i].role;
                        var variables = controller.datasourceVarModel.getProperty("/variables");
                        if (variables){
                            for (var j=0;j<variables.length;j++){
                                if (variables[j].name === name && variables[j].type.key === type.key){
                                    // Found a matching variable.  Overlay the role.
                                    var prop = "/variables/" + j + "/role";
                                        controller.datasourceVarModel.setProperty(prop, role);
                                }
                            }
                        }
                    }
                }

                view.setAndBindModel(controller.datasourceVarModel , "/variables");
            };

            var onComplete = function(oJson){
                controller.processControls(view);
                busySpinner.show(false);
            };

            dataSourceMetaProxy.getMetadata(envirData.id, onSuccess, null, onComplete);
        }

        dataSourceMetaProxy.getEnvironment(onEnvirSuccess);

        busySpinner.show(true,
            sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.title.txt"),
            sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.metadataMsg.txt"));
    },*/

    initDialog : function(channelId, eventId, data) {
        if (data && data.links) {
            this.environmentId = data.environmentId;
            this.metadata = data.metadata;
            this.getDatasourceVariables(data.links);
        }
    }
});