(function() {

    jQuery.sap.require(
        "sas.hc.m.Dialog",
        "sas.hc.m.Button",
        "sas.modelfactory.util.LogUtil"
    );

    sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", {

        dialog: null,
        closeButton: null,
        saveButton: null,
        saveAsButton: null,
        modelTemplateView: null,

        /*
        The default buttons for this dialog only includes the "Close" button.
        If you want the save, save as, then you need to call the
        respective dialog.add<x> method ... eg.
            dialog.addSaveAsButton();
         */
        createContent: function(controller) {
            var me = this;

            sas.applicationSwitcher.updateTitle(sas.config.getAppId(), sas.config.getAppName());
            sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), true);

            this.modelTemplateView = sap.ui.view({type: sap.ui.core.mvc.ViewType.JS, id: "TemplateView", viewName: "sas.modelfactory.templates.ModelTemplates", viewData: me});

            this.saveButton = new sas.hc.m.Button({
                text: sas.modelfactory.RB.getText("viewModelTemplate.button.save.txt"),
                enabled: false,
                press: function() {
                    me.updateModel(me);
                    me.dialog.close();
                    controller.onTemplateSave.call(controller, me.modelTemplateView);
                }
            });

            this.saveAsButton = new sas.hc.m.Button({
                //text: sas.modelfactory.RB.getText("viewModelTemplate.button.saveAs.txt"),
                text: sas.modelfactory.RB.getText("viewModelTemplate.button.saveAs.txt"),
                visible: true,
                press: function() {
                    me.updateModel(me);
                    me.dialog.close();
                    controller.onTemplateSaveAs.call(controller, me.modelTemplateView);
                }
            });

            this.closeButton = new sas.hc.m.Button({
                    text: sas.modelfactory.RB.getText("viewModelTemplate.button.done.txt"),
                    press: function() {
                        me.dialog.close();
                    }
                }),


            this.dialog = new sas.hc.m.Dialog({
                title: sas.modelfactory.RB.getText("viewModelTemplate.title.txt"),
                contentWidth: "85%",
                contentHeight: "85%",
                content: me.modelTemplateView,
                endButton: this.closeButton,   // The default button is close
                afterClose: function() {
                    sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), false);
                    me.dialog.destroy();
                }
            });


            this.dialog.addSaveButton = function() {
                me.dialog.setBeginButton(me.saveButton);
            };

            this.dialog.addSaveAsButton = function() {
                me.dialog.setMiddleButton(me.saveAsButton);
            };

            return me.dialog;
        },


        updateModel: function(me) {
            var model = me.modelTemplateView.getModel().oData.template;
            var use = me.modelTemplateView.useListOrder;
            var modelList = me.modelTemplateView.modelList;
            var ignore = me.modelTemplateView.ignoreListOrder;
            var lastUsePos = 3; // Skip sample and partition

            logUtil.log("use order:", use);
            logUtil.log("ignore order:", ignore);

            // Walk through each item in the use list
            for (var i=0; i<use.length; i++) {
                // Find the matching component in the model
                for (var j=0; j<model.properties.length; j++) {
                    var compItem = model.properties[j];
                    if (compItem) {
                        //logUtil.log("ModelTemplates view component:", compItem.groupName, compItem.componentName, compItem.propertyName, compItem.propertyValue, compItem.position);
                        if (compItem.groupName === "Modify") {
                            if (compItem.propertyName === "_use") {
                                if (compItem.componentName === use[i]) {
                                    // Update the position in the model
                                    compItem.position = i + 3; // Adjust for Sample and Partition
                                    lastUsePos = compItem.position;
                                    // Update the propertyValue
                                    compItem.propertyValue = "Y";
                                    logUtil.log("Component:", compItem.componentName, compItem.propertyName, "placed in position", compItem.position, "with use value", compItem.propertyValue);
                                } else if (compItem.componentName === "Sample" || compItem.componentName === "Partition"){
                                    logUtil.log("Component:", compItem.componentName, compItem.propertyName, "placed in position", compItem.position, "with use value", compItem.propertyValue);
                                }
                            }
                        }
                    }
                }
            }

            // Update the model and score positions
            for (var j=0; j<model.properties.length; j++) {
                var compItem = model.properties[j];
                if (compItem) {
                    //logUtil.log("ModelTemplates view component:", compItem.groupName, compItem.componentName, compItem.propertyName, compItem.propertyValue, compItem.position);
                    if (compItem.groupName === "Model") {
                        if (compItem.propertyName === "_use" && compItem.propertyValue === "Y") {
                            // Update the position in the model
                            compItem.position = lastUsePos + 1;
                            lastUsePos = compItem.position;
                            // Update the propertyValue
                            logUtil.log("Component:", compItem.componentName, compItem.propertyName, "placed in position", compItem.position, "with use value", compItem.propertyValue);
                        }
                    } else if (compItem.groupName === "Report") {
                        if (compItem.propertyName === "_use" && compItem.propertyValue === "Y") {
                            if (compItem.componentName === "Score") {
                                // Update the position in the model
                                compItem.position = lastUsePos + 1;
                                lastUsePos = compItem.position;
                                // Update the propertyValue
                                logUtil.log("Component:", compItem.componentName, compItem.propertyName, "placed in position", compItem.position, "with use value", compItem.propertyValue);
                            }
                        }
                    }
                }
            }

            // Walk through each item in the ignore list
            for (var i=0; i<ignore.length; i++) {
                // Find the matching component in the model
                for (var j=0; j<model.properties.length; j++) {
                    var compItem = model.properties[j];
                    if (compItem) {
                        //logUtil.log("ModelTemplates view component:", compItem.groupName, compItem.componentName, compItem.propertyName, compItem.propertyValue, compItem.position);
                        if (compItem.groupName === "Modify") {
                            if (compItem.propertyName === "_use") {
                                if (compItem.componentName === ignore[i]) {
                                    // Update the position in the model
                                    compItem.position = 0;
                                    // Update the propertyValue
                                    compItem.propertyValue = "N";
                                    logUtil.log("Component:", compItem.componentName, compItem.propertyName, "placed in position", compItem.position, "with use value", compItem.propertyValue);
                                }
                            }
                        }
                    }
                }
            }
            me.modelTemplateView.getModel().setProperty("/template", model);
        }
    });

})();