(function() {
    
    jQuery.sap.require(
        "sap.ui.model.Sorter",
        "sap.ui.model.json.JSONModel",
        "sas.hc.m.Button",
        "sas.hc.ui.commons.Accordion",
        "sas.hc.ui.commons.AccordionSection",
        "sas.hc.m.CheckBox",
        "sas.hc.m.Dialog",
        "sas.hc.m.FlexBox",
        "sas.hc.m.IconTabFilter",
        "sas.hc.m.Input",
        "sas.hc.m.Label",
        "sas.hc.m.List",
        "sas.hc.m.StandardListItem",
        "sas.hc.m.Panel",
        "sas.hc.m.Select",
        "sas.hc.m.Text",
        "sas.hc.m.HelpPopup",
        "sas.modelfactory.ModelFactoryEvents",
        "sas.modelfactory.proxies.PreferencesProxy"
    );
    
    sap.ui.jsview("sas.modelfactory.dialog.NewProjectDialog", {

        dataSourceSelect: null,
        varList: null,
        name: null,
        partCheck: null,
        metadataPathLabel: null,

        getControllerName : function() {
            return "sas.modelfactory.dialog.NewProjectDialog";
        },

        createContent: function(controller) {
            var me = this;

            this.dataSourceSelect = new sas.hc.m.Select('datasourceselect', {
                autoAdjustWidth: false,
                width: "100%",
                items: {
                    path: "/datasetData",
                    sorter: new sap.ui.model.Sorter("name", false),
                    template: new sap.ui.core.Item({
                        key: "{id}",
                        text: "{name}",
                        tooltip: "{library}"
                    })
                },
                selectedKey: "{/statuskey}",
                change : [controller.onDataSourceSelectorChange, controller]
            });

            // Create the List, but make it invisible until the target/segment variables are known
            this.varList = new sas.hc.m.List({
                noDataText: " ",
                visible: false
//                mode: sap.m.ListMode.Delete, "delete": function (evt) {
//                    var item = evt.getParameter("listItem");
//                    var obj = evt.getSource();
//                    obj.removeItem(item);
//                    if (obj.getItems().length === 0) {
//                        obj.setVisible(false);
//                    }
//                }
            }).addStyleClass("newProjectVariablesList").addStyleClass("newProjectDialog-labelPadding");

            this.name = new sas.hc.m.Input({
                type: sap.m.InputType.Text,
                placeholder: sas.modelfactory.RB.getText("new.project.placeholder.name.txt"),
                required: true,
                liveChange: function(evt) {
                    var text = this.getValue();
                    var state = (text.length > 0);
                    var bus = sas.eventBus;
                    bus.publish("newProjectDialog", "buttonState", {"enabled": state});
                }
            });

            var browseButton = new sas.hc.m.Button({
                text: sas.modelfactory.RB.getText("new.project.button.browse.txt"),
                width: "100%",
                press:function(){
                    controller.openDialog();
                }
            }).addStyleClass("newProjectDialog-browseBtn");

            this.metadataPathLabel = new sas.hc.m.Input({required: true});
            this.metadataPathLabel.setEditable(false);
            this.metadataPathLabel.setWidth("22.8rem");

            var buttonHBox = new sap.ui.layout.HorizontalLayout(this.createId("BrowseButtonHBoxLayout"), {
                content: [
                    this.metadataPathLabel,
                    browseButton
                ]
            }).addStyleClass("newProjectDialog-hBox");

            // Determine a number to use for the auto-generated project name
            var maxNum = 0;
            var model = sap.ui.getCore().getModel("projectList");
            if (model){
                var projects = model.getProperty("/projectList");
                if (projects !== undefined && projects !== null && projects.length > 0) {
                    for (var i = 0; i < projects.length; i++) {
                        var parts = projects[i].displayName.split(" ");
                        if (parts.length > 0) {
                            var thisNum = parseInt(parts[parts.length-1]);
                            if (!isNaN(thisNum) && thisNum > maxNum)
                                maxNum = thisNum;
                        }
                    }
                }
            }

            projNum = maxNum + 1;

            me.name.setValue(sas.modelfactory.RB.getText("new.project.default.name.txt") + " " + projNum);

            var createProjectForm = new sas.hc.m.Panel({
                id: "createProjectForm",
                width: "100%",
                content: [
                    new sas.hc.m.Label({
                        text: sas.modelfactory.RB.getText("new.project.name.txt"),
                        required: true
                    }),
                    this.name,
                    new sas.hc.m.Label({
                        text: sas.modelfactory.RB.getText("new.project.metadata.txt"),
                        required: true,
                        width: "100%"
                    }).addStyleClass("newProjectDialog-labelPadding"),
                    buttonHBox,
                    new sas.hc.m.FlexBox({
                        height: "25px",
                        justifyContent: sap.m.FlexJustifyContent.SpaceBetween,
                        items: [
                            new sas.hc.m.Label({
                                text: sas.modelfactory.RB.getText("new.project.datasource.txt"),
                                required: true
                            }),
                            new sas.hc.m.Link({
                                text: sas.modelfactory.RB.getText("new.project.datasource.link.txt"),
                                press: function (evt) {
                                    controller.onNewDataSourcePress.call(controller);
                                }
                            })
                        ]
                    }).addStyleClass("newProjectDialog-labelPadding"),
                    this.dataSourceSelect,
                    this.varList,
                    new sas.hc.m.FlexBox({
                        height: "25px",
                        justifyContent: sap.m.FlexJustifyContent.SpaceBetween,
                        items: [
                            this.partCheck = new sas.hc.m.CheckBox({
                                text: sas.modelfactory.RB.getText("new.project.partition.txt"),
                                selected: true
                            }).addStyleClass("newProjectDialog-labelPadding"),
                            new sas.hc.m.HelpPopup({
                                library: "sas.modelfactory",
                                key: "newProjectDialog"
                            })
                        ]
                    })
                ]
            });

            return createProjectForm;
        },

        setModel: function(model) {
            this.dataSourceSelect.setModel(model);

            // Fire the data source change event to update target/segment variables list
            var selected = this.dataSourceSelect.getSelectedItem();
            if (!selected && model.oData.datasetData.length > 0){
                selected = model.oData.datasetData[0];
                this.dataSourceSelect.setSelectedItem(selected);
            }

            if (selected) {
                this.dataSourceSelect.fireChange({selectedItem: selected});

                var bus = sas.eventBus;
                bus.publish("newProjectDialog", "buttonState", {"enabled": true});
            }
        }

    });

})();