/**
 * Created by sajkp on 1/26/15.
 */

(function() {

    jQuery.sap.require(
        "sas.hc.m.Button",
        "sas.hc.m.Dialog",
        "sas.hc.m.Label",
        "sas.hc.ui.layout.GridData",
        "sas.hc.ui.commons.RadioButtonGroup",
        "sas.hc.ui.core.Item",
        "sas.hc.ui.layout.form.Form",
        "sas.hc.ui.layout.form.ResponsiveGridLayout",
        "sas.hc.ui.layout.form.FormContainer",
        "sas.modelfactory.proxies.ProjectProxy",
        "sas.modelfactory.util.LogUtil"
    );

    sap.ui.jsfragment("sas.modelfactory.dialog.RegisterModelsDialog", {

        modelGrp: null,
        project: null,
        controller: null,

        createContent: function(controller) {
            var me = this;
            this.controller = controller;
            this.project = controller.registerModelsFragData;

            this.modelGrp = new sas.hc.ui.commons.RadioButtonGroup({
            });
            var oItem = new sas.hc.ui.core.Item({
                text : sas.modelfactory.RB.getText("registerModelsDialog.allModels.txt"),
                tooltip : sas.modelfactory.RB.getText("registerModelsDialog.tooltip.allModels.txt"),
                key : "ALL_MODELS"
            });
            this.modelGrp.addItem(oItem);
            oItem = new sas.hc.ui.core.Item({
                text : sas.modelfactory.RB.getText("registerModelsDialog.championModels.txt"),
                tooltip : sas.modelfactory.RB.getText("registerModelsDialog.tooltip.championModels.txt"),
                key : "CHAMPION_MODELS"
            });
            this.modelGrp.addItem(oItem);

            var modelGrpLabel = new sas.hc.m.Label({
                text: sas.modelfactory.RB.getText("registerModelsDialog.model.title.txt") + ": " + this.project.displayName,
                labelFor: this.modelGrp,
                required: false,
                layoutData: new sas.hc.ui.layout.GridData({span: "L2 M2 S12"})
            });
            this.modelGrp.setSelectedIndex(1);    // Default to champion


            var form =  new sas.hc.ui.layout.form.Form({
                layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                    // change M breakpoint so the label is to the left of the
                    // field for the default dialog size.
                    breakpointM: 600
                }),
                formContainers: [
                    new sas.hc.ui.layout.form.FormContainer({
                        formElements: [
                            new sas.hc.ui.layout.form.FormElement({
                                label: modelGrpLabel,
                                fields: [this.modelGrp]
                            })
                        ]
                    })
                ]
            })

            var dialog = new sas.hc.m.Dialog({
                title: sas.modelfactory.RB.getText("registerModelsDialog.title.txt"),
//                contentWidth: "600px",
//                contentHeight: "300px",
                content: [form],
                controller: controller,
                project: this.project,
                beginButton: new sas.hc.m.Button({
                    text: sas.modelfactory.RB.getText("registerModelsDialog.button.register.txt"),
                    press: function() {
                        me.onRegisterModels(controller, me);
                        dialog.close();
                    }
                }),
                endButton: new sas.hc.m.Button({
                    text: sas.modelfactory.RB.getText("registerModelsDialog.button.cancel.txt"),
                    press: function() {
                        dialog.close();
                        controller.unlockProject(controller.registerModelsFragData);
                    }
                }),
                afterClose: function() {
                    dialog.destroy();
                }
            });

            return dialog;
        },

        onRegisterModels: function(dialog) {

            var project = this.project;
            var controller = this.controller;

            var onSuccess = function() {
                logUtil.log("Project registration onSuccess.");
                // Update the project table

                // begin polling @TODO
                controller.refresh();
            };

            var onError = function(xhr, err, msg){
                logUtil.log("Project registration onError.");
                controller.unlockProject(project);
                var json = xhr.responseJSON;
                var errMsg = sas.modelfactory.RB.getText("registerModelsDialog.error.registerModels.txt") +
                    "\n\n" + sas.modelfactory.RB.getText("registerModelsDialog.error.status.txt") + xhr.status +
                    "\n\n " + json.message;
                sas.hc.m.MessageBox.error(errMsg);
            };

            var onComplete = function(){

            };

            var models = "full";

            // Build the data options: "full" or "champion"
            var selItem = this.modelGrp.getSelectedItem();
            if (selItem){
                var key = selItem.getKey();
                if (key === "CHAMPION_MODELS"){
                    models = "champion";
                }
            }

            // Request to register the models
            projectProxy.registerProject(this.project.id, models, onSuccess, onError, onComplete);
        }
    });

})();