jQuery.sap.require(
    "sas.modelfactory.proxies.DatasourceProxy",
    "sas.modelfactory.proxies.DataSourceMetaProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.util.ContentUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil",
    "sas.hc.m.StandardListItem"
);

sap.ui.controller("sas.modelfactory.dialog.RetrainDialog", {

    view: null,
    datasourceModel: null,
    projectModel: null,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     */
	onInit: function() {

        this.view = this.getView();
        /*this.datasourceModel = sap.ui.getCore().getModel("datasources");
        if (this.datasourceModel){
            // the datasources have already been loaded
            this.projectModel = sap.ui.getCore().getModel("openProject");
            if (this.projectModel) {
                var memberName = this.projectModel.getProperty("/project/memberName");
                this.datasourceModel.setData({"memberName": memberName}, true);
                this.view.setModel(this.datasourceModel);
            }
        }
        else */{
            // we need to load datasources
            this.getDataset();
        }
        this.refreshMessages(this.view.msgList);

        sas.eventBus.subscribe("newProjectEvent", "initialSelection", this.handleEvent, this);
    },

    onAfterRendering: function() {
        var model = this.view.dataSourceSelect.getModel();
        if (model && model.oData.datasetData){
            var bus = sas.eventBus;
            bus.publish("retrainDialog", "buttonState", {"enabled": (model.oData.datasetData.length > 0)});
        }
    },

    onExit: function() {
        sas.eventBus.unsubscribe("newProjectEvent", "initialSelection", this.handleEvent, this);
    },

    handleEvent: function(channel, msg, data) {
        var view = this.oView;
        if (data && data.key) {
            // Set the selected datasource
            view.dataSourceSelect.setSelectedKey(data.key);

            // Fire the change event for the data source selector
            var selectedItem = view.dataSourceSelect.getItemByKey(data.key);
            var evt = {selectedItem: selectedItem};
            view.dataSourceSelect.fireChange(evt);
        }
    },

    getDataset: function(onComplete) {
        // we need to load data from midtier
        var controller = this;
        this.datasourceModel = new sap.ui.model.json.JSONModel();
        var model = this.datasourceModel;
        var view = this.oView;
        var handleDatasetJson = function(oJson){
            logUtil.log("Datasources: ", oJson);
            busySpinner.show(false);
            var t = {datasetData: oJson.items};
            model.setData(t);
            this.projectModel = sap.ui.getCore().getModel("openProject");
            if (this.projectModel) {
                model.setData({memberName: this.projectModel.oData.project.memberName}, true);
            }
            view.setModel(model);

            sap.ui.getCore().setModel(model, "datasources");
        };
        datasourceProxy.getDatasources(handleDatasetJson, null, onComplete);
        if (onComplete === undefined){
            busySpinner.show(true,
                sas.modelfactory.RB.getText("retrainDialog.busy.title.txt"),
                sas.modelfactory.RB.getText("retrainDialog.busy.description.txt"));
        }
    },

    refreshMessages: function(selector) {
        var msgList = [];
        var msg;
        var item;
        var str;

        // build the messages:
        // All segments warning...
        var item = new sas.hc.m.StandardListItem({
            title: sas.modelfactory.RB.getText("retrainDialog.all.segments.txt"),
            icon: sas.icons.HC.ALERT
        }).addStyleClass("ProjectDataView-warning");
        msgList.push(item);

        // Remove existing
        selector.removeAllItems();

        // If there are messages in the msgList array, add them to the List (aka selector) control
        if (msgList.length > 0) {
            // Add each one and make it selected
            for (var i = 0; i < msgList.length; i++) {
                selector.addItem(msgList[i]);
            }
            // Make the list control visible
            selector.setVisible(true);
        }
        else
            selector.setVisible(false);
    },


    onNewDataSourcePress : function() {
        var controller = this;

        // Go straight to the content navigator
        controller.onSASMetaDataButtonPressed.call(controller)
    },

    onNewDataSourceSave: function(view) {
        logUtil.log("onNewDataSourceSave: ", view);
    },

    onDataSourceSelectorChange: function(evt) {
        var controller = this;
        var targetVars = [];
        var segmentVars = [];

        // Turn off the target/segment count bars initially
        //controller.getView().msgList.setVisible(false);

        var onSuccess = function(oJson) {
            if (oJson.items.length > 0) {
                var items = oJson.items;
                for (var i = 0; i < items.length; i++) {
                    var variable = items[i];
                    if (variable.role.key === "TARGET")
                        targetVars.push(variable.name);
                    else if (variable.role.key === "SEGMENT")
                        segmentVars.push(variable.name);
                }
            }
        };

        var onComplete = function() {
            controller.refreshMessages(controller.getView().msgList);
        };

        var id;
        var selectedItem = evt.getParameter("selectedItem");
        if (selectedItem.id);
            id = selectedItem.id;

        if (id === undefined)
            id = selectedItem.getProperty("key");

        // Search the datasource model for the currently selected item
        var items = controller.getView().dataSourceSelect.getModel().getData().datasetData;
        for (var i = 0; i < items.length; i++) {

            // If we found the datasource selection
            if (items[i].id === id) {
                selectedItem = items[i];

                // Get target and segment count data
                if (selectedItem && selectedItem.links) {
                    messageProxy.request(selectedItem.links, "checkForPossibleTargetsAndSegments", "GET", "json", null, onSuccess, null, onComplete);
                }

                break;
            }
        }
    },

    onSASMetaDataButtonPressed : function() {
        var controller = this;

        var onDatasourceSelected = function(data) {
            onSuccess = function(oJson, status, xhr){
                // The newly created datasource
                var loc = xhr.getResponseHeader('Location');
                var id = loc.substring(loc.lastIndexOf("/")+1, loc.length);

                if (id) {
                    // Update the datasource table
                    controller.getDataset(function(){
                        sas.eventBus.publish("newProjectEvent", "initialSelection", {key: id});
                    });
                }
            };

            var onError = function(xhr, err, msg) {
                var errMsg = sas.modelfactory.RB.getText("datasetView.error.label.txt");
                var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            onComplete = function(oJson) {
                busySpinner.show(false);
            };

            datasourceProxy.createDatasource(data, onSuccess, null, onComplete);
        };

        sas.modelfactory.util.ContentUtil.showDatasourceBrowser(onDatasourceSelected);
    }
});