(function() {

    jQuery.sap.require(
        "sap.ui.layout.HorizontalLayout",
        "sas.hc.m.Button",
        "sas.hc.m.CheckBox",
        "sas.hc.m.Dialog",
        "sas.hc.ui.layout.Grid",
        "sas.hc.m.Label",
        "sas.hc.m.Panel",
        "sas.hc.m.ToolbarSpacer",
        "sap.m.RadioButton",
        "sas.hc.m.HelpPopup",
        "sap.ui.layout.VerticalLayout",
        "sas.modelfactory.ModelFactoryEvents",
        "sas.modelfactory.proxies.ChampionProxy"
    );

    sap.ui.jsview("sas.modelfactory.dialog.projectdefaultsettings.Rules", {

        criterionLabel: null,
        criterionSelection: null,
        depthLabel: null,
        depthInput: null,
        minObservationsLabel: null,
        minObservationsInput: null,
        minEventRateLabel: null,
        minEventRateInput: null,
        numberFormat: null,

        getControllerName : function() {
            return "sas.modelfactory.dialog.projectdefaultsettings.Rules";
        },

        createContent: function(controller) {
            var me = this;

            this.criterionLabel = new sas.hc.m.Label({
                text: sas.modelfactory.RB.getText("championCriteria.criterion.txt"),
                visible: false
            }).addStyleClass("sapMLabel sapMRbBLabel");

            this.criterionSelect = new sas.hc.m.Select({
                //width: "300px",
                items: {
                    path: "/criteriaSelections",
                    template: new sap.ui.core.Item({
                        key: "{key}",
                        text: "{i18nResourceName}"
                    })
                },
                visible: false,
                selectedKey: "{/criterionSelection}",
                change : [controller.onCriterionSelected, controller]
            }).addStyleClass("editMetadataDialog-dropdown");

            this.depthLabel = new sas.hc.m.Label({
                visible: false,
                text: sas.modelfactory.RB.getText("championCriteria.depth.txt")
            }).addStyleClass("sapMLabel sapMRbBLabel");

            this.depthSelect = new sas.hc.m.Select({
                width: "100px",
                items: {
                    path: "/depth",
                    template: new sap.ui.core.Item({
                        key: "{Id}",
                        text: "{Name}"
                    })
                },
                selectedKey: "{/depthSelection}",
                visible: false,
                change : function(oControlEvent) {
                }
            }).addStyleClass("editMetadataDialog-dropdown");

            this.minObservationsLabel = new sas.hc.m.Label({
                text: sas.modelfactory.RB.getText("rules.minObservations.txt")
            }).addStyleClass("sapMLabel sapMRbBLabel");

            this.minObservationsInput = new sas.modelfactory.control.rangeinput.RangeInput({
                width: "100px",
                type: "Number",
                //path: "/minObservations",
                allowIntegerOnly: true,
                min: 1,
                hasMin: true,
                hasMax: false,
                excludeMin: false,
                isInteger: true,
                enableGroupingSeparator: false
            }).addStyleClass("editMetadataDialog-textInput");

            this.minEventRateLabel = new sas.hc.m.Label({
                text: sas.modelfactory.RB.getText("rules.minEventRate.txt")
            }).addStyleClass("sapMLabel sapMRbBLabel");

            this.minEventRateInput = new sas.modelfactory.control.rangeinput.RangeInput({
                width: "100px",
                //path: "/minEventRate",
                type: "Number",
                allowIntegerOnly: false,
                min: 0,
                max: 1,
                hasMin: true,
                hasMax: true,
                excludeMin: true,
                excludeMax: true,
                enableGroupingSeparator: false
            }).addStyleClass("editMetadataDialog-textInput");

            var controls = new sap.ui.layout.VerticalLayout({
                width: "100%",
                content: [
                    this.minObservationsLabel,
                    this.minObservationsInput,
                    this.minEventRateLabel,
                    this.minEventRateInput,
                    this.criterionLabel,
                    this.criterionSelect,
                    this.depthLabel,
                    this.depthSelect
                ]
            });

            var helpIcon = new sas.hc.m.HelpPopup({
                library: "sas.modelfactory",
                key: "editProjectRulesSettings"
            });

            var championCriteriaPanel = new sas.hc.m.Panel({
                headerToolbar: new sas.hc.m.Toolbar({
                    height: "41px",
                    content:[
                        new sas.hc.m.Label({
                        text: sas.modelfactory.RB.getText("projectDefaultSettings.rules.tab.txt")
                        }),
                        new sas.hc.m.ToolbarSpacer(),
                        helpIcon
                    ]
                }),
                width: "100%",
                showCollapseIcon: false,
                content: [
                    controls
                ]
            });

            return championCriteriaPanel;
        },

        setModel: function(project) {
            var controller = this.getController();
            var view = this;

            var onSuccess = function(oJson) {
                var championCriteriaModel = null;
                var championKey = null;
                if (controller.targetType === "INTERVAL") {
                    championKey =  project.intervalChampionVariable.key;
                } else {
                    if (controller.targetType !== "UNDEFINED") {
                        championKey =  project.championVariable.key;
                    }
                }
                championCriteriaModel = new sap.ui.model.json.JSONModel({criteriaSelections: oJson.items, criterionSelection: championKey});
                view.criterionSelect.setModel(championCriteriaModel);

                // Fire the change event to enable/disable Depth based upon the current criterion selection
                view.criterionSelect.fireChange({selectedItem: championKey});
            }

            var onError = function(xhr, err, msg) {
                view.criterionSelect.setEnabled(false);
            };

            var onComplete = function(oJson) {
            }

            if (controller.targetType !== "UNDEFINED") {
                // Retrieve the list of criterion selections for user selection
                championProxy.getChampionVariables(controller.targetType, onSuccess, onError, onComplete);
            } else {
                view.criterionSelect.setEnabled(false);
            }

            this.numberFormat = sap.ui.core.format.NumberFormat.getFloatInstance({
                    groupingEnabled: false,
                    groupingSeparator: null
                },
                sap.ui.getCore().getConfiguration().getLocale());

            var depth = project.depth;
            if (depth === "")
                depth = "10";

            var championDepthModel = new sap.ui.model.json.JSONModel({depth: controller.championDepthData, depthSelection: depth});
            this.depthSelect.setModel(championDepthModel);

            var minObservationsValue = this.numberFormat.format(project.minObservations);
            this.minObservationsInput.setValue(minObservationsValue);

            var eventPercent = project.minEventRate;

            var minEventRateValue = this.numberFormat.format(eventPercent);
            this.minEventRateInput.setValue(minEventRateValue);
        }
    });

})();