jQuery.sap.require(
    "sas.modelfactory.proxies.DatasourceProxy",
    "sas.modelfactory.proxies.DataSourceMetaProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.util.RootUtil",
    "sas.modelfactory.util.ContentUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil",
    "sas.hc.m.MessageBox"
);

sap.ui.controller("sas.modelfactory.main.DatasourceTable", {

    datasourceModel: null,
    metadataModel: null,
    environmentId: null,
    fragInitData: null,
    dataLoaded: false,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.modelfactory.main.DatasourceTable
     */
    onInit: function() {

        var bus = sas.eventBus;
        bus.subscribe("initialDataLoadEvent", "showView", this.refreshData, this);

        this.datasourceModel = sap.ui.getCore().getModel("datasources");
        this.metadataModel = new sap.ui.model.json.JSONModel();
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.main.DatasourceTable
     */
    onBeforeRendering: function() {

        logUtil.log("sas.modelfactory.main.DatasourceTable onBeforeRendering called.");
    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.main.DatasourceTable
     */
    onAfterRendering: function() {
        //this.processControls();
    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.main.DatasourceTable
     */
	onExit: function() {
        sas.eventBus.unsubscribe("initialDataLoadEvent", "showView", this.refreshData, this);
	},

    refreshData: function(channel, msg, data) {
        if (data == "DATASOURCES_VIEW") {
            this.getDataset();
        }
    },

    onNewPress: function(selectedObj) {
        var controller = this;

        // New data source dialog
        //var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewDataSourceDialog", this);
        //dialog.setModel(this.getView().getModel());
        //dialog.open();

        // Go straight to the content navigator
        controller.onSASMetaDataButtonPressed.call(controller)
    },

    onNewDataSourceSave: function() {

    },

    getDataset: function() {
        var controller = this;
        var model = new sap.ui.model.json.JSONModel();
        var view = this.oView;
        this.datasourceModel = model;

        var onSuccess = function (oJson)
        {
            logUtil.log("Datasources: ", oJson);
            if (!controller.dataLoaded){
                controller.dataLoaded = true;
                model.setData({datasetData: oJson.items});
                view.setAndBindModel(model, "/datasetData");
            } else {
                // Update the data
                view.datasetTable.getModel().setProperty("/datasetData", oJson.items);
            }
            var dsmodel = sap.ui.getCore().getModel("datasources");
            if(dsmodel === undefined){
                dsmodel = new sap.ui.model.json.JSONModel();
                sap.ui.getCore().setModel(dsmodel, "datasources");
            }
            sap.ui.getCore().getModel("datasources").setProperty("/datasetData", oJson.items);
        };

        var onComplete = function(oJson) {
            if (!controller.environmentId){
                controller.getEnvironment();
            }
        };

        //datasourceProxy.getDatasources(onSuccess);
        var rootModel = sap.ui.getCore().getModel("root");
        if (rootModel === undefined) {
            // Root links not loaded yet
            sas.modelfactory.util.RootUtil.getRootLinks(controller, controller.getDataset);
        } else {
            var links = rootModel.getProperty("/root");

            // Get the datasource list
            messageProxy.request(links, "datasources", "GET", "json", null, onSuccess, null, onComplete);
        }
    },

    getDatasetMeta: function(links) {
        var view = this.oView;
        var controller = this;

        var handleDatasetMetaJson = function(oJson){
            controller.metadataModel.setData({metaData: oJson.items});

            // Pass data to the view
            var data = {
                links: links,
                environmentId: controller.environmentId,
                metadata: oJson.items
            };
            sas.eventBus.publish("defineMetadataEvent", "selectedDatasource", data);
        };

        var onComplete = function(oJson){
            busySpinner.show(false);
        };

        dataSourceMetaProxy.getMetadata(controller.environmentId, handleDatasetMetaJson, null, onComplete);
        busySpinner.show(true,
            sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.title.txt"),
            sas.modelfactory.RB.getText("defineMetadataDialog.busyDialog.metadataMsg.txt"));
    },

    getEnvironment: function() {
        var view = this.oView;
        var controller = this;

        var onSuccess = function(oJson){
            // There should only be one environment.  Get the first one.
            var envirData = oJson.items[0];
            if (envirData){
                controller.environmentId = envirData.id;
            }
        }

        dataSourceMetaProxy.getEnvironment(onSuccess)
    },

    updateActions: function() {
        var view = this.oView;
        var selectedItems = this.getView().objectManager.getSelectedItems();

        if (selectedItems.length > 1) {
            view.newBtn.setEnabled(false);
            view.defineBtn.setEnabled(false);
            view.removeBtn.setEnabled(true);
        } else if (selectedItems.length == 1) {
            view.newBtn.setEnabled(true);
            view.defineBtn.setEnabled(true);
            view.removeBtn.setEnabled(true);
        } else {
            view.newBtn.setEnabled(false);
            view.defineBtn.setEnabled(false);
            view.removeBtn.setEnabled(false);
        }
    },

    /*onNewProject: function(evt) {
        var index = this.oView.datasetTable.getSelectedIndex();
        var context = this.oView.datasetTable.getContextByIndex(index);
        if (context) {
            var selectedData = context.getObject();
            logUtil.log(selectedData);

            var appView = this.oView.getViewData();
            if (appView) {
                var appController = appView.getController();
                if (appController) {

                    // Navigate to the project view
                    appController.showView("PROJECTS_VIEW");

                    // Pass the create project dialog over to the ProjectTable controller
                    var nav = this.oView.getParent();
                    if (nav) {
                        var projView = nav.getPages()[0];
                        if (projView && projView.getController()){
                            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewProjectDialog", projView.getController());
                            sas.eventBus.publish("newProjectEvent", "initialSelection", {key: selectedData.id});
                            dialog.open();
                        }
                    }
                }
            }
        }
    },*/

    onNewProject: function(evt) {
        var view = this.getView();
        var items = view.objectManager.getSelectedItems();
        var selectedData = items[0];
        if (selectedData) {
            logUtil.log(selectedData);
            view.objectManager.setEditMode(false);

            var appView = this.oView.getViewData();
            if (appView) {
                var appController = appView.getController();
                if (appController) {

                    // Navigate to the project view
                    appController.showView("PROJECTS_VIEW");

                    // Pass the create project dialog over to the ProjectTable controller
                    var nav = this.oView.getParent();
                    if (nav) {
                        var projView = nav.getPages()[0];
                        if (projView && projView.getController()){
                            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewProjectDialog", projView.getController());
                            sas.eventBus.publish("newProjectEvent", "initialSelection", {key: selectedData.id});
                            dialog.open();
                        }
                    }
                }
            }
        }
    },

    /*onDefine: function(evt) {

        var index = this.oView.datasetTable.getSelectedIndex();
        logUtil.log(index);

        var context = this.oView.datasetTable.getContextByIndex(index);
        if (context) {
            var selectedData = context.getObject();
            logUtil.log(selectedData.id);

            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.DefineMetadataDialog", this);
            dialog.open();

            // Pass data to the view
            var bus = sas.eventBus;
            bus.publish("defineMetadataEvent", "selectedDatasource", {links: selectedData.links});
        }
    },*/

    onDefine: function(evt) {
        var controller = this;
        var items = this.getView().objectManager.getSelectedItems();
        var selectedData = items[0];
        if (selectedData) {
            logUtil.log(selectedData.id);

            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.DefineMetadataDialog", this);
            dialog.open();

            // Pass data to the view
            //var bus = sas.eventBus;
            //bus.publish("defineMetadataEvent", "selectedDatasource", {links: selectedData.links});

            // Get the metadata
            controller.getDatasetMeta(selectedData.links);
        }
    },

    deleteDatasources: function(){
        var view = this.oView;
        var items = this.getView().objectManager.getSelectedItems();
        for (var i=0;i<items.length; i++ ) {
            var item = items[i];
            if (item) {
                // Delete each datasource
                var id = item.id;
                logUtil.log("Delete datasource: ", item.name, item.id);
                var onComplete = function(oJson){
                    view.objectManager.setEditMode(false);
                };

                datasourceProxy.deleteDatasource(id, function(oJson){
                    // Update the datasource table
                    view.getController().getDataset();
                }, null, onComplete);
            }
        }
    },

    onRemove: function(evt) {
        var controller = this;
        var confirmDialog =  sap.ui.jsfragment("sas.modelfactory.dialog.ConfirmDialog", this);
        confirmDialog.initDialog(sas.modelfactory.RB.getText("datasetView.confirmDelete.title.txt"),
            sas.modelfactory.RB.getText("datasetView.confirmDelete.txt"),
            controller.deleteDatasources,
            function(){
                // Nothing to do here
            }
        );
        confirmDialog.setInitialFocus(confirmDialog.getEndButton());
        confirmDialog.open();
    },

    onExplore: function(evt) {
        var index = this.oView.datasetTable.getSelectedIndex();
        alert("Explore index: " + index);
    },

    onCreateMetadata: function(dialog) {
        var controller = this;
        var origData = dialog.clonedArray;
        var dataset = dialog.table.getModel().oData;
        var metadata = controller.metadataModel.getProperty("/metaData");
        logUtil.log(origData);
        logUtil.log(dataset);

        for (var i=0;i<origData.variables.length;i++) {
            var existingEntry = false;
            var entry = null;
            var origRole = origData.variables[i].role?origData.variables[i].role.key:"";
            var newRole = dataset.variables[i].role?dataset.variables[i].role.key:"";
            if (origRole !== newRole) {
                logUtil.log("--------------> Change found: ", dataset.variables[i]);
                // Check if this variable already has a metadata entry
                if (metadata && metadata.length > 0){
                    for (var j=0;j<metadata.length;j++){
                        if (metadata[j].name === dataset.variables[i].name &&
                            metadata[j].type.key === dataset.variables[i].type.key){
                            // Found a matching metadata entry
                            existingEntry = true;
                            entry = metadata[j];
                            break;
                        }
                    }
                }

                if (existingEntry){
                    // Update an existing metadata entry
                    var newData = dataset.variables[i];
                    entry.role = newData.role;
                    controller.updateMetadata(entry.id, entry);
                } else {
                    // Create a new metadata entry
                    var enumModel = sap.ui.getCore().getModel("enums");
                    var levelMap = {};
                    var orderMap = {};
                    var transMap = {};
                    var imputeMap = {};
                    if (enumModel){
                        levelMap = enumModel.getProperty("/levels");
                        orderMap = enumModel.getProperty("/orders");
                        transMap = enumModel.getProperty("/transformationMethods");
                        imputeMap = enumModel.getProperty("/imputations");
                        if (levelMap === undefined || orderMap === undefined || transMap === undefined || imputeMap === undefined){
                            logUtil.error("Enumeration Error: enum map missing", levelMap, orderMap, transMap, imputeMap);
                            return;
                        }
                    }
                    var newData = dataset.variables[i];
                    if (newData !== undefined){
                        entry = {
                            "environmentId" : controller.environmentId,
                            "name" : newData.name,
                            "role" : newData.role,
                            "label" : newData.label,
                            "level" : levelMap["AUTO"],                    // Default values
                            "order" : orderMap["DEFAULT"],                 // Default values
                            "transformationMethod" : transMap["DEFAULT"],  // Default values
                            "imputationMethod" : imputeMap["DEFAULT"],     // Default values
                            "type" : newData.type
                        };
                        controller.createMetadata(entry);
                    }
                }
            }
        }
    },

    createMetadata: function(entry) {
        var controller = this;
        var view = this.getView();
        if (entry && controller.environmentId) {
            var onSuccess = function(oJson){

            };

            var onError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("datasetView.error.createMetadata.txt");
                var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            var onComplete = function(oJson){
                view.objectManager.setEditMode(false);
            };

            dataSourceMetaProxy.createMetadata(controller.environmentId, entry, onSuccess, onError, onComplete);
        }
    },

    updateMetadata: function(id, entry) {
        var controller = this;
        var view = this.getView();
        if (controller.environmentId && entry) {
            var onSuccess = function(oJson){
                view.objectManager.setEditMode(false);
            };

            var onError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("datasetView.error.updateMetadata.txt");
                var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            dataSourceMetaProxy.updateMetadata(controller.environmentId, id, entry, onSuccess, onError);
        }
    },

    // Begin: LOCAL FILE UPLOAD Controller Code
    //
    localFileUploadDialog : {},

    onUploadButtonPress: function(controller) {
        localFileUploadDialog = sap.ui.jsfragment("sas.modelfactory.dialog.UploadDialog", this);
        if (localFileUploadDialog) {
            localFileUploadDialog.open();
            localFileUploadDialog.setModel(
                new sap.ui.model.json.JSONModel({
                    types: []
                })
            );
        }

    },

    onUpload: function(evt, fileUploaderControl) {
        logUtil.log("  -- Uploading file to: " +  fileUploaderControl.getUploadUrl());
        logUtil.log("  -- Uploading file: " +  fileUploaderControl.getValue());
        //alert("Starting upload " + fileUploaderControl.getValue());
        fileUploaderControl.upload();
    },

    onUploadComplete: function(oControlEvent) {
        logUtil.log("  -- File Uploaded.");
        //alert("Upload complete. ");
        localFileUploadDialog.close();
    },

    getContentTypeHeaderName: function () {
        return "Content-Type";
    },

    // returns file content type for the upload
    getContentTypeValue: function() {
        var parameter = new sap.ui.commons.FileUploaderParameter ("fileUploadParameter",
            {
                name: "Content-type",
                x: 1000,
                value: 'application/sas.vnd.datamining.datasource.import.remote.options+json  \
                        { "dataset": "HMEQ",  "type": "SAS", "maximumRows": 10000, "maximumColumes": 5000, "skipRows": 0, "guessRows": 50 }'
            });
        return [parameter];
    },

    // End: LOCAL FILE UPLOAD Controller Code

    onSASMetaDataButtonPressed : function() {
        var controller = this;

        var onDatasourceSelected = function(data) {
            onSuccess = function(oJson){
                // Update the datasource table
                controller.getDataset();
            };

            var onError = function(xhr, err, msg) {
                var errMsg = sas.modelfactory.RB.getText("datasetView.error.label.txt");
                var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            onComplete = function(oJson) {
                busySpinner.show(false);
            };

            datasourceProxy.createDatasource(data, onSuccess, null, onComplete);
        };

        sas.modelfactory.util.ContentUtil.showDatasourceBrowser(onDatasourceSelected);
    },

    // Begin: onDatabase
    onDatabaseButtonPressed : function(controller) {
        alert("Database  pressed");
    }
});