jQuery.sap.require(
    "sas.modelfactory.proxies.DatasourceProxy",
    "sas.modelfactory.proxies.DataSourceMetaProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.util.LogUtil",
    "sas.hc.m.MessageBox",
    "sas.modelfactory.util.RootUtil"
);

sap.ui.controller("sas.modelfactory.main.MetadataTable", {

    datasourceModel: null,
    metadataModel: null,
    environmentId: null,
    fragInitData: null,
    dataLoaded: false,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.modelfactory.main.MetadataTable
     */
    onInit: function() {

        var bus = sas.eventBus;
        bus.subscribe("initialDataLoadEvent", "showView", this.refreshData, this);

        this.metadataModel = new sap.ui.model.json.JSONModel();
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.main.MetadataTable
     */
    onBeforeRendering: function() {

        logUtil.log("sas.modelfactory.main.MetadataTable onBeforeRendering called.");
    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.main.MetadataTable
     */
    onAfterRendering: function() {

    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.main.MetadataTable
     */
	onExit: function() {
        sas.eventBus.unsubscribe("initialDataLoadEvent", "showView", this.refreshData, this);
	},

    refreshData: function(channel, msg, data) {
        if (data == "METADATA_VIEW") {
            this.getDatasetMeta();
        }
    },

    getDatasetMeta: function() {
        var view = this.oView;
        var controller = this;
        var model = this.metadataModel;

        var onSuccess = function(oJson){
            // There should only be one environment.  Get the first one.
            var envirData = oJson.items[0];
            if (envirData){
                controller.environmentId = envirData.id;

                var handleDatasetMetaJson = function(oJson){
                    if (!controller.dataLoaded){
                        controller.dataLoaded = true;
                        model.setData({metaData: oJson.items});
                        view.setAndBindMetaModel(model , "/metaData");
                    } else {
                        // Update the data
                        view.metadataTable.getModel().setProperty("/metaData", oJson.items);
                    }
                };

                dataSourceMetaProxy.getMetadata(envirData.id, handleDatasetMetaJson);
            }
        }

        var rootModel = sap.ui.getCore().getModel("root");
        if (rootModel === undefined) {
            // Root links not loaded yet
            sas.modelfactory.util.RootUtil.getRootLinks(controller, controller.getDatasetMeta);
        } else {
            dataSourceMetaProxy.getEnvironment(onSuccess);
        }
    },


    /*handleMetadataRowSelectionChange: function(oEvent) {
        this.processControls();
    },

    processControls: function() {
        // Multiple variables selected
        var view = this.oView;
        var selected = view.metadataTable.getSelectedIndices();
        if (selected.length > 1) {
            view.editBtn.setEnabled(false);
            view.removeBtn.setEnabled(true);
        } else if (selected.length == 1) {
            view.editBtn.setEnabled(true);
            view.removeBtn.setEnabled(true);
        } else {
            view.editBtn.setEnabled(false);
            view.removeBtn.setEnabled(false);
        }
    },*/

    updateActions: function() {
        var view = this.oView;
        var selectedItems = this.getView().objectManager.getSelectedItems();

        if (selectedItems.length > 1) {
            view.editBtn.setEnabled(false);
            view.removeBtn.setEnabled(true);
        } else if (selectedItems.length == 1) {
            view.editBtn.setEnabled(true);
            view.removeBtn.setEnabled(true);
        } else {
            view.editBtn.setEnabled(false);
            view.removeBtn.setEnabled(false);
        }
    },

    deleteMetadata: function() {
        var view = this.oView;
        var controller = this;
        var items = this.getView().objectManager.getSelectedItems();
        for (var i=0;i<items.length; i++ ) {
            // Delete each metadata entry
            var entry = items[i];
            if (entry){
                controller.removeMetadata(entry);
            }
        }
    },

    onRemove: function(evt) {
        var controller = this;
        var confirmDialog =  sap.ui.jsfragment("sas.modelfactory.dialog.ConfirmDialog", this);
        confirmDialog.initDialog(sas.modelfactory.RB.getText("metadataView.confirmDelete.title.txt"),
            sas.modelfactory.RB.getText("metadataView.confirmDelete.txt"),
            controller.deleteMetadata,
            function(){
                // Nothing to do here
            }
        );
        confirmDialog.setInitialFocus(confirmDialog.getEndButton());
        confirmDialog.open();
    },

    removeMetadata: function(oData) {
        logUtil.log("removeMetadata: ", oData);
        var varId = oData.id;
        var view = this.oView;
        var controller = this;

        var onSuccess = function(oJson){
            // Update the table
            controller.getDatasetMeta();
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("datasetView.error.deleteMetadata.txt");
            var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            view.objectManager.setEditMode(false);
        };

        if (controller.environmentId && varId){
            dataSourceMetaProxy.deleteMetadata(controller.environmentId, varId, onSuccess, onError, onComplete)
        };
    },

    editPress: function(evt) {
        var items = this.getView().objectManager.getSelectedItems();
        var selectedData = items[0];
        if (selectedData) {
            logUtil.log(selectedData);
            this.fragInitData = {data: selectedData};

            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.EditMetadataDialog", this);
            dialog.open();
        }
    },

    onUpdateMetadata: function(id, name, type, label, transform, impute, role, level, order) {
        logUtil.log("onUpdateMetadata: ", id, name, type, label, transform, impute, role, level, order);
        var controller = this;
        var entry = this.fragInitData.data;
        entry.role = role;
        entry.level = level;
        entry.order = order;
        entry.transformationMethod = transform;
        entry.imputationMethod = impute;
        controller.updateMetadata(id, entry);
    },

    updateMetadata: function(id, entry) {
        var view = this.oView;
        var controller = this;
        if (controller.environmentId && entry) {
            var onSuccess = function(oJson){
                // Update the list
                controller.getDatasetMeta();
            };

            var onError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("datasetView.error.updateMetadata.txt");
                var title = sas.modelfactory.RB.getText("datasetView.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            var onComplete = function(oJson){
                view.objectManager.setEditMode(false);
            };

            dataSourceMetaProxy.updateMetadata(controller.environmentId, id, entry, onSuccess, onError, onComplete);
        }
    }
});