jQuery.sap.require(
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.proxies.TemplateProxy",
    "sas.modelfactory.util.LogUtil",
    "sas.modelfactory.util.RootUtil"
);

sap.ui.controller("sas.modelfactory.main.ModelTemplateTable", {

    view: null,
    dataLoaded: false,
    isNewModel: false,
    initialLoad: true,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.modelfactory.main.ModelTemplateTable
     */
	onInit: function() {

        this.view = this.oView;

        sas.eventBus.subscribe("initialDataLoadEvent", "showView", this.refreshData, this);
	},

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.main.ModelTemplateTable
     */
	onBeforeRendering: function() {

        logUtil.log("sas.modelfactory.main.ModelTemplateTable onBeforeRendering called.");
	},

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.main.ModelTemplateTable
     */
	onAfterRendering: function() {

	},

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.main.ModelTemplateTable
     */
	onExit: function() {
        sas.eventBus.unsubscribe("initialDataLoadEvent", "showView", this.refreshData, this);
	},

    refreshData: function(channel, msg, data)  {
        if (data == "MODEL_TEMPLATES_VIEW") {
            this.getTemplateList();
        }
    },

    updateActions: function() {
        var view = this.getView();
        var selectedItems = this.getView().objectManager.getSelectedItems();
        var sasonlySelected = false;

        for (var i=0; i < selectedItems.length; i++) {
            if (selectedItems[i].sasonly) {
                sasonlySelected = true;
                break;
            }
        }

        if (selectedItems.length > 1) {
            view.defaultBtn.setEnabled(this.isFavoriteTheSame(selectedItems, selectedItems[0].favorite) && !selectedItems[0].favorite );
            view.notDefaultBtn.setEnabled(this.isFavoriteTheSame(selectedItems, selectedItems[0].favorite) && selectedItems[0].favorite );;
            view.removeBtn.setEnabled(!sasonlySelected);
            view.renameBtn.setEnabled(false);
            view.openBtn.setEnabled(false);
        } else if (selectedItems.length == 1) {
            view.defaultBtn.setEnabled(!selectedItems[0].favorite);
            view.notDefaultBtn.setEnabled(selectedItems[0].favorite);
            view.removeBtn.setEnabled(!sasonlySelected);
            view.renameBtn.setEnabled( !sasonlySelected);
            view.openBtn.setEnabled(true);
        } else {
            view.defaultBtn.setEnabled(false);
            view.notDefaultBtn.setEnabled(false);
            view.removeBtn.setEnabled(false);
            view.renameBtn.setEnabled(false);
            view.openBtn.setEnabled(false);
        }
    },

    isFavoriteTheSame: function(items, value) {
        var allTheSame = true;
        for (var i=0;i<items.length;i++){
            if (items[i].favorite !== value){
                allTheSame  = false;
                break;
            }
        }
        return allTheSame ;
    },

    onOpenButtonPress: function(evt) {
        var controller = this;
        var view = this.getView();
        var items = view.objectManager.getSelectedItems();

        $.each(items, function(index, item) {
            controller.onEditTemplate(item.id);
        });

        view.objectManager.setEditMode(false);
    },

    openItem: function(evt) {
        var controller = this;
        var item = evt.getParameter("item");
        if (item){
            controller.onEditTemplate(item.id);
        }
    },

    onNewPress: function(selectedObj) {
        var controller = this;
        controller.isNewModel = true;
        var onSuccess = function (oJson) {
            controller.newModelDialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewModelTemplateDialog", controller);

            // fetch the blank template- The algorithms are the in the properties where groupName = "Model"
            // compile a unique list
            var templates = oJson.items;

            var onBlankTemplateSuccess = function (oJson) {
                var algorithms = sap.ui.getCore().getModel("algorithms").getData().algorithmsForDisplay;
                controller.blankModel = oJson;

                var model = new sap.ui.model.json.JSONModel();
                model.setData({algorithms: algorithms});
                controller.newModelDialog.setModel(model);
                controller.newModelDialog.open();
                sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: oJson, saveEnabled: true});
            };

            var onBlankTemplateError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.loading.desc.txt");
                var title = sas.modelfactory.RB.getText("Loading template");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            for (var i=0; i<templates.length; i++) {
                logUtil.log("name: = " + templates[i].name);
                if (templates[i].name === "Blank Template") {
                    var links = templates[i].links;
                    messageProxy.request(links, "self", "GET", "json", null, onBlankTemplateSuccess, onBlankTemplateError);
                    break;
                }
            }
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.loading.desc.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.loading.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function () {
            sas.eventBus.unsubscribe("ModelTemplateEvent", "dataLoaded", this.modelDataLoadedHandler, this);
        };

        templateProxy.getTemplates(onSuccess, onError, onComplete);
    },

    onNewModelTemplateCreate: function(name, dialog ) {
        var selectedItem = dialog.modelAlgorithm.getSelectedItem();
        logUtil.log("onNewModelTemplateCreate: ", this);
        logUtil.log("onNewModelTemplateCreate: ", name.getValue());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getText());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getKey());

        var controller = this;

        // Find the selected algorithm index.
        var algorithms = selectedItem.getModel().getData().algorithms;
        var algorithmIndex = -1;
        for (var i=0; i< algorithms.length; i++){
            if (algorithms[i].id === selectedItem.getKey()){
                algorithmIndex = i;
                break;
            }
        }

        // initialize the blankModel and "use" the respective model properties component
        var newModel = controller.blankModel;
        newModel.name = name.getValue();
        newModel.displayName = name.getValue();
        newModel.createdBy = null;
        newModel.modifiedBy = null;
        newModel.createdTimestamp = null;
        newModel.modifiedTimestamp = null;
        newModel.links = null;
        newModel.hidden = false;
        newModel.sasonly = false;
        // newModel.position = 1;
        newModel.description = "User created model";
//        newModel.algorithmName = algorithms[algorithmIndex].displayName;
        newModel.miningAlgorithm = {id: algorithms[algorithmIndex].id, internalName: algorithms[algorithmIndex].internalName};
        newModel.favorite = dialog.defaultCheckBox.getSelected();
        for (var i=0; i<newModel.properties.length; i++) {
            if (newModel.properties[i].groupName === 'Model') {
                if (newModel.properties[i].componentName === algorithms[algorithmIndex].internalName) {
                    if (newModel.properties[i].propertyName === '_use') {
                        newModel.properties[i].propertyValue = 'Y';
                        newModel.properties[i].position = 1;
                    }
                }
            }
        }

        // open the model dialog
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", this);
        var title =  name.getValue();
        dialog.setTitle(title);
        dialog.addSaveButton();
        dialog.open();
        sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: newModel, saveEnabled: true});

    },

    onEditTemplate: function(id) {
        // Find the selected template
        this.isNewModel = false;
        var list = this.oView.table.getModel().getProperty("/templateData");
        if (list) {
            for (var i=0; i < list.length; i++) {
                if (list[i].id === id) {
                    if (list[i].sasonly) {
                        this.isNewModel = true;
                    }
                    this.getTemplate(list[i].links);
                    break;
                }
            }
        }
    },


    onTemplateSave: function(dialog) {
        var controller = this;
        var model = dialog.getModel().oData.template;
        // logUtil.log("Save model template:", JSON.stringify(model.oData.template));

        var onSuccess = function (oJson)
        {
            // Templated updated, refresh the model template list.
            controller.getTemplateList();
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.updateModelTemplate.txt") +
                " " + xhr.responseJSON.message;
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        // Update the model template
        if (controller.isNewModel) {
                templateProxy.createGlobalModelTemplate( model, onSuccess, onError);
        } else {
            templateProxy.updateTemplate(model.id, model, onSuccess, onError);
        }
    },

    onTemplateSaveAs: function(dialog) {
        var controller = this;
        var model = dialog.getModel().oData.template;

        // Update the model template
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
        dialog.renameHandler = this.saveAsTemplateRenameHandler;

        var dialogModel = new sap.ui.model.json.JSONModel();
        dialogModel.setData({template: model,
            templateData: controller.oView.table.getModel().oData.templateData});
        dialog.bindElement("/template");
        dialog.uniqueNamePath = "/templateData";
        dialog.setNameProperty("displayName");

        dialog.setModel(dialogModel);
        dialog.open();
    },

    saveAsTemplateRenameHandler: function(dialog) {
        var controller = this;
        var model = dialog.getModel();
        model.oData.template.name = model.oData.template.displayName;

        controller.oView.setBusy(true);

        var onSuccess = function() {
            controller.getTemplateList(controller.project);
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.saveAsModelTemplate.txt") +
                " " + xhr.responseJSON.message;
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            controller.view.setBusy(false);
        };

        templateProxy.createGlobalModelTemplate( model.oData.template, onSuccess, onError, onComplete);
    },

    getTemplate: function(links) {
        var controller = this;
        var view = this.oView;

        var onSuccess = function(oJson){
            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", controller);
            var title = sas.modelfactory.RB.getText(oJson.displayName);
            dialog.setTitle(title);
            if (!oJson.sasonly) {
                dialog.addSaveButton();
            }
            dialog.addSaveAsButton();
            dialog.open();
            // publish to event bus
            sas.eventBus.publish("ModelTemplateEvent", "editTemplate", {
                template: oJson});
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("templateView.errorMsg.getTemplate.txt");
            var title = sas.modelfactory.RB.getText("templateView.errorMsg.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        //templateProxy.getTemplate(templateId, onSuccess, onError);
        // Get the global template
        if (links) {
            messageProxy.request(links, "self", "GET", "json", null, onSuccess, onError);
        }
    },

    getTemplateList: function() {
        var controller = this;
        var view = this.oView;
        var model = new sap.ui.model.json.JSONModel();

        var onSuccess = function(oJson){
            logUtil.log("Model Templates:", oJson);
            // This code removes hidden templates such as the blank template
            var i = 0;
            while ( i<oJson.items.length) {
                if (oJson.items[i].hidden) {
                    oJson.items.splice(i,1);
                } else {
                    i++;
                }
            }
            if (!controller.dataLoaded){
                controller.dataLoaded = true;
                model.setData({templateData: oJson.items});
                view.setAndBindModel(model, "/templateData");
            } else {
                // Update the data
                view.table.getModel().setProperty("/templateData", oJson.items);
            }

            if (controller.initialLoad) {
                controller.initialLoad = false;

                // Apply initial sorting
                /*var table = view.byId("table");
                var oBinding = table.getBinding("items");
                var aSorters = [];
                aSorters.push(new sap.ui.model.Sorter("name", false));
                oBinding.sort(aSorters);*/
            }
        };

        //templateProxy.getTemplates(onSuccess);
        var rootModel = sap.ui.getCore().getModel("root");
        if (rootModel === undefined) {
            // Root links not loaded yet
            sas.modelfactory.util.RootUtil.getRootLinks(controller, controller.getTemplateList);
        } else {
            var links = rootModel.getProperty("/root");

            // Get the global template list
            messageProxy.request(links, "templates", "GET", "json", null, onSuccess);
        }
    },

    // Action only supported for single select.
    onMarkAsDefault: function(evt) {

        logUtil.log ("onMarkAsDefault Click");
        var view = this.getView();
        var controller = this;
        var errorCount = 0;  // kindly only display 3 alerts and then a 4th final alert
        var selectedItems = this.getView().objectManager.getSelectedItems();
        var templateCount = selectedItems.length;
        var workCompletedCount = 0;


        var onGetSuccess = function(oJson){
             // Toggle the favorite
            var onPutSuccess = function() {
                controller.initialLoad = true;
                controller.getTemplateList();
            };
            var onPutError = function(xhr, err, msg){
                errorCount++;
                var errMsg;
                var title;
                if (errorCount <= 3) {
                    errMsg = sas.modelfactory.RB.getText("viewModeltemplate.error.default.txt") +
                        " " + xhr.responseJSON.message;
                    title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                    sas.hc.m.MessageBox.error(errMsg, title);
                } else if (errorCount === 4) {
                    errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.tooManyErrors.txt") +
                        " " + xhr.responseJSON.message;
                    title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                    sas.hc.m.MessageBox.error(errMsg, title);
                }
            };
            var onPutComplete = function () {
                workCompletedCount++;
                if (workCompletedCount === templateCount) {
                    view.objectManager.setEditMode(false);
                    controller.getTemplateList();
                }
            };

            var selectedData = oJson;
            selectedData.favorite = !(selectedData.favorite);
            templateProxy.updateTemplate(selectedData.id, selectedData, onPutSuccess, onPutError, onPutComplete);
         };

         var onGetError = function () {
             view.objectManager.setEditMode(false);
         };



        if (selectedData && selectedData.links) {
            var links = selectedData.links;
            messageProxy.request(links, "self", "GET", "json", null, onGetSuccess, onGetError);
        }


        while (selectedItems.length> 0) {
            // Get the first item in the selection and then remove it from the selection.
            var selectedData = selectedItems[0];
            if (selectedData && selectedData.links) {
                var links = selectedData.links;
                messageProxy.request(links, "self", "GET", "json", null, onGetSuccess, onGetError);
            }
            selectedItems.splice(0,1);
        }
    },

    deleteTemplates: function(){
        var controller = this;
        var view = this.getView();
        var errorCount = 0;  // kindly only display 3 alerts and then a 4th final alert
        var selectedItems = this.getView().objectManager.getSelectedItems();
        var onSuccess = function(oJson){
            // nothing to do if successfull
        };
        var onError = function(xhr, err, msg){
            errorCount++;
            var errMsg;
            var title;
            if (errorCount <= 3) {
                errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.deleteModelTemplate.txt") +
                    " " + xhr.responseJSON.message;
                title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            } else if (errorCount === 4) {
                errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.tooManyErrors.txt") +
                    " " + xhr.responseJSON.message;
                title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            }
        };

        var onComplete = function () {
            if (selectedItems.length === 0) {
                view.objectManager.setEditMode(false);
                controller.getTemplateList();
            }
        };

        while (selectedItems.length> 0) {
            // Delete the first item in the selection and then remove it from the selection.
            templateProxy.deleteGlobalModelTemplate(selectedItems[0].id, onSuccess, onError, onComplete);
            selectedItems.splice(0,1);
        }
    },

    onRemove: function(evt) {
        logUtil.log ("onRemove Click");
        var controller = this;
        var confirmDialog =  sap.ui.jsfragment("sas.modelfactory.dialog.ConfirmDialog", this);
        confirmDialog.initDialog(sas.modelfactory.RB.getText("templateView.confirmDelete.title.txt"),
            sas.modelfactory.RB.getText("templateView.confirmDelete.txt"),
            controller.deleteTemplates,
            function(){
                // Nothing to do here
            }
        );
        confirmDialog.setInitialFocus(confirmDialog.getEndButton());
        confirmDialog.open();
    },

    onRename: function(evt) {
        logUtil.log ("onRename Click");
        var item = this.getView().objectManager.getSelectedItems()[0];
        if (item) {
            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
            dialog.renameHandler = this.renameHandler;
            var model = new sap.ui.model.json.JSONModel();
            model.setData({template: item,
                templateList: this.oView.table.getModel().oData.templateData});
            dialog.bindElement("/template");
            dialog.uniqueNamePath = "/templateList";
            dialog.setNameProperty("displayName");
            dialog.setModel(model);
            dialog.open();
        }
    },

    renameHandler: function(dialog) {
        var controller = this;
        var templateRef = dialog.getModel().oData.template;



        var onSuccess = function (oJson) {
            var onUpdateSuccess = function () {

            };
            var onUpdateError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.updateModelTemplate.txt") +
                    " " + xhr.responseJSON.message;
                var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };
            var onComplete = function(oJson){
                controller.getTemplateList();
                controller.getView().objectManager.setEditMode(false);
            }
            // reset the name and update
            var modelTemplate = oJson;
            modelTemplate.displayName = templateRef.displayName;
            modelTemplate.name = templateRef.displayName;

            templateProxy.updateTemplate(modelTemplate.id, modelTemplate, onUpdateSuccess, onUpdateError, onComplete);
        };
        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("templateView.errorMsg.getTemplate.txt") +
                " " + xhr.responseJSON.message;
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
            controller.getView().objectManager.setEditMode(false);
        };

        var links = templateRef.links;
        messageProxy.request(links, "self", "GET", "json", null, onSuccess, onError, null);

    }


});