jQuery.sap.require(
    "sap.m.MessageToast",
    "sas.modelfactory.proxies.ModelProxy",
    "sas.modelfactory.proxies.SegmentsProxy",
    "sas.modelfactory.proxies.JobProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.report.BaseGraphController",
    "sas.modelfactory.util.PreferencesUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sas.modelfactory.report.BaseGraphController.extend("sas.modelfactory.project.results.ProjectResults", {

    loaded: false,
    exitProject: false,
    stopPolling: false,
    isPolling: false,
    oneLastPollReq: false,  // Work-around for mid-tier problem
    pollingTime: 0,
    pollingCount: 0,
    CONST_POLL_TIME: 5000,     // 5 secs.
    CONST_POLL_TIME_1: 10000,  // 10 secs.
    CONST_POLL_TIME_2: 20000,  // 20 secs.
    CONST_POLL_TIME_3: 60000,  // 60 secs.
    CONST_POLL_TIME_4: 300000, // 5 mins.
    CONST_POLL_TIME_5: 900000, // 15 mins.
    CONST_BACKOFF_1: 24,       // Back-off to CONST_POLL_TIME_1 at 2 mins.
    CONST_BACKOFF_2: 42,       // Back-off to CONST_POLL_TIME_2 at 5 mins.
    CONST_BACKOFF_3: 57,       // Back-off to CONST_POLL_TIME_3 at 10 mins.
    CONST_BACKOFF_4: 77,       // Back-off to CONST_POLL_TIME_4 at 30 mins.
    CONST_BACKOFF_5: 82,       // Back-off to CONST_POLL_TIME_5 at 60 mins.
    project: null,
    profileBuilt: false,
    graphsLoaded: false,
    segSumCount: -1,
    championOverrideChanged: false,
    reportFragData: null,
    scoreCodeFragData: null,
    jobCount: 0,
    errorMsg: null,
    preferenceId: null,

    onInit: function(){
        // Call the super
        sas.modelfactory.report.BaseGraphController.prototype.onInit.apply(this);

        sas.eventBus.subscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.subscribe("ProjectEvent", "projectRetrain", this.handleProjectRetrainEvent, this);
        sas.eventBus.subscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);
        sas.eventBus.subscribe("ProjectEvent", "reloadResults", this.handleProjectReloadEvent, this);

        this.pollingTime = this.CONST_POLL_TIME;
        this.pollingCount = 0;

        var controller = this;

        this.getView().addEventDelegate({
            onBeforeFirstShow: function() {
                controller.noResultsHandler(controller.getView());
            },

			onBeforeShow: function() {
                logUtil.log("ProjectResults onBeforeShow hit");
                controller.stopPolling = false;
                controller.initializeView();
            },

            onBeforeHide: function() {
                logUtil.log("ProjectResults onBeforeHide hit");
                controller.stopPolling = true;
            }
        });

        this.getView().table.attachAfterColumnsDialogClose(function(oEvent) {
            // If OK was pressed, persist the column manager changes
            if (oEvent.getParameter("acceptedChange"))
                controller.onColumnManagerClose();
        });
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.project.results.ProjectResults
     */
    onBeforeRendering: function ()
    {

    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.project.settings.ProjectRules
     */
    onAfterRendering: function() {
        var controller = this;

    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.project.Project
     */
    onExit: function() {
        logUtil.log("sas.modelfactory.project.results.ProjectResults onExit called");

        this.exitProject = true;

        // Stop any polling
        this.stopPolling = true;

        sas.eventBus.unsubscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.unsubscribe("ChampionOverrideChanged", this.project.id, this.onChampionOverrideHandler, this);
        sas.eventBus.unsubscribe("ProjectEvent", "projectRetrain", this.handleProjectRetrainEvent, this);
        sas.eventBus.unsubscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);
        sas.eventBus.unsubscribe("ProjectEvent", "reloadResults", this.handleProjectReloadEvent, this);

        // Destroy the refreshable graphs from the segment summary graph list
        if (this.getView().graphGrid) {
            var tiles = this.getView().graphGrid.getContent();
            if (tiles.length > 0) {
                for (var i = 0; i < tiles.length; i++) {
                    var graph = tiles[i].getImage();
                    if (graph && graph.destroyRefreshable)
                        graph.destroyRefreshable();
                }
            }
        }

        sap.ui.getCore().setModel(null, "summaryGraphs");
    },

    updateBuildProfileState: function(sChannelId, sEventId, oData) {
        var controller = this;
        if (oData){
            controller.buildProfileAvailable = oData.buildProfileAvailable;
            controller.profileBuilt = oData.profileBuilt;

            // If the profile is not built but are segments are loaded, the profile
            // has been updated. Time to reload.
            if (!controller.profileBuilt && controller.loaded){
                controller.handleProfileBuiltEvent();
            }
        }
    },

    handleProfileBuiltEvent : function(channelId, eventId, data) {
        var view = this.getView();
        var controller = this;

        // Clear any existing columns
        view.table.removeAllColumns();

        // Clear column prefs
        if (controller.project){
            var key = "FactoryMiner.ResultsColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";
            sas.modelfactory.util.PreferencesUtil.deleteColumnPrefs(controller.project.id, key);
        }

        // Need to reload the profile table
        controller.loaded = false;
        view.table.getModel(null);
        view.table.unbindRows();
    },

    handleProjectLoadedEvent: function(channelId, eventId, data) {
        var controller = this;
        if (data){
            controller.buildProfileAvailable = data.buildProfileAvailable;
            controller.profileBuilt = data.profileBuilt;

            controller.initializeView();
        }
    },

    handleProjectReloadEvent : function(channelId, eventId, data) {
        this.initializeView();
    },

    handleProjectRetrainEvent : function(channelId, eventId, data) {
        var controller = this;
        var view = controller.getView();

        // Remove the segment summary reports
        var reports = view.graphGrid.getContent();
        if (reports){
            view.graphGrid.removeAllContent();
            view.getModel().setProperty("/reports", null);
        }
    },

    initializeView : function() {
        var controller = this;
        var view = controller.getView();

        // Get the current open project
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
            controller.buildProfileAvailable = model.getProperty("/buildProfileAvailable");
            controller.profileBuilt = model.getProperty("/profileBuilt");
        }

        if (project) {
            controller.project = project;

            // If the profile has been built, get the segments for the project
            if (controller.project && controller.profileBuilt){
                controller.getSegments(controller.project);
            }
            // Otherwise, present the no results graphic.
            else {
                controller.noResultsHandler(view);
            }

            // Mark the graphs as not loaded
            controller.graphsLoaded = false;

            // Register for Champion Changes the current open project
            sas.eventBus.subscribe("ChampionOverrideChanged", controller.project.id, controller.onChampionOverrideHandler, controller);

            controller.updateMessages();
        }
    },

    projectQueryHandler: function(channelId, eventId, oData) {
        oData.projectQuery(this.project);
    },

    onChampionOverrideHandler: function (evt) {
        this.championOverrideChanged=true;
        this.getSegments(this.project);
    },

    onOpenPress: function(evt) {
        var indices = this.oView.table.getSelectedIndices();
        this.onDrillIn(indices[0]);
    },

    onRowSelectionClick: function (evt)
    {
        var controller = this;
        var indices = this.oView.table.getSelectedIndices();
        logUtil.log("rows selected: " + indices);

        var runningStatus = function(selection) {
            for(var i=0;i<selection.length;i++){
                var context = controller.getView().table.getContextByIndex(selection[i]);
                if (context){
                    var item = context.getObject();
                    if (item.statusCounts.RUNNING > 0 || item.statusCounts.SCHEDULED > 0){
                        return true;
                    }
                }
            }
            return false;
        };

        var  needsRunning = function(selection) {
            for(var i=0;i<selection.length;i++){
                var context = controller.getView().table.getContextByIndex(selection[i]);
                if (context){
                    var item = context.getObject();
                    if (item.statusCounts.RUN_NEEDED > 0 || item.statusCounts.CANCELED > 0 || item.statusCounts.ERROR > 0 || item.statusCounts.FAILED > 0){
                        return true;
                    }
                }
            }
            return false;
        };

        // Enable/disable the actions button
        if (indices.length == 0) {
            this.oView.actionButton.setEnabled(false);
            this.byId("openBtn").setEnabled(false);
            this.oView.openSegBtn.setEnabled(false);
            this.byId("runBtn").setEnabled(false);
            this.byId("stopBtn").setEnabled(false);
            this.byId("historyBtn").setEnabled(false);
            this.byId("reportBtn").setEnabled(false);
            this.byId("scoreCodeBtn").setEnabled(false);
//            this.byId("retrainBtn").setEnabled(false);
            //this.byId("exploreBtn").setEnabled(false);
        } else if (indices.length == 1) {
            this.oView.actionButton.setEnabled(true);
            this.byId("openBtn").setEnabled(true);
            this.oView.openSegBtn.setEnabled(true);
            this.byId("runBtn").setEnabled(needsRunning(indices));
            this.byId("stopBtn").setEnabled(runningStatus(indices));
            this.byId("historyBtn").setEnabled(true);
            this.byId("reportBtn").setEnabled(true);
            this.byId("scoreCodeBtn").setEnabled(true);
            //this.byId("retrainBtn").setEnabled(true);
            //this.byId("exploreBtn").setEnabled(true);
        } else if (indices.length > 1) {
            this.oView.actionButton.setEnabled(true);
            this.byId("openBtn").setEnabled(false);
            this.oView.openSegBtn.setEnabled(false);
            this.byId("runBtn").setEnabled(needsRunning(indices));
            this.byId("stopBtn").setEnabled(runningStatus(indices));
            this.byId("historyBtn").setEnabled(false);
            this.byId("reportBtn").setEnabled(true);
            this.byId("scoreCodeBtn").setEnabled(true);
            //this.byId("retrainBtn").setEnabled(false);
            //this.byId("exploreBtn").setEnabled(false);
        }
    },

    onDrillIn: function(index) {
        var controller = this;
        var context = this.oView.table.getContextByIndex(index);
        if (context)
        {
            var selectedData = context.getObject();
            var objectContentPage = sap.ui.getCore().byId(this.getView().data("objectContentPageId"));
            var segmentId = selectedData.segmentId;
            var segmentName = sas.modelfactory.RB.getText("projectResults.column.segment.txt") + " " + segmentId;

            // If we located the segment
            if (segmentId !== null) {
                var drillItem = {
                    id: jQuery.sap.uid(),
                    type: "drill",
                    name: segmentName,
                    level: 1,
                    projId: controller.project.id,
                    segId: selectedData.id,
                    depth: controller.project.depth
                };
                objectContentPage.drillIn(drillItem, segmentName);
            }
        }
    },

    onDrillInLinkPress: function(link) {
        var controller = this;
        var objectContentPage = sap.ui.getCore().byId(this.getView().data("objectContentPageId"));

        var view = this.getView();
        var segmentId = null;
        if (link.data()){
            segmentId = link.data().id;
        }
        var segmentName = link.getText();

        // If we located the segment
        if (segmentId !== null) {
            var drillItem = {
                id: jQuery.sap.uid(),
                type: "drill",
                name: segmentName,
                level: 1,
                projId: this.project.id,
                segId: segmentId,
                depth: this.project.depth
            };
            objectContentPage.drillIn(drillItem, segmentName);
        }
    },

    onResultAction: function (evt)
    {
        var controller = this;
        var view = this.getView();
        var id = evt.getParameter("id");
        logUtil.log("onResultAction menuItem: ", id);
        var menuItem = id.substring(id.lastIndexOf("--")+2, id.length);

        var tempStr = "";
        var action;
        var indices = this.oView.table.getSelectedIndices();

        switch (menuItem)
        {
            case "openBtn":
                // Open only the first item
                controller.onDrillIn(indices[0]);
                break;
            case "runBtn":
                var count = indices.length;
                var skipCount = 0;
                busySpinner.show(true,
                    sas.modelfactory.RB.getText("projectView.busyDialog.title.txt"),
                    sas.modelfactory.RB.getText("projectView.busyDialog.runDescription.txt")
                );

                for (var i=0; i<indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                    {
                        var selectedData = context.getObject();

                        var onRunSuccess = function(oJson){
                            // Have jobs started?
                            var jobs = oJson.jobs;
                            logUtil.log("ProjectResults onRunSuccess:", jobs);
                            if (jobs && jobs.length === 0){
                                // Segment has no jobs to run.
                                skipCount++;
                            }
                        };

                        var onRunError = function(xhr, err, msg){
                            var errMsg = sas.modelfactory.RB.getText("projectResults.error.runSegment.txt");
                            var title = sas.modelfactory.RB.getText("projectResults.error.title.txt");
                            sas.hc.m.MessageBox.error(errMsg, title);
                        };

                        var onRunComplete = function(oJson){
                            count--;
                            if (count === 0 && controller.project){
                                busySpinner.show(false);
                                setTimeout(function(){sas.eventBus.publish("ProjectEvent", "running", {});}, 2000);

                                if (controller.project && controller.profileBuilt){
                                    controller.getSegments(controller.project);
                                }
                            }
                        };

                        // Run the specific model
                        segmentsProxy.runSegmentModels(controller.project.id, selectedData.id, onRunSuccess, onRunError, onRunComplete);
                    }
                }

                break;
            case "stopBtn":
                var count = indices.length;
                busySpinner.show(true,
                    sas.modelfactory.RB.getText("projectView.busyDialog.cancel.title.txt"),
                    sas.modelfactory.RB.getText("projectView.busyMsg.cancelJobs.txt")
                );
                for (var i=0; i<indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    var selectedData = context.getObject();
                    if (context)
                    {
                        var onCancelSuccess = function(oJson){

                        };

                        var onCancelError = function(xhr, err, msg){
                            var errMsg = sas.modelfactory.RB.getText("projectResults.error.stopRun.txt");
                            var title = sas.modelfactory.RB.getText("projectResults.error.title.txt");
                            sas.hc.m.MessageBox.error(errMsg, title);
                        };

                        var onCancelComplete = function(oJson){
                            count--;
                            if (count === 0 && controller.project){
                                busySpinner.show(false);

                                if (controller.project && controller.profileBuilt){
                                    controller.getSegments(controller.project);
                                }
                            }
                        };

                        // Cancel the job for the segment
                        jobProxy.cancelJob(controller.project.id, controller.project.jobTicketId, selectedData.id, null, onCancelSuccess, onCancelError, onCancelComplete);
                    }
                }
                break;
            case "historyBtn":
                // Build a list of selected segment ids
                var segIds = [];
                for (var i=0; i < indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                        segIds.push(context.getObject().id);
                }
                if (segIds.length  > 0) {
                    // Present the history dialog
                    var onSuccess = function(oJson) {
                        // Create the distribution report
                        jQuery.sap.require("sas.modelfactory.dialog.ViewHistoryDialog");

                        var dialog = new sas.modelfactory.dialog.ViewHistoryDialog({
                            title: sas.modelfactory.RB.getText("projectHistory.title.txt"),
                            content: sap.ui.view({
                                type: sap.ui.core.mvc.ViewType.JS,
                                viewName: "sas.modelfactory.project.settings.ProjectHistory",
                                viewData: {items: oJson.items, segmentId: segIds[0]}
                            }),
                            contentHeight: "90%"
                        });

                        dialog.open();
                    };

                    var onComplete = function() {
                        busySpinner.show(false);
                    }

                    // Get the history data
                    segmentsProxy.getHistoryTableForSegment(controller.project.id, segIds[0], onSuccess, null, onComplete);
                }
                break;
            case "scoreCodeBtn":
                // Build a list of selected segment ids
                var segIds = [];
                for (var i=0; i < indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                        segIds.push(context.getObject().id);
                }
                logUtil.log("score code for ids:", segIds);
                if (segIds.length  > 0){
                    // Prepare the data for the score code dialog
                    controller.scoreCodeFragData = {
                        projectId: controller.project.id,
                        segments: segIds,
                        models: null
                    };

                    // Open the score code dialog
                    var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.DownloadScoreDialog", this);
                    dialog.open();
                }
                break;
            case "reportBtn":
                // Build a list of selected segment ids
                var segIds = [];
                for (var i=0; i < indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                        segIds.push(context.getObject().id);
                }
                logUtil.log("report for ids:", segIds);
                if (segIds.length  > 0){
                    // Prepare the data for a report
                    controller.reportFragData = {
                        projectId: controller.project.id,
                        segmentId: null,
                        reportType: "segment",
                        items: segIds
                    };

                    // Open the report dialog
                    var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ReportDialog", this);
                    dialog.open();
                }
                break;
        }

        if (action && action.length > 0)
            sap.m.MessageToast.show(action + " for " + indices);
    },

    getSegments: function (project)
    {
        var view = this.oView;
        var controller = this;

        var onComplete = function(oJson){
            if (!controller.exitProject){
                if (!controller.loaded){
                    busySpinner.show(false);
                }

                // Update the status filters
                view.getStatusData();

                // Update the controls
                controller.onRowSelectionClick(null);
                // Clean XHR object up
                /*if( $req != null ){
                    $req.onreadystatechange = $req.abort = noop;
                    delete $req;
                    $req = null;
                    options = null;
                }*/
            }
        };

        var onSuccess = function (oJson)
        {
            if (!controller.exitProject){
                if (!controller.loaded){
                    controller.loaded = true;
                    busySpinner.show(false);
                    var model = new sap.ui.model.json.JSONModel();
                    model.setData({segments: oJson.items});
                    view.setAndBindModel(model, "/segments");

                    // Update table with column preferences
                    var key = "FactoryMiner.ResultsColumnPreferences.";
                    if (controller.getTargetType() === "INTERVAL")
                        key = key + "Numeric";
                    else
                        key = key + "Class";

                    var callback = function(preferenceId) {
                        controller.preferenceId = preferenceId;
                    }

                    sas.modelfactory.util.PreferencesUtil.getColumnPref(controller.project.id, key, view.table, callback);
                } else {
                    // Update the data
                    view.table.getModel().setProperty("/segments", oJson.items);
                }

                // Adjust table length
                var tableLen = 18/*oJson.items.length*/;
                var curLength = view.table.getVisibleRowCount();
                if (curLength !== tableLen){
                    view.table.setVisibleRowCount(tableLen);
                    view.invalidate();
                }

                // Determine the statistic column and update if necessary
                var curLabel = null;
                if (view.statCol) {
                    curLabel = view.statCol.getLabel();
                }
                if (curLabel && oJson && oJson.items.length > 0){
                    var transStatCol = sas.modelfactory.RB.getText(oJson.items[0].championStatisticName);
                    var statColText = curLabel.getText();
                    if (statColText !== transStatCol){
                        // Update the label
                        view.statCol.setLabel(new sas.hc.m.Text({textAlign: "End", wrapping: true, text: transStatCol}));

                        // update the filter label:
                        view.championPanel.setTitle(new sas.hc.ui.core.Title({text: transStatCol}));
                    }
                }

                // If not already started, poll segment status
                if (!controller.isPolling) {
                    controller.pollSegmentStatus(project);
                }

                // Build results summary reports
                if (controller.project && controller.profileBuilt){
                    // Get the summary reports for the segments
                    controller.getSegmentSummaryReports(controller.project);
                }
                tableLen = null;
                transStatCol = null;
                curLabel = null;
            }
        };

        segmentsProxy.getSegments(project.id, onSuccess, null, onComplete);

        if (!controller.loaded){
            busySpinner.show(true,
                sas.modelfactory.RB.getText("projectResults.busyDialog.title.txt"),
                sas.modelfactory.RB.getText("projectResults.busyDialog.loadingSegments.txt")
            );
        }
    },

    pollSegmentStatus: function(project) {
        var view = this.oView;
        var controller = this;
        var segList = view.table.getModel().getProperty("/segments");
        var isRunning = false;
        for(var i=0;i<segList.length;i++){
            if (segList[i].statusCounts.RUNNING > 0 || segList[i].statusCounts.SCHEDULED > 0){
                isRunning = true;
                break;
            }
        }

        // If any segments are RUNNING, poll until finished
        if (isRunning && !controller.stopPolling){
            view.runningIcon.addStyleClass("status-running");
            // Get the models for the selected segment
            logUtil.log("---------- POLLING SEGMENTS", Date(), "----------");
            controller.isPolling = true;
            controller.oneLastPollReq = true;
            if (project){
                controller.pollingCount++;
                if (controller.pollingCount <= controller.CONST_BACKOFF_1){
                    controller.pollingTime = controller.CONST_POLL_TIME;
                } else if ((controller.pollingCount > controller.CONST_BACKOFF_1) &&
                    (controller.pollingCount <= controller.CONST_BACKOFF_2)){
                    controller.pollingTime = controller.CONST_POLL_TIME_1;
                } else if ((controller.pollingCount > controller.CONST_BACKOFF_2) &&
                    (controller.pollingCount <= controller.CONST_BACKOFF_3)){
                    controller.pollingTime = controller.CONST_POLL_TIME_2;
                } else if ((controller.pollingCount > controller.CONST_BACKOFF_3) &&
                    (controller.pollingCount <= controller.CONST_BACKOFF_4)){
                    controller.pollingTime = controller.CONST_POLL_TIME_3;
                } else if ((controller.pollingCount > controller.CONST_BACKOFF_4) &&
                    (controller.pollingCount <= controller.CONST_BACKOFF_5)){
                    controller.pollingTime = controller.CONST_POLL_TIME_4;
                } else if (controller.pollingCount > controller.CONST_BACKOFF_5){
                    controller.pollingTime = controller.CONST_POLL_TIME_5;
                }
                logUtil.log("           POLLING TIME -->", controller.pollingTime);

                setTimeout(function(){
                    controller.isPolling = false;
                    if (!controller.stopPolling){
                        // For memory debugging on Chrome
                        /*console.log("jsHeapSizeLimit:", performance.memory.jsHeapSizeLimit,
                            "totalJSHeapSize:", performance.memory.totalJSHeapSize,
                            "usedJSHeapSize:", performance.memory.usedJSHeapSize);*/
                        controller.getSegments(project);
                    }
                }, controller.pollingTime);
            }
        } else {
            // Work-around until mid-tier fixes problem of reporting run complete but
            // not having finished updating the champions.
            if (controller.oneLastPollReq){
                controller.oneLastPollReq = false;
                setTimeout(function(){
                    logUtil.log("********* Run Monitor: Polling one last time ------------------------", Date());
                    // Force a reload of the graphs
                    controller.graphsLoaded = false;
                    controller.getSegments(project);
                }, controller.pollingTime);
            }
            controller.isPolling = false;
            controller.pollingTime = controller.CONST_POLL_TIME;
            controller.pollingCount = 0;
            view.runningIcon.removeStyleClass("status-running");
        }
        segList = null;
    },

    getSegmentSummaryReports: function(project) {
        // Update the graphs
        var controller = this;
        var view = this.oView;
        var segList = view.table.getModel().oData.segments;
        if (segList){
            var finishedCount = 0;
            for(var i=0;i<segList.length;i++){
                /*if ((segList[i].status === "completed" || segList[i].status === "error")){
                 finishedCount++;
                 }*/
                if ((segList[i].statusCounts.RUNNING === 0 && segList[i].statusCounts.SCHEDULED === 0)){
                    finishedCount++;
                }
            }

            // Check to see if the champions list has updated.
            if (controller.segSumCount !== finishedCount){
                controller.segSumCount = finishedCount;
                controller.graphsLoaded = false;
            }

            // Check to see if the champions model has changed.
            if (controller.championOverrideChanged){
                controller.graphsLoaded = false;
                controller.championOverrideChanged = false;
            }

        }

        var onSuccess = function(reports){

            var hasData = false;
            for (var i = 0; i < reports.items.length; i++) {
                if (reports.items[i].data.length > 0) {
                    hasData = true;
                    break;
                }
            }

            // If the graphs have data to render
            if (hasData) {
                controller.setReports(reports);
                controller.graphsLoaded = true;
            }
            // There are no results.  Present the no results graphic.
            else {
                controller.noResultsHandler(view);
            }
        };

        // Get the summary reports
        if (!controller.graphsLoaded && controller.segSumCount > 0) {
            segmentsProxy.getSegmentSummaryReports(project.id, onSuccess);
        }
    },

    updateReports: function(view, reports) {
        // Overridden function
        // We don't call super here, as we don't want the updateReports behavior in BaseGraphController

        // Just set the list of reports onto the model for update
        view.getModel().setProperty("/reports", reports);
    },

    noResultsHandler: function(view) {
        var controller = this;
        view.graphGrid.removeAllContent();

        var image = new sas.hc.m.Image({
            src: sas.icons.HC.ZEROSTATEGRAPH,
            height: "300px",
            width: "100%"
        });

        var msg;
        if (controller.profileBuilt)
            msg = sas.modelfactory.RB.getText("projectResults.noResults.txt");
        else
            msg = sas.modelfactory.RB.getText("projectResults.noProfileResults.txt");

        var imageTile = new sas.modelfactory.report.ReportTile({
            name: msg,
            showExpandButton: false,
            image: image
        });
        imageTile.setLayoutData(new sas.hc.ui.layout.GridData({span: "L4 M4 S4"}));

        var fillerHBox = new sap.ui.layout.HorizontalLayout({
        });
        fillerHBox.setLayoutData(new sas.hc.ui.layout.GridData({span: "L4 M4 S4"}));

        view.graphGrid.addContent(fillerHBox);
        view.graphGrid.addContent(imageTile);
    },

    onColumnMove: function(evt) {
        logUtil.log("Results table column moved:", evt.getParameter("column").sId, "moved to", evt.getParameter("newPos"));

        // A column has moved
        var params = evt.getParameters();
        this.columnsMoved(params);
    },

    columnsMoved: function(params) {
        logUtil.log("Columns changed.  Do something with columns.")
        if (params) {
            var controller = this;
            var colMoved = params.column;
            var pos = params.newPos;
            var table = controller.getView().table;
            var cols = null;
            if (table){
                cols = table.getColumns();
            }

            // Update the columms
            if (table && cols){
                for(var i=0;i<cols.length;i++){
                    if (cols[i].getId() === colMoved.getId()){
                        cols.splice(i, 1);
                        break;
                    }
                }
                cols.splice(pos, 0, colMoved);
            }
            table.removeAllColumns();
            for(var j=0;j<cols.length;j++){
                table.addColumn(cols[j]);
            }

            // Store column preferences
            var key = "FactoryMiner.ResultsColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";

            var storeSuccess = function(args) {
                controller.preferenceId = args.id;
            }

            sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "results_", table, controller.preferenceId, storeSuccess);
        }
    },

    onColumnManagerClose: function() {
        var controller = this;
        var key = "FactoryMiner.ResultsColumnPreferences.";
        if (controller.getTargetType() === "INTERVAL")
            key = key + "Numeric";
        else
            key = key + "Class";

        var table = controller.getView().table;

        var storeSuccess = function(args) {
            controller.preferenceId = args.id;
        }

        // Store column preferences
        sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "results_", table, controller.preferenceId, storeSuccess);
    },

    getTargetType: function() {
        var targetType = "UNDEFINED";
        var targetModel = sap.ui.getCore().getModel("targetType");
        if (targetModel){
            var level = targetModel.getProperty("/level");
            if (level){
                targetType = level.key;
                logUtil.log("Project target type:", targetType);
            }
        }
        return targetType;
    },

    updateMessages: function() {
        // update error message
        var controller = this;
        this.errorMsg = [];

        // Check for any errors...
        if (!controller.profileBuilt) {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.noProfile.txt");
            this.errorMsg.push(errorStr);
        }

        if (this.project.profileStatus.key === "MODIFIED") {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.modifiedProfile.txt");
            this.errorMsg.push(errorStr);
        }

        // Update the error display
        if (this.errorMsg.length  > 0) {
            var errorTxt = this.buildErrorMessage();
            var warningList = this.getView().msgList;
            if (warningList){
                warningList.removeAllItems();
                warningList.setVisible(true);
                var list = this.buildErrorList();
                for (var i = 0; i < list.length; i++) {
                    warningList.addItem(list[i]);
                }
            }
        } else {
            this.getView().msgList.setVisible(false);
        }
    },

    buildErrorMessage: function(){
        var controller = this;
        var errorTxt = sas.modelfactory.RB.getText("projectProfileView.errorMsg.txt") + "\n\n";
        for (var i=0;i<this.errorMsg.length;i++){
            errorTxt += "\u2022 " + this.errorMsg[i] + "\n";
        }
        return errorTxt;
    },

    buildErrorList: function(){
        var controller = this;
        var varList = [];
        for (var i=0;i<this.errorMsg.length;i++){
            var item = new sas.hc.m.StandardListItem({
                title: this.errorMsg[i],
                icon: sas.icons.HC.ALERT
            }).addStyleClass("ProjectDataView-warning");
            varList.push(item);
        }
        return varList;
    }
});