jQuery.sap.require(
    "sas.hc.ui.table.Table",
    "sas.hc.ui.table.Column",
    "sas.hc.ui.commons.Panel",
    "sas.hc.ui.commons.RangeSlider",
    "sas.hc.ui.commons.TextView",
    "sas.hc.m.ActionSheet",
    "sas.hc.m.Input",
    "sas.hc.m.SearchField",
    "sas.hc.m.Button",
    "sas.hc.m.CheckBox",
    "sas.hc.m.Dialog",
    "sas.hc.m.FlexItemData",
    "sas.hc.m.Input",
    "sas.hc.m.Link",
    "sas.hc.m.Text",
    "sas.hc.m.VBox",
    "sap.ui.layout.Grid",
    "sap.ui.layout.GridData",
    "sap.ui.layout.HorizontalLayout",
    "sap.ui.layout.VerticalLayout",
    "sas.modelfactory.proxies.PreferencesProxy",
    "sas.modelfactory.control.ContentSplitter",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.jsview("sas.modelfactory.project.results.ProjectResults", {

    graphGrid: null,
    carousel: null,
    statCol: null,
    completeCheckbox: null,
    runningCheckbox: null,
    scheduledCheckbox: null,
    failedCheckbox: null,
    errorCheckbox: null,
    canceledCheckbox: null,
    needsRunCheckbox: null,
    excludedCheckbox: null,
    segmentValueCols: null,
    openSegBtn: null,
    runningIcon: null,
    table: null,

    getControllerName : function() {
        return "sas.modelfactory.project.results.ProjectResults";
    },

    createContent : function(oController) {
        this.addStyleClass("FactoryMiner-ProjectResults");
        var view = this;

        // Create control for error messages, but make it invisible until errors are determined
        this.msgList = new sas.hc.m.List({
            noDataText: " ",
            visible: false
        }).addStyleClass("ProjectDataView-warning").addStyleClass("projectSettingsTabs-vLayout");

        this.graphGrid = sap.ui.jsfragment(this.createId("train_graphs"),
            "sas.modelfactory.report.RefreshableGraphLayout", oController);

        //Create an ActionSheet Control
        var oActionSheet = new sas.hc.m.ActionSheet({
            title: "Choose Your Action",
            showCancelButton: true,
            placement: sap.m.PlacementType.Bottom,
            buttons: [
                new sas.hc.m.Button({
                    id: this.createId("openBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.open.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                }),
                new sas.hc.m.Button({
                    id: this.createId("runBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.run.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                }),
                new sas.hc.m.Button({
                    id: this.createId("stopBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.stop.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                }),
                new sas.hc.m.Button({
                    id: this.createId("scoreCodeBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.scoreCode.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                }),
                new sas.hc.m.Button({
                    id: this.createId("historyBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.history.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                }),
                new sas.hc.m.Button({
                    id: this.createId("reportBtn"),
                    text: sas.modelfactory.RB.getText("projectResults.menuItem.report.txt"),
                    enabled: false,
                    press: [oController.onResultAction, oController, this]
                })
            ]
        });

        this.openSegBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.OPEN,
            tooltip: sas.modelfactory.RB.getText("projectResults.tooltip.open.txt"),
            enabled: false,
            press: [oController.onOpenPress, oController, this]
        }).addStyleClass("segBtn-action");

        this.actionButton = new sas.hc.m.Button({
            text: sas.modelfactory.RB.getText("projectResults.action.segBtn.txt"),
            icon: sas.icons.HC.ARROWSMALLDOWNBTN,
            iconFirst: false,
            press : function (evt) {
                oActionSheet.openBy(evt.getSource());
            },
            enabled: false
        }).addStyleClass("actionButton-downArrow");

        // Add filter
        this.filter = new sas.hc.m.SearchField({
            type: sap.m.InputType.Text,
            placeholder: sas.modelfactory.RB.getText("projectResults.filter.placeholder.txt")
        }).addStyleClass("modelFactoryMain-filter");

        var buttonHBox = new sap.ui.layout.HorizontalLayout(this.createId("ButtonHBoxLayout"), {
            content: [
                this.openSegBtn,
                this.actionButton
            ],
            width: "100%"
        }).addStyleClass("actionButton-floatRight");

        var tableHBox = new sap.ui.layout.HorizontalLayout(this.createId("ProjectFilter"), {
            content: [
                /*this.filter,*/
                buttonHBox
            ],
            width: "100%"
        }).addStyleClass("modelFactoryMain-tableHBox").addStyleClass("segmentTable-tableHBox");

        // Add table
        this.table = new sas.hc.ui.table.Table({
            id: this.createId("ResultsTable"),
            editable: true,
            visibleRowCount: 5,
            selectionBehavior: sap.ui.table.SelectionBehavior.Row,
            selectionMode: sap.ui.table.SelectionMode.Multi,
            rowSelectionChange: [oController.onRowSelectionClick, oController],
            columnMove: [oController.onColumnMove, oController]
        }).addStyleClass("projectResults-table");

        var statusAllFunction = function(oEvent) {
            var filters = [];

            view.allCheckbox.setSelected(true);
            view.completeCheckbox.setSelected(false);
            view.runningCheckbox.setSelected(false);
            view.scheduledCheckbox.setSelected(false);
            view.failedCheckbox.setSelected(false);
            view.errorCheckbox.setSelected(false);
            view.canceledCheckbox.setSelected(false);
            view.needsRunCheckbox.setSelected(false);
            view.excludedCheckbox.setSelected(false);

            // Also reapply existing filters for the obs and champion sliders
            var andFilters = [];
            var aggregations = view.table.getBinding("rows").aFilters;
            if (aggregations && aggregations.length > 0){
                for (var i=0;i<aggregations.length;i++){
                    if (aggregations[i].bAnd){
                        var filterList = aggregations[i].aFilters;
                        for (var j=0;j<filterList.length;j++){
                            if (!filterList[j]._bMultiFilter){
                                andFilters.push(filterList[j]);
                            }
                        }
                    }
                }
            }

            var allFilters = null;
            if (andFilters.length > 0) {
                allFilters = new sap.ui.model.Filter(andFilters, true);
            }

            view.table.getBinding("rows").filter(allFilters);
        };

        var statusChangeFunction = function(oEvent) {
            var filters = [];

            // When other status checkboxes change, turn off the All checkbox
            view.allCheckbox.setSelected(false);

            // Build the filter according to the selections
            if (view.completeCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/COMPLETED", sap.ui.model.FilterOperator.GT, 0));
            if (view.runningCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/RUNNING", sap.ui.model.FilterOperator.GT, 0));
            if (view.scheduledCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/SCHEDULED", sap.ui.model.FilterOperator.GT, 0));
            if (view.failedCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/FAILED", sap.ui.model.FilterOperator.GT, 0));
            if (view.errorCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/ERROR", sap.ui.model.FilterOperator.GT, 0));
            if (view.canceledCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/CANCELED", sap.ui.model.FilterOperator.GT, 0))
            if (view.needsRunCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/RUN_NEEDED", sap.ui.model.FilterOperator.GT, 0));
            if (view.excludedCheckbox.getSelected())
                filters.push(new sap.ui.model.Filter("statusCounts/EXCLUDED", sap.ui.model.FilterOperator.GT, 0));

            // If no checkboxes are selected, reselect the View All
            if (filters.length === 0)
                view.allCheckbox.setSelected(true);

            // Also reapply existing filters for the obs and champion sliders
            var andFilters = [];
            var aggregations = view.table.getBinding("rows").aFilters;
            if (aggregations && aggregations.length > 0){
                for (var i=0;i<aggregations.length;i++){
                    if (aggregations[i].bAnd){
                        var filterList = aggregations[i].aFilters;
                        for (var j=0;j<filterList.length;j++){
                            if (!filterList[j]._bMultiFilter){
                                andFilters.push(filterList[j]);
                            }
                        }
                    }
                }
            }

            if (filters.length > 0){
                var statusFilters = new sap.ui.model.Filter(filters, false);
                andFilters.push(statusFilters);
            }

            var allFilters = null;
            if (andFilters.length > 0){
                allFilters = new sap.ui.model.Filter(andFilters, true);
            }

            view.table.getBinding("rows").filter(allFilters);
            view.invalidate();
        };

        this.allCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusAllCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.all.txt"),
            selected: true,
            select: statusAllFunction
        });
        this.completeCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusCompleteCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.completed.txt"),
            select: statusChangeFunction
        });
        this.runningCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusRunningCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.running.txt"),
            select: statusChangeFunction
        });
        this.scheduledCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusScheduledCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.scheduled.txt"),
            select: statusChangeFunction
        });
        this.failedCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusFailedCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.failed.txt"),
            select: statusChangeFunction
        });
        this.errorCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusErrorCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.error.txt"),
            select: statusChangeFunction
        });
        this.canceledCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusCanceledCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.canceled.txt"),
            select: statusChangeFunction
        });
        this.needsRunCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusNeedsRunCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.needsRun.txt"),
            select: statusChangeFunction
        });
        this.excludedCheckbox = new sas.hc.m.CheckBox({
            id: this.createId("statusExcludedCheckbox"),
            text:  sas.modelfactory.RB.getText("projectResults.status.excluded.txt"),
            select: statusChangeFunction
        });

        var statusLayout = new sap.ui.layout.VerticalLayout(this.createId("StatusLayout"), {
            content: [
                this.completeCheckbox,
                this.runningCheckbox,
                this.scheduledCheckbox,
                this.errorCheckbox,
                this.canceledCheckbox,
                this.needsRunCheckbox,
                this.failedCheckbox,
                this.excludedCheckbox
            ]
        }).addStyleClass("projectResults-StatusLayout");

        this.statusPanel = new sas.hc.ui.commons.Panel({
            text: sas.modelfactory.RB.getText("projectResults.column.status.txt"),
            content: [
                this.allCheckbox,
                new sas.hc.ui.commons.HorizontalDivider(),
                statusLayout
            ],
            showCollapseIcon: false
        }).addStyleClass("projectResults-StatusPanel");

        this.obsSlider = new sas.hc.ui.commons.RangeSlider({
            stepLabels: true,
            value: 0,
            value2: 0
        }).addStyleClass("projectResults-slider");

        var obsReset = new sas.hc.ui.commons.Button({
            id: this.createId("resetObs"),
            icon: sas.icons.HC.RESET,
            tooltip: sas.modelfactory.RB.getText("projectResults.reset.txt"),
            press: function() {
                view.initObsSliderValues();
                var evt = {value: view.obsSlider.getMin(), value2: view.obsSlider.getMax()};
                view.obsSlider.fireChange(evt);
            }
        });

        this.obsPanel = new sas.hc.ui.commons.Panel({
            content: [
                this.obsSlider
            ],
            showCollapseIcon: false,
            height: "170px"
        }).addStyleClass("projectResults-StatusPanel");
        this.obsPanel.addButton(obsReset);

        this.championSlider = new sas.hc.ui.commons.RangeSlider({
            stepLabels: true,
            value: 0,
            value2: 0
        }).addStyleClass("projectResults-slider");

        var championReset = new sas.hc.ui.commons.Button({
            id: this.createId("resetChampion"),
            icon: sas.icons.HC.RESET,
            tooltip: sas.modelfactory.RB.getText("projectResults.reset.txt"),
            press: function() {
                view.initChampionSliderValues();
                var evt = {value: view.championSlider.getMin(), value2: view.championSlider.getMax()};
                view.championSlider.fireChange(evt);
            }
        });

        this.championPanel = new sas.hc.ui.commons.Panel({
            content: [
                this.championSlider
            ],
            showCollapseIcon: false,
            height: "170px"
        }).addStyleClass("projectResults-StatusPanel");
        this.championPanel.addButton(championReset);

        var handlePrefJson = function (oJson) {
            if (oJson.preferences && oJson.preferences.length > 0) {
                view.splitter.setSplitterPosition(oJson.preferences[0].value);
            }
        };

        prefProxy.getPreferences({"name": "ModelFactory.ResultsPanelLeftSide"}, handlePrefJson);

        this.splitter = new sas.modelfactory.control.ContentSplitter({
            id: this.createId("ModelFactory.ResultsPanelLeftSide"),
            splitterOrientation: sap.ui.commons.Orientation.Vertical,
            splitterPosition: "15%",
            minSizeFirstPane: "0.5%",
            showScrollBars: false
        });

        this.leftSide = new sap.ui.layout.VerticalLayout(this.createId("LeftSide"), {
            content: [
                this.statusPanel,
                this.obsPanel,
                this.championPanel
            ],
            width: "100%"
        }).addStyleClass("projectResults-StatusLayout");

        // Create the right side
        this.rightSide = new sap.ui.layout.VerticalLayout(this.createId("RightSide"), {
            content: [
                tableHBox,
                this.table
            ],
            width: "100%"
        });

        this.splitter.addFirstPaneContent(this.leftSide);
        this.splitter.addSecondPaneContent(this.rightSide);

        var graphPanel = new sas.hc.ui.commons.Panel({
            text: sas.modelfactory.RB.getText("projectResults.panel.title.txt"),
            height: "420px",
            showCollapseIcon: true,
            content:[
                this.graphGrid
            ]
        }).addStyleClass("projectResults-graphPanel");

        this.tableBox = new sap.ui.layout.VerticalLayout(this.createId("ProjectResultsLayout"), {
            content: [
                this.msgList,
                graphPanel,
                this.splitter
            ],
            width: "100%"
        });

        return this.tableBox;
    },

    addColumns: function() {
        var oController = this.getController();

        this.isPartitioned = false;
        var projModel = sap.ui.getCore().getModel("openProject");
        if (projModel){
            var project = projModel.getProperty("/project");
            if (project){
                this.isPartitioned = project.usePartition;
            }
        }

        var targetType = "UNDEFINED";
        var targetModel = sap.ui.getCore().getModel("targetType");
        if (targetModel){
            var level = targetModel.getProperty("/level");
            if (level){
                targetType = level.key;
                logUtil.log("Project target type:", targetType);
            }
        }

        var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
        if (localeStr === null)
            localeStr = "en_US";

        var floatFmt = sap.ui.core.format.NumberFormat.getFloatInstance({
            minFractionDigits: 4,
            maxFractionDigits: 4,
            groupingEnabled: true
        }, new sap.ui.core.Locale(localeStr));

        /* var nameCol = new sas.hc.ui.table.Column(this.createId("results_target"), {
         label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectResults.column.target.txt")}),
         width: "7%",
         template: new sas.hc.ui.commons.TextView({text: "{targetName}"}),
         sortProperty: "targetName",
         filterProperty: "targetName"});
         this.table.addColumn(nameCol);*/

        var segCol = this.byId("results_segId");
        if (segCol === undefined){
            segCol = new sas.hc.ui.table.Column(this.createId("results_segId"), {
                label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectResults.column.segmentId.txt")}),
                width: "10%",
                template: new sas.hc.m.Link({
                    text: "{segmentId}",
                    target: "_self",
                    press: function(evt) {
                        oController.onDrillInLinkPress.call(oController, this);
                    }
                }).bindProperty("text",{
                    path: "segmentId",
                    formatter: function(segmentId) {
                        return sas.modelfactory.RB.getText("projectResults.column.segment.txt") + " " + segmentId;
                    }
                }).data("id", "{id}"),
                /*template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path: "segmentId",
                    formatter: function(segmentId) {
                        return sas.modelfactory.RB.getText("projectResults.column.segment.txt") + " " + segmentId;
                    }
                }),*/
                sortProperty: "segmentId",
                filterProperty: "segmentId"
            });
        }
        this.table.addColumn(segCol);

        this.segmentValueCols = [];
        var segmentValueCol = this.byId("results_segVal1");
        if (segmentValueCol === undefined){
            segmentValueCol = new sas.hc.ui.table.Column(this.createId("results_segVal1"), {
                label: new sas.hc.m.Text({text: "SegVal1"}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path: "segmentData",
                    formatter: function(data) {
                        if (data && data.length > 0) {
                            if (data[0].isNumeric)
                                return data[0].numVal.toString();
                            else
                                return data[0].charVal;
                        }
                    }
                }),
                sortProperty: "segmentData/0/charVal",
                filterProperty: "segmentData/0/charVal",
                visible: false
            });
        }
        this.table.addColumn(segmentValueCol);
        this.segmentValueCols.push(segmentValueCol);

        segmentValueCol = this.byId("results_segVal2");
        if (segmentValueCol === undefined){
            segmentValueCol = new sas.hc.ui.table.Column(this.createId("results_segVal2"), {
                label: new sas.hc.m.Text({text: "SegVal2"}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path: "segmentData",
                    formatter: function(data) {
                        if (data && data.length > 1) {
                            if (data[1].isNumeric)
                                return data[1].numVal.toString();
                            else
                                return data[1].charVal;
                        }
                    }
                }),
                sortProperty: "segmentData/1/charVal",
                filterProperty: "segmentData/1/charVal",
                visible: false
            });
        }
        this.table.addColumn(segmentValueCol);
        this.segmentValueCols.push(segmentValueCol);

        segmentValueCol = this.byId("results_segVal3");
        if (segmentValueCol === undefined){
            segmentValueCol = new sas.hc.ui.table.Column(this.createId("results_segVal3"), {
                label: new sas.hc.m.Text({text: "SegVal3"}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path: "segmentData",
                    formatter: function(data) {
                        if (data && data.length > 2) {
                            if (data[2].isNumeric)
                                return data[2].numVal.toString();
                            else
                                return data[2].charVal;
                        }
                    }
                }),
                sortProperty: "segmentData/2/charVal",
                filterProperty: "segmentData/2/charVal",
                visible: false
            });
        }
        this.table.addColumn(segmentValueCol);
        this.segmentValueCols.push(segmentValueCol);

        var col = this.byId("results_champion");
        if (col === undefined){
            col = new sas.hc.ui.table.Column(this.createId("results_champion"), {
                label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectResults.column.champion.txt")}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path: "championModel",
                    formatter:function(champ) {
                        if (champ){
                            return champ.name;
                        }
                    }
                }),
                sortProperty: "champion",
                filterProperty: "champion"
            });
        }
        this.table.addColumn(col);

        col = this.byId("results_override");
        if (col === undefined){
            col = new sas.hc.ui.table.Column(this.createId("results_override"), {
                label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectResults.column.championOverriden.txt")}),
                width: "9%",
                template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                    path:"championOverridden",
                    formatter:function(championOverridden) {
                        if (championOverridden){
                            return sas.modelfactory.RB.getText("projectResults.column.championOverridenYes.txt");
                        } else {
                            return sas.modelfactory.RB.getText("projectResults.column.championOverridenNo.txt");
                        }
                    }}),
                sortProperty: "championOverridden",
                filterProperty: "championOverridden"
            });
        }
        this.table.addColumn(col);

        this.statCol = this.byId("results_lift");
        if (this.statCol === undefined){
            this.statCol = new sas.hc.ui.table.Column(this.createId("results_lift"), {
                label: new sas.hc.m.Text({textAlign: "End", /*wrapping: true,*/ text: sas.modelfactory.RB.getText("projectResults.column.lift.txt")}),
                width: "10%",
                //template: new sas.hc.ui.commons.TextView({text: "{championStatisticValue}"}),
                template: new sas.hc.ui.commons.TextView({textAlign: "End"}).bindProperty("text", {
                    path:"championStatisticValue",
                    formatter:function(stat) {
                        if (stat != undefined && ( stat !== "NaN")){
                            return floatFmt.format(stat);
                        }
                    }}),
                sortProperty: "championStatisticValue",
                filterProperty: "championStatisticValue",
                styleClass: "fmColumnTextAlignEnd"
            });
        }
        this.table.addColumn(this.statCol);

        col = this.byId("results_obs_t");
        if (col === undefined){
            col = new sas.hc.ui.table.Column(this.createId("results_obs_t"), {
                label: new sas.hc.m.Text({textAlign: "End", /*wrapping: true,*/ text: sas.modelfactory.RB.getText("projectResults.column.observations_t.txt")}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{trainObs}"}),
                sortProperty: "trainObs",
                filterProperty: "trainObs",
                visible: !this.isPartitioned,
                styleClass: "fmColumnTextAlignEnd"
            });
        }
        this.table.addColumn(col);

        col = this.byId("results_obs_v");
        if (col === undefined){
            col = new sas.hc.ui.table.Column(this.createId("results_obs_v"), {
                label: new sas.hc.m.Text({textAlign: "End", /*wrapping: true,*/ text: sas.modelfactory.RB.getText("projectResults.column.observations_v.txt")}),
                width: "10%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{validateObs}"}),
                hAlign: "Right",
                sortProperty: "validateObs",
                filterProperty: "validateObs",
                visible: this.isPartitioned,
                styleClass: "fmColumnTextAlignEnd"
            });
        }
        this.table.addColumn(col);

        var completedStatusCol = this.byId("results_status_completed");
        if (completedStatusCol === undefined){
            completedStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_completed"), {
                /*label: new sas.hc.m.Text({
                    textAlign: "Center",
                    icon: sas.icons.HC.COMPLETESTATUS,
                    tooltip: sas.modelfactory.RB.getText("projectResults.status.completed.txt")
                }).addStyleClass("status-icon"),*/
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.COMPLETESTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.completed.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/COMPLETED}"}),
                sortProperty: "statusCounts/COMPLETED",
                filterProperty: "statusCounts/COMPLETED"
            });
        }
        this.table.addColumn(completedStatusCol);

        var runningStatusCol = this.byId("results_status_running");
        if (runningStatusCol === undefined){
            runningStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_running"), {
                label: this.runningIcon = new sas.hc.m.Image({
                    src: sas.icons.HC.PROGRESSSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.running.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/RUNNING}"}),
                sortProperty: "statusCounts/RUNNING",
                filterProperty: "statusCounts/RUNNING"
            });
        }
        this.table.addColumn(runningStatusCol);

        var scheduledStatusCol = this.byId("results_status_scheduled");
        if (scheduledStatusCol === undefined){
            scheduledStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_scheduled"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.SCHEDULEDSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.scheduled.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/SCHEDULED}"}),
                sortProperty: "statusCounts/SCHEDULED",
                filterProperty: "statusCounts/SCHEDULED"
            });
        }
        this.table.addColumn(scheduledStatusCol);

        var errorStatusCol = this.byId("results_status_error");
        if (errorStatusCol === undefined){
            errorStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_error"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.WARNINGSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.error.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/ERROR}"}),
                sortProperty: "statusCounts/ERROR",
                filterProperty: "statusCounts/ERROR"
            });
        }
        this.table.addColumn(errorStatusCol);

        var canceledStatusCol = this.byId("results_status_canceled");
        if (canceledStatusCol === undefined){
            canceledStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_canceled"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.CANCELEDSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.canceled.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/CANCELED}"}),
                sortProperty: "statusCounts/CANCELED",
                filterProperty: "statusCounts/CANCELED"
            });
        }
        this.table.addColumn(canceledStatusCol);

        var run_neededStatusCol = this.byId("results_status_run_needed");
        if (run_neededStatusCol === undefined){
            run_neededStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_run_needed"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.REVIEWSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.needsRun.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/RUN_NEEDED}"}),
                sortProperty: "statusCounts/RUN_NEEDED",
                filterProperty: "statusCounts/RUN_NEEDED"
            });
        }
        this.table.addColumn(run_neededStatusCol);

        var failedStatusCol = this.byId("results_status_failed");
        if (failedStatusCol === undefined){
            failedStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_failed"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.ERRORSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.failed.txt")),
                width: "5%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/FAILED}"}),
                sortProperty: "statusCounts/FAILED",
                filterProperty: "statusCounts/FAILED"
            });
        }
        this.table.addColumn(failedStatusCol);

        var excludedStatusCol = this.byId("results_status_excluded");
        if (excludedStatusCol === undefined){
            excludedStatusCol = new sas.hc.ui.table.Column(this.createId("results_status_excluded"), {
                label: new sas.hc.m.Image({
                    src: sas.icons.HC.FLAGGEDSTATUS,
                    width: "1rem",
                    height: "1rem"
                }).setTooltip(sas.modelfactory.RB.getText("projectResults.status.excluded.txt")),
                width: "6%",
                template: new sas.hc.ui.commons.TextView({textAlign: "End", text: "{statusCounts/EXCLUDED}"}),
                sortProperty: "statusCounts/EXCLUDED",
                filterProperty: "statusCounts/EXCLUDED"
            });
        }
        this.table.addColumn(excludedStatusCol);

        this.filter.attachLiveChange(function(oEvent) {
            this.table.filter(segCol, this.filter.getValue());
        }, this);

        // Define a function to handle changes in the Observation filter
        var obsChangeFunction = function(oEvent) {
            var view = this;
            var filters = [];
            var colName;
            if (this.isPartitioned)
                colName = "validateObs";
            else
                colName = "trainObs";
            this.obsFilter = new sap.ui.model.Filter(colName, sap.ui.model.FilterOperator.BT, oEvent.getParameters().value, oEvent.getParameters().value2);
            filters.push(this.obsFilter);

            // Also reapply any existing filters for the status selections and champion slider
            var aggregations = view.table.getBinding("rows").aFilters;
            if (aggregations && aggregations.length > 0){
                for (var i=0;i<aggregations.length;i++){
                    if (aggregations[i].bAnd){
                        var filterList = aggregations[i].aFilters;
                        for (var j=0;j<filterList.length;j++){
                            if (filterList[j].sPath !== colName){
                                filters.push(filterList[j]);
                            }
                        }
                    }
                }
            }

            var allFilters = null;
            if (filters.length > 0){
                allFilters = new sap.ui.model.Filter(filters, true);
            }

            this.table.getBinding("rows").filter(allFilters);
        };

        // Attach this callback to both the Change and LiveChange events.
        // Change is fired when the input box is typed in.  LiveChange is fired on mouse movements
        this.obsSlider.attachChange(obsChangeFunction, this);
        this.obsSlider.attachLiveChange(obsChangeFunction, this);

        // Define a function to handle changes in the Champion criterion filter
        var championChangeFunction = function(oEvent) {
            var view = this;
            var filters = [];
            this.championFilter = new sap.ui.model.Filter("championStatisticValue", sap.ui.model.FilterOperator.BT, oEvent.getParameters().value, oEvent.getParameters().value2);
            filters.push(this.championFilter);

            // Also reapply any existing filters for the status selections and obs slider
            var aggregations = view.table.getBinding("rows").aFilters;
            if (aggregations && aggregations.length > 0){
                for (var i=0;i<aggregations.length;i++){
                    if (aggregations[i].bAnd){
                        var filterList = aggregations[i].aFilters;
                        for (var j=0;j<filterList.length;j++){
                            if (filterList[j].sPath !== "championStatisticValue"){
                                filters.push(filterList[j]);
                            }
                        }
                    }
                }
            }

            var allFilters = null;
            if (filters.length > 0){
                allFilters = new sap.ui.model.Filter(filters, true);
            }

            this.table.getBinding("rows").filter(allFilters);
        };

        // Attach this callback to both the Change and LiveChange events.
        // Change is fired when the input box is typed in.  LiveChange is fired on mouse movements
        this.championSlider.attachChange(championChangeFunction, this);
        this.championSlider.attachLiveChange(championChangeFunction, this);

        var title;
        if (this.isPartitioned)
            title = sas.modelfactory.RB.getText("projectResults.column.observations_v.txt");
        else
            title = sas.modelfactory.RB.getText("projectResults.column.observations_t.txt");
        this.obsPanel.setTitle(new sas.hc.ui.core.Title({text: title}));
        this.obsPanel.setTooltip(title);
    },

    getStatusData: function(){
        // Initialize counts
        var completeCount = 0;
        var runningCount = 0;
        var scheduledCount = 0;
        var failedCount = 0;
        var errorCount = 0;
        var canceledRunCount = 0;
        var needsRunCount = 0;
        var excludedCount = 0;

        var model = this.table.getModel();
        if (model){
            //var segments = model.oData.segments;
            var segments = model.getProperty("/segments");

            if (segments && segments.length > 0){
                for (var i=0;i<segments.length;i++){
                    if (segments[i].statusCounts.COMPLETED > 0){
                        completeCount += segments[i].statusCounts.COMPLETED;
                    }
                    if (segments[i].statusCounts.RUNNING > 0){
                        runningCount += segments[i].statusCounts.RUNNING;
                    }
                    if (segments[i].statusCounts.SCHEDULED > 0){
                        scheduledCount += segments[i].statusCounts.SCHEDULED;
                    }
                    if (segments[i].statusCounts.FAILED > 0){
                        failedCount += segments[i].statusCounts.FAILED;
                    }
                    if (segments[i].statusCounts.ERROR > 0){
                        errorCount += segments[i].statusCounts.ERROR;
                    }
                    if (segments[i].statusCounts.CANCELED > 0){
                        canceledRunCount += segments[i].statusCounts.CANCELED;
                    }
                    if (segments[i].statusCounts.RUN_NEEDED > 0){
                        needsRunCount += segments[i].statusCounts.RUN_NEEDED;
                    }
                    if (segments[i].statusCounts.EXCLUDED > 0){
                        excludedCount += segments[i].statusCounts.EXCLUDED;
                    }
                }
            }
        }

        this.completeCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.completed.txt") + " (" + completeCount + ")");
        this.runningCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.running.txt") + " (" + runningCount + ")");
        this.scheduledCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.scheduled.txt") + " (" + scheduledCount + ")");
        this.failedCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.failed.txt") + " (" + failedCount + ")");
        this.errorCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.error.txt") + " (" + errorCount + ")");
        this.canceledCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.canceled.txt") + " (" + canceledRunCount + ")");
        this.needsRunCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.needsRun.txt") + " (" + needsRunCount + ")");
        this.excludedCheckbox.setText(sas.modelfactory.RB.getText("projectResults.status.excluded.txt") + " (" + excludedCount + ")");
        segments = null;
        model = null;
    },

    initObsSliderValues: function(){
        // Initialize counts
        var obsMax = 0;

        var model = this.table.getModel();
        if (model && model.oData.segments) {
            var segments = model.oData.segments;
            if (segments.length > 0) {

                // Obs max is initialized to the first segment's obs
                if (this.isPartitioned) {
                    obsMax = segments[0].validateObs;
                }
                else {
                    obsMax = segments[0].trainObs;
                }

                for (var i = 1; i < segments.length; i++){
                    if (this.isPartitioned) {
                        if (segments[i].validateObs > obsMax){
                            obsMax = segments[i].validateObs;
                        }
                    }
                    else {
                        if (segments[i].trainObs > obsMax){
                            obsMax = segments[i].trainObs;
                        }
                    }
                }
            }

            // Round up to the nearest 100 for Obs max
            obsMax = Math.ceil(obsMax / 100) * 100;

            this.obsSlider.setMin(0);
            this.obsSlider.setMax(obsMax);
            this.obsSlider.setValue(0);
            this.obsSlider.setValue2(obsMax);
            this.obsSlider.setSmallStepWidth(1);
        }
        else {
            this.obsSlider.setValue(this.obsSlider.getMin());
            this.obsSlider.setValue2(this.obsSlider.getMin());
        }
    },

    initChampionSliderValues: function(){
        // Initialize counts
        var championMin = 0;
        var championMax = 1;
        var championStep = 0.001;
        var divisor;

        var model = this.table.getModel();
        if (model && model.oData.segments){
            var segments = model.oData.segments;
            if (segments.length > 0) {

                // Find the largest statistic value in the list
                championMax = segments[0].championStatisticValue;;
                championMin = championMax;
                for (var i = 1; i < segments.length; i++) {
                    if (segments[i].championStatisticValue > championMax)
                        championMax = segments[i].championStatisticValue;
                    else if (segments[i].championStatisticValue < championMin)
                        championMin = segments[i].championStatisticValue;
                }

                if (isNaN(championMin) || isNaN(championMax)) {
                    championMin = 0;
                    championMax = 1;
                }
                // If max value is greater than zero but less than 1, then set the max to 1
                else if (championMax > 0 && championMax <= 1)
                    championMax = 1;
                // Otherwise, set the max to a multiple of 100 or 10, whichever is needed
                else {
                    if (Math.abs(championMax) >= 100) {
                        divisor = 100;
                        championStep = 1;
                    }
                    else {
                        divisor = 10;
                        championStep = 0.1;
                    }
                    championMax = Math.ceil(championMax / divisor) * divisor;
                }

                // If min value is >= 0, then set the min to 0
                if (championMin >= 0)
                    championMin = 0;
                // Otherwise, min is negative so set min to a negative multiple of 10 or 100, whichever is needed
                else {
                    if (Math.abs(championMin) <= 10) {
                        divisor = 10;
                        championStep = 0.1;
                    }
                    else {
                        divisor = 100;
                        championStep = 1;
                    }
                    championMin = Math.floor(championMin / divisor) * divisor;
                }

                this.championSlider.setMin(championMin);
                this.championSlider.setMax(championMax);
                this.championSlider.setValue(championMin);
                this.championSlider.setValue2(championMax);
                this.championSlider.setSmallStepWidth(championStep);
            }
        }
        else {
            this.championSlider.setValue(this.championSlider.getMin());
            this.championSlider.setValue2(this.championSlider.getMin());
        }
    },

    setAndBindModel: function(model, bindPath) {
        var view = this;
        this.table.setModel(model);
        this.table.bindRows(bindPath);

        var segments = model.oData.segments;
        if (segments.length > 0) {
            this.addColumns();

            // Initially sort the table by segment number
            this.table.sort(this.table.getColumns()[0]);

            // Select the first row in the table
            this.table.setSelectedIndex(0);
            var context = this.table.getContextByIndex(0);
            var evt = {rowIndex: 0, rowContext: context, rowIndices: [0]};
            this.table.fireRowSelectionChange(evt);

            // Init the slider filters based upon actual segment data
            view.initObsSliderValues();
            view.initChampionSliderValues();

            // Determine the statistic column
            var transStatCol = sas.modelfactory.RB.getText(segments[0].championStatisticName);
            view.statCol.setLabel(new sas.hc.m.Text({textAlign: "End", /*wrapping: true,*/ text: transStatCol}));

            // Post process the bind for segment values
            if (segments[0].segmentData) {

                // Appropriately name the segment value column(s) (are always the same for all segments...)
                var segmentText = sas.modelfactory.RB.getText("projectResults.column.segment.txt");
                for (var i = 0; i < segments[0].segmentData.length; i++) {
                    var columnLabel = segmentText + " (" + segments[0].segmentData[i].columnName + ")";
                    this.segmentValueCols[i].setLabel(new sas.hc.m.Text({text: columnLabel}));
                    this.segmentValueCols[i].setVisible(true);
                }

                // Remove the unneeded segment value column(s) preassigned above (MAX of 3 allowed)
                for (var i = 2; i >= segments[0].segmentData.length; i--) {
                    this.table.removeColumn(this.segmentValueCols[i]);
                }
            }

            // Wait until now to set the champion label on the slider, as it may have changed
            this.championPanel.setTitle(new sas.hc.ui.core.Title({text: transStatCol}));
            this.championPanel.setTooltip(transStatCol);

            // Adjust the running status column header
            /*setTimeout(function(){
                var domId = "#" + view.byId("results_status_running").sId;
                jQuery(domId).css("textAlign", "center");
            }, 1000);*/
        }
    },

    displayGraph: function (evt)
    {
        var id = evt.getParameter("id");
        var image = document.getElementById(id).src;

        var dialog = new sas.hc.m.Dialog({
            title: "Graph",
            stretch: jQuery.support.touch,
            beginButton: new sas.hc.m.Button({
                text: "Close",
                press: function ()
                {
                    dialog.close();
                }
            }),
            afterClose: function ()
            {
                dialog.destroy();
            }
        });

        dialog.addContent(
            new sap.ui.commons.Image({
                src: image
            }));
        dialog.open();
    }

});