jQuery.sap.require(
    "sas.hc.m.MessageBox",
    "sas.modelfactory.proxies.ProjectProxy",
    "sas.modelfactory.proxies.ResultsProxy",
    "sas.modelfactory.util.PreferencesUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.controller("sas.modelfactory.project.settings.ProjectData", {

    fragInitData: null,
    freqCount: null,
    keyCount: null,
    segmentCount: null,
    targetCount: null,
    inputCount: null,
    errorMsg: null,
    project: null,
    profileBuilt: false,
    dialog: null,
    dataLoaded: false,
    preferenceId: null,
    initTableSize: 0,
    buildProfileAvailable: true,
    targetAndSegmentDefined: false,
    modelsHaveBeenRun: false,

/**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.sas.modelfactory.project.settings.ProjectData
     */
    onInit: function() {
        sas.eventBus.subscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.subscribe("ProjectEvent", "projectRetrained", this.handleRetrainingEvent, this);

        var controller = this;

        this.getView().addEventDelegate({
            onBeforeFirstShow: function() {
                logUtil.log("ProjectData onBeforeFirstShow hit");
                controller.getView().setHeight("100%");
            },

            onBeforeShow: function() {
                logUtil.log("ProjectData onBeforeShow hit");
                controller.initializeView();
                controller.handleMetadataRowSelectionChange(null);
            }
        });

        this.getView().table.attachAfterColumnsDialogClose(function(oEvent) {
            // If OK was pressed, persist the column manager changes
            if (oEvent.getParameter("acceptedChange"))
                controller.onColumnManagerClose();
        });
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.project.settings.ProjectData
     */
    onBeforeRendering: function() {

    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.project.settings.ProjectData
     */
    onAfterRendering: function() {

        //jQuery(".modelFactoryMain-buttonHBox").parent().css("float", "right");

        // Initial scan of data variables
        var controller = this;
        if (controller.oView.table.getModel()){
            setTimeout(function(){controller.processControlCounts();}, 0);
        }
    },


    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.project.settings.ProjectData
     */
    onExit: function() {
        logUtil.log("sas.modelfactory.project.settings.ProjectData: onExit called");

        sas.eventBus.unsubscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.unsubscribe("ProjectEvent", "projectRetrained", this.handleRetrainingEvent, this);

        sap.ui.getCore().setModel(null, "targetType");
    },

    handleRetrainingEvent: function(channelId, eventId, data) {
        this.dataLoaded = false;
        this.initializeView();
    },

    handleProjectLoadedEvent: function(channelId, eventId, data) {
        var controller = this;
        if (data){
            controller.buildProfileAvailable = data.buildProfileAvailable;
            controller.profileBuilt = data.profileBuilt;
            controller.targetAndSegmentDefined = data.targetAndSegmentDefined;
            controller.modelsHaveBeenRun = data.modelsHaveBeenRun;

            controller.initializeView();
        }
    },

    initializeView : function() {
        var controller = this;

        // Get the current open project
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
            controller.buildProfileAvailable = model.getProperty("/buildProfileAvailable");
            controller.profileBuilt = model.getProperty("/profileBuilt");
            controller.targetAndSegmentDefined = model.getProperty("/targetAndSegmentDefined");
            controller.modelsHaveBeenRun = model.getProperty("/modelsHaveBeenRun");
        }

        if (project) {
            controller.project = project;

            // Get the datasource variables for the project
            if (controller.project){
                controller.getVariables(controller.project);
            }
        }
    },


    handleMetadataRowSelectionChange: function(oEvent) {
        var controller = this;
        var index = this.oView.table.getSelectedIndex();
        var selected = this.oView.table.getSelectedIndices();

        // Enable/disable the edit button correctly
        this.oView.editButton.setEnabled((selected.length > 0) && controller.isTargetOrSegmentChangeAllowed(selected));

        // Enable/disable the view distribution button correctly
        if (selected.length === 1) {
            this.oView.viewDistributionButton.setEnabled(true);
        } else {
            this.oView.viewDistributionButton.setEnabled(false);
        }
    },

    editPress: function(evt) {
        var indices = this.oView.table.getSelectedIndices();
        var array = [];
        for (var i=0;i<indices.length;i++) {
            var context = this.oView.table.getContextByIndex(indices[i]);
            var selectedData = context.getObject();
            logUtil.log(selectedData);
            array.push(selectedData);
        }

        this.fragInitData = {
            selectedVars: array,
            project: this.project
        };
        logUtil.log("Edit data variables:", this.fragInitData);

        var dialog = sap.ui.jsfragment(this.createId("dataVarFragment"), "sas.modelfactory.dialog.DataVariablesDialog", this);
        dialog.open();
    },

    viewDistributionPress: function(evt) {
        var controller = this;

        var context = this.oView.table.getContextByIndex(this.oView.table.getSelectedIndex());
        var thisVar = context.getObject();
        var title = sas.modelfactory.RB.getText("projectDataView.viewDistribution.title.txt");

        busySpinner.show(true,
            title,
            sas.modelfactory.RB.getText("projectDataView.viewDistribution.loading.txt")
        );

        var onViewDistribution = function(oJson) {
            // Create the distribution report
            jQuery.sap.require("sas.modelfactory.dialog.ViewDistributionDialog");

            var dialog = new sas.modelfactory.dialog.ViewDistributionDialog({
                title: title + ": " + thisVar.name,
                content: sap.ui.view({
                    type: sap.ui.core.mvc.ViewType.JS,
                    id: controller.createId("ViewDistribution"),
                    viewName: "sas.modelfactory.report.ExpandedGraph",
                    viewData: {items: [oJson]}
                }),
                contentWidth: "90%",
                contentHeight: "90%"
            });

            dialog.open();
        };

        var onComplete = function() {
            busySpinner.show(false);
        }

        // Get the distribution plot
        resultsProxy.getDistributionPlot(controller.project.id, thisVar.id, onViewDistribution, null, onComplete);
    },

    onUpdateDatasource: function(role, level, order, transform, impute) {
        logUtil.log("onUpdateDatasource: ", role, level, order, transform, impute );
        logUtil.log("on this data:", this.fragInitData);
        var enumModel = sap.ui.getCore().getModel("enums");
        var roleMap = {};
        var levelMap = {};
        var orderMap = {};
        var transMap = {};
        var imputeMap = {};
        if (enumModel){
            roleMap = enumModel.getProperty("/roles");
            levelMap = enumModel.getProperty("/levels");
            orderMap = enumModel.getProperty("/orders");
            transMap = enumModel.getProperty("/transformationMethods");
            imputeMap = enumModel.getProperty("/imputations");

            if (roleMap === undefined || levelMap === undefined || orderMap === undefined || transMap === undefined || imputeMap === undefined){
                logUtil.error("Enumeration Error: enum map missing", roleMap, levelMap, orderMap, transMap, imputeMap);
                return;
            }

            var mixed = "MIXED";
            var savedData = this.fragInitData.selectedVars;
            for(var i=0;i<savedData.length;i++){
                if (role != mixed)
                    savedData[i].role = role?roleMap[role]:" ";
                if (level != mixed)
                    savedData[i].level = level?levelMap[level]:" ";
                if (order != mixed)
                    savedData[i].order = order?orderMap[order]:" ";
                if (transform != mixed)
                    savedData[i].transformationMethod = transform?transMap[transform]:" ";
                if (impute != mixed)
                    savedData[i].imputationMethod = impute?imputeMap[impute]:" ";
            }

            // Update the datasource with the user changes
            this.updateAllVariables(savedData);
            logUtil.log("Saved datasource variables:", savedData);
        }
    },

    processControlCounts: function(){
        var controller = this;

        // Clear the error message
        this.errorMsg = [];
        this.clearAllErrors();

        // Update the variable counts
        this.analyzeData();

        // ----------------------------------------
        //              Role Rules
        // ----------------------------------------

        // At most 1 FREQ variable.
        var error_maxFREQ = this.freqCount > 1;

        // At most 1 KEY variable
        var error_maxKEY = this.keyCount > 1;

        // Maximum of 3 SEGMENT variables
        var error_maxSEG = this.segmentCount > 3;

        // Minimum of 1 SEGMENT variable
        var error_minSEG = this.segmentCount < 1;

        // Minimum of 1 TARGET variable
        var error_minTARGET = this.targetCount < 1;

        // For first release we will only support 1 target
        var error_maxTARGET = this.targetCount > 1;

        // At most 1 TARGET variable if SEGMENTS defined
        // If no SEGMENTS defined, no limit on amount of TARGET variables
        // This rule does not apply for the first release
        //var error_SEG_TARGET = this.segmentCount >= 1 && this.targetCount > 1;

        // Minimum of 1 INPUT variable
        var error_minINPUT = this.inputCount < 1;

        var errorArray = controller.getView().errorArray;

        // Does the profile need to be rebuilt?
        if (this.project && this.project.profileStatus.key === "MODIFIED") {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.modifiedProfile.txt");
            this.errorMsg.push(errorStr);
        }

        // Process the rule errors
        if (error_maxFREQ){
            errorArray.push("FREQ");
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.maxFREQ.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("FREQ Error:", errorStr);
        }

        if (error_maxKEY){
            errorArray.push("KEY");
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.maxKEY.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("KEY Error:", errorStr);
        }

        if (error_maxSEG){
            errorArray.push("SEGMENT");
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.maxSEG.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("SEGMENT Error:", errorStr);
        }

        if (error_minSEG){
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.minSEG.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("SEGMENT Error:", errorStr);
        }

        if (error_minTARGET){
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.minTARGET.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("TARGET Error:", errorStr);
        }

        // For first release we will only support 1 target
        if (error_maxTARGET){
            errorArray.push("TARGET");
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.maxTARGET.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("TARGET Error:", errorStr);
        }

        // This rule does not apply for the first release
        /*if (error_SEG_TARGET){
         var errArray = this.findRoleVariables("TARGET");
         this.markError(errArray);
         errArray = this.findRoleVariables("SEGMENT");
         this.markError(errArray, "Error message test.");
         var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.SEG_TARGET.txt");
         this.errorMsg.push(errorStr);
         logUtil.log("TARGET SEGMENT Conflict Error:", errorStr);
         }*/

        if (error_minINPUT){
            var errorStr = sas.modelfactory.RB.getText("projectDataView.errorStr.minINPUT.txt");
            this.errorMsg.push(errorStr);
            logUtil.log("INPUT Error:", errorStr);
        }

        this.markError(errorArray);

        // Update the error display
        var objectContentPage = sap.ui.getCore().byId(controller.getView().data("objectContentPageId"));
        var view = null;
        if (objectContentPage){
            view = objectContentPage.getParent();
        };

        if (this.errorMsg.length  > 0 && controller.dataLoaded) {
            var errorTxt = this.buildErrorMessage();
            var warningList = this.getView().varList;
            if (warningList){
                warningList.removeAllItems();
                warningList.setVisible(true);
                var list = this.buildErrorList();
                for (var i = 0; i < list.length; i++) {
                    warningList.addItem(list[i]);
                }
            }

            var top = this.getView().pageLayout.getTop();
            var size = 48 + 30 + this.errorMsg.length * 40;
            size += "px";
            top.setSize(size);

            // There are errors ... Disable the build profile button
            if (view){
                view.buildProfileButton.setEnabled(false);
            }

        } else {
            this.getView().varList.setVisible(false);
            var top = this.getView().pageLayout.getTop();
            top.setSize("48px");

            // Enable the build profile button if no jobsHaveRun yet
            if (view && controller.project){
                view.buildProfileButton.setEnabled(controller.project.buildProfileAvailable);
            }
        }
    },

    markError: function(list, msg){
        var view = this.getView();
        view.roleCol.invalidate();
    },

    clearAllErrors: function(){
        var view = this.getView();
        view.errorArray = [];
        view.roleCol.invalidate();
    },

    findRoleVariables: function(variable){
        var varArray = [];

        // Scan the data variables
        var data = this.oView.table.getModel().oData.preData;
        for (var i=0;i<data.length;i++){
            var context = this.oView.table.getContextByIndex(i);
            var selectedData = context.getObject();
            if (selectedData.role.key === variable)
                varArray.push(i);
        }
        return varArray;
    },

    analyzeData: function() {
        // Initialize counts
        this.freqCount = 0;
        this.keyCount = 0;
        this.segmentCount = 0;
        this.targetCount = 0;
        this.inputCount = 0;

        // Scan the data variables and increment variable counts
        var data = this.oView.table.getModel().oData.preData;
        if (data){
            for (var i=0;i<data.length;i++){
                if (data[i].role.key === "KEY")
                    this.keyCount++;
                if (data[i].role.key === "FREQ")
                    this.freqCount++;
                if (data[i].role.key === "SEGMENT")
                    this.segmentCount++;
                if (data[i].role.key === "TARGET")
                    this.targetCount++;
                if (data[i].role.key === "INPUT")
                    this.inputCount++;
            }
        }
        logUtil.log("KEY:", this.keyCount, "\nFREQ:", this.freqCount, "\nSEG:", this.segmentCount, "\nTARGET:", this.targetCount, "\nINPUT:", this.inputCount);
    },

    buildErrorMessage: function(){
        var controller = this;
        var errorTxt = sas.modelfactory.RB.getText("projectDataView.errorMsg.txt") + "\n\n";
        for (var i=0;i<this.errorMsg.length;i++){
            errorTxt += "\u2022 " + this.errorMsg[i] + "\n";
        }
        return errorTxt;
    },

    buildErrorList: function(){
        var controller = this;
        var varList = [];
        for (var i=0;i<this.errorMsg.length;i++){
            var item = new sas.hc.m.StandardListItem({
                title: this.errorMsg[i],
                icon: sas.icons.HC.ALERT
            }).addStyleClass("ProjectDataView-warning");
            varList.push(item);
        }

        return varList;
    },

    showDataVariableErrors: function(evt) {
        // Update the error display
        if (this.errorMsg.length  > 0){
            var errorTxt = this.buildErrorMessage();
            sas.hc.m.MessageBox.error(errorTxt);
        }
    },


    getVariables: function(proj) {

        var model = new sap.ui.model.json.JSONModel();
        var view = this.oView;
        var controller = this;

        var onSuccess = function(oJson){
            if (!controller.dataLoaded){
                controller.dataLoaded = true;
                model.setData({preData: oJson.items});
                view.setAndBindModel(model, "/preData");
            } else {
                // Update the data
                view.table.getModel().setProperty("/preData", oJson.items);
            }

            // Save the preferenceId for later
            var callback = function(preferenceId) {
                controller.preferenceId = preferenceId;
            }

            // Get the column preferences
            sas.modelfactory.util.PreferencesUtil.getColumnPref(controller.project.id, "FactoryMiner.DataColumnPreferences", view.table, callback);

            // Store the target type
            controller.getTargetType();
        };

        var onComplete = function(oJson) {
            // Initial scan of data variables
            setTimeout(function(){controller.processControlCounts();}, 0);
        };

        //projectProxy.getProjectDataVariables(proj.id, handlePreProcessingJson, null, handlePreProcessingComplete);
        // Get the project data variables
        var links = proj.links;
        if (links){
            messageProxy.request(links, "variables", "GET", "json", null, onSuccess, null, onComplete)
        }
    },

    updateVariables: function(varList){
        var controller = this;
        var count = varList.length;
        busySpinner.show(true,
            sas.modelfactory.RB.getText("projectDataView.busyDialog.title.txt"),
            sas.modelfactory.RB.getText("projectDataView.busyDialog.updatingDataVars.txt")
        );

        var onSuccess = function(oJson){

        };

        var onComplete = function(oJson) {
            count--;

            if (count == 0){
                busySpinner.show(false);

                // Update the variable list
                controller.getVariables(controller.project);
            }
        };

        for (var i=0;i<varList.length;i++){
            // Update each variable
            var variable = varList[i];
            //projectProxy.updateProjectDataVariable(controller.projectId, variable.id, variable, onSuccess, null, onComplete);
            setTimeout(function(){onComplete();}, 1000);
        }
    },

    updateAllVariables: function(varList){
        var controller = this;
        busySpinner.show(true,
            sas.modelfactory.RB.getText("projectDataView.busyDialog.title.txt"),
            sas.modelfactory.RB.getText("projectDataView.busyDialog.updatingDataVars.txt")
        );

        var onSuccess = function(oJson){
            controller.getProjectStatus();
        };

        var onError = function(xhr, err, msg){
            var json = xhr.responseJSON;
            var errMsg = (xhr.status?(xhr.status + ":  "):"") + msg + "\n\n" +
                sas.modelfactory.RB.getText("error.errorMore.txt") + "\n" +
                sas.modelfactory.RB.getText("error.errorCode.txt") + " " + (json!==undefined && json.errorCode?json.errorCode:"")  + "\n" +
                sas.modelfactory.RB.getText("error.errorMsg.txt") + " " + (json!==undefined && json.message?json.message:"")  + "\n" +
                sas.modelfactory.RB.getText("error.errorDetails.txt") + " " + (json!==undefined && json.details?json.details:"") + "\n" +
                sas.modelfactory.RB.getText("error.errorUrl.txt") + " " + this.url;
            sas.hc.m.MessageBox.error(errMsg);

            // Update the variable list in case of an error to roll back the change
            controller.getVariables(controller.project);
        };

        var onComplete = function(oJson) {
            busySpinner.show(false);
        };

        /*var data = this.oView.table.getModel().oData.preData;
        for (var i=0;i<varList.length;i++){
            // Update each variable
            var varId = varList[i].id;
            for (var j=0;j<data.length;j++){
                if (data[j].id === varId){
                    data[j] = varList[i];
                }
            }
        }*/

        projectProxy.updateProjectDataVariables(controller.project.id, varList, onSuccess, onError, onComplete);
        logUtil.log("Updated datasource :", varList);
    },

    getTargetType: function()  {
        var view = this.oView;
        var controller = this;

        var variableData = view.table.getModel().getProperty("/preData");
        if (variableData && variableData.length > 0){
            var found = false;
            for (var i=0;i<variableData.length;i++){
                var variable = variableData[i];
                if (variable.role.key === "TARGET"){
                    found = true;
                    var targetType = variable.level;
                    var model = new sap.ui.model.json.JSONModel();
                    model.setData({"level": targetType});
                    sap.ui.getCore().setModel(model, "targetType");
                    logUtil.log("Store target type:", targetType.key);
                    break;
                }
            }
            if (!found){
                // Clear target type
                var model = new sap.ui.model.json.JSONModel();
                model.setData({"level": null});
                sap.ui.getCore().setModel(model, "targetType");
            }
        }
    },

    getProjectStatus: function() {
        var controller = this;

        var onSuccess = function(oJson) {
            controller.buildProfileAvailable = oJson.buildProfileAvailable;
            controller.targetAndSegmentDefined = oJson.targetAndSegmentDefined;
            controller.modelsHaveBeenRun = oJson.modelsHaveBeenRun;
            var links = oJson.links;
            var profileUri = messageProxy.findUri(links, "profile", "GET");
            if (profileUri) {
                controller.profileBuilt = true;
            } else {
                controller.profileBuilt = false;
            }

            sap.ui.getCore().getModel("openProject").setProperty("/buildProfileAvailable", controller.buildProfileAvailable);
            sap.ui.getCore().getModel("openProject").setProperty("/profileBuilt", controller.profileBuilt);

            logUtil.log("publish updated project status:", oJson);
            var data = {
                "buildProfileAvailable": controller.buildProfileAvailable,
                "profileBuilt": controller.profileBuilt
            };
            sas.eventBus.publish("ProjectEvent", "profileState", data);
        };

        var onError = function(xhr, err, msg){
            var json = xhr.responseJSON;
            var errMsg = (xhr.status?(xhr.status + ":  "):"") + msg + "\n\n" +
                sas.modelfactory.RB.getText("error.errorMore.txt") + "\n" +
                sas.modelfactory.RB.getText("error.errorCode.txt") + " " + (json!==undefined && json.errorCode?json.errorCode:"")  + "\n" +
                sas.modelfactory.RB.getText("error.errorMsg.txt") + " " + (json!==undefined && json.message?json.message:"")  + "\n" +
                sas.modelfactory.RB.getText("error.errorDetails.txt") + " " + (json!==undefined && json.details?json.details:"") + "\n" +
                sas.modelfactory.RB.getText("error.errorUrl.txt") + " " + this.url;
            logUtil.error("Error retrieving the buildProfileState:", errMsg);
        };

        var onComplete = function(oJson) {
        };

        // Get the profile build state
        if (controller.project){
            projectProxy.getProjectStatus(controller.project.id, onSuccess, onError, onComplete);
        }
    },

    isTargetOrSegmentChangeAllowed: function(selected) {
        var controller = this;
        var view = this.getView();
        var allowed = true;
        for (var i=0; i<selected.length; i++) {
            var context = view.table.getContextByIndex(selected[i]);
            var selectedData = context.getObject();
            if ((controller.targetAndSegmentDefined) && (this.project.jobsHaveBeenRun/*controller.modelsHaveBeenRun*/) &&  // Should be using modelsHaveBeenRun but
                (selectedData.role.key !== "INPUT" && selectedData.role.key !== "REJECTED")) {                             // not currently working.
                allowed = false;
                break;
            }
        }
        return allowed;
    },

    onColumnMove: function(evt) {
        logUtil.log("Results table column moved:", evt.getParameter("column").sId, "moved to", evt.getParameter("newPos"));

        // A column has moved
        var params = evt.getParameters();
        this.columnsMoved(params);
    },

    columnsMoved: function(params) {
        logUtil.log("Columns changed.  Do something with columns.")
        if (params) {
            var controller = this;
            var colMoved = params.column;
            var pos = params.newPos;
            var table = controller.getView().table;
            var cols = null;
            if (table){
                cols = table.getColumns();
            }

            // Update the columms
            if (table && cols){
                for(var i=0;i<cols.length;i++){
                    if (cols[i].getId() === colMoved.getId()){
                        cols.splice(i, 1);
                        break;
                    }
                }
                cols.splice(pos, 0, colMoved);
            }
            table.removeAllColumns();
            for(var j=0;j<cols.length;j++){
                table.addColumn(cols[j]);
            }

            var storeSuccess = function(args) {
                controller.preferenceId = args.id;
            }

            // Store column preferences
            sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, "FactoryMiner.DataColumnPreferences", "data_", table, controller.preferenceId, storeSuccess);
        }
    },

    onColumnManagerClose: function() {
        var controller = this;

        var storeSuccess = function(args) {
            controller.preferenceId = args.id;
        }

        // Store column preferences
        var table = controller.getView().table;
        sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, "FactoryMiner.DataColumnPreferences", "data_", table, controller.preferenceId, storeSuccess);
    }

});