jQuery.sap.require(
    "sas.modelfactory.proxies.SegmentsProxy",
    "sas.modelfactory.report.BaseGraphController"
);

sas.modelfactory.report.BaseGraphController.extend("sas.modelfactory.project.settings.ProjectHistory", {

    project: null,
    historyReports: null,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf testproject2.testView
     */
	onInit: function() {
        // Call the super
        sas.modelfactory.report.BaseGraphController.prototype.onInit.apply(this);
	},

    onExit: function() {
        logUtil.log("sas.modelfactory.project.settings.ProjectHistory: onExit called");
        var menu = this.getView().tileMenu;
        if (menu) {
            menu.destroyItems();
        }
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf testproject2.testView
     */
	onBeforeRendering: function() {
        var controller = this;
        var view = controller.getView();

        // Get the current open project and depth
        this.depth = "";
        this.project = null;

        var projectModel = sap.ui.getCore().getModel("openProject");
        if (projectModel){
            this.project = projectModel.getProperty("/project");
            if (this.project){
                this.depth = this.project.depth;
            }
        }

        if (this.depth === "")
            this.depth = "10";

        var model = new sap.ui.model.json.JSONModel();
        var viewData = view.getViewData();
        model.setData({models: viewData.items});
        view.setAndBindModel(model, "/models");

        // Generate a graph for the champion criterion for this project
        if (this.project) {
            controller.getHistoryCriterionReports(viewData.segmentId, this.project.championVariable);
        }
    },

    getHistoryCriterionReports: function(segmentId, championVariable) {
        var controller = this;

        var onSuccess = function (reports) {
            controller.historyReports = reports.items;

            if (controller.historyReports.length > 0) {
                var foundI = 0;
                var thisKey;
                if (championVariable) {
                    for (var i = 0; i < controller.historyReports.length; i++) {

                        thisKey = controller.historyReports[i].reportKey;

                        // Try to get the reportKey off the parameter map
                        var parm = controller.historyReports[i].parameterMap["reportKey"];
                        if (parm !== undefined) {
                            thisKey = parm.values[0];
                        }

                        if (thisKey.toUpperCase() === championVariable.key.toUpperCase()) {
                            foundI = i;
                            break;
                        }
                    }
                }

                var thisReport = controller.historyReports[foundI];
                thisReport.menu = controller.getView().tileMenu;
                thisReport.minimize = false;
                thisReport.xInteger = true;

                // Set the reports for rendering
                controller.setReports({items: [thisReport]});
            }
        };

        // Random graph generation - get the summary reports for now
        segmentsProxy.getHistoryGraphsForSegment(this.project.id, segmentId, onSuccess);
    },

    getIndexedHistoryCriterionReport: function(statKey) {
        var controller = this;
        var thisKey;

        for (var i = 0; i < controller.historyReports.length; i++) {

            thisKey = controller.historyReports[i].reportKey;

            // Try to get the reportKey off the parameter map
            var parm = controller.historyReports[i].parameterMap["reportKey"];
            if (parm !== undefined) {
                thisKey = parm.values[0];
            }

            // Check to see if this is the correct report
            if (thisKey.toUpperCase() === statKey.toUpperCase()) {

                // Found it.  Set the menu and turn minimize off
                var thisReport = controller.historyReports[i];
                thisReport.menu = controller.getView().tileMenu;
                thisReport.minimize = false;
                thisReport.xInteger = true;

                // Set the report onto the controller for rendering
                controller.setReports({items: [thisReport]});
                break;
            }
        }
    },

    findStat: function(championArr, statKey) {
        var foundStatI = -1;
        for (var j = 0; j < championArr.length; j++) {
            if (championArr[j].key.toUpperCase() === statKey) {
                foundStatI = j;
                break;
            }
        }

        return foundStatI;
    },

    handleSelect: function(oEvent) {
        var statKey = oEvent.getParameters().id.substring(5);
        this.getIndexedHistoryCriterionReport(statKey);
    }
});