jQuery.sap.require(
    "sas.hc.ui.table.Table",
    "sas.hc.ui.table.Column",
    "sas.hc.ui.commons.TextView",
    "sap.ui.core.Icon",
    "sas.hc.m.SearchField",
    "sas.hc.m.Text",
    "sap.ui.layout.VerticalLayout"
);

sap.ui.jsview("sas.modelfactory.project.settings.ProjectModel", {

    filter: null,
    table: null,
    viewPropBtn: null,
    newTemplateFromAlgorithmBtn: null,
    newTemplateFromGlobalTemplateBtn: null,
    removeModelTemplate: null,
    pageLayout: null,
    msgList: null,

    /** Specifies the Controller belonging to this View.
     * In the case that it is not implemented, or that "null" is returned, this View does not have a Controller.
     * @memberOf sas.modelfactory.project.settings.ProjectModel
     */
    getControllerName : function() {
        return "sas.modelfactory.project.settings.ProjectModel";
    },

    /** Is initially called once after the Controller has been instantiated. It is the place where the UI is constructed.
     * Since the Controller is given to this method, its event handlers can be attached right away.
     * @memberOf sas.modelfactory.project.settings.ProjectModel
     */
    createContent : function(oController) {
        this.addStyleClass("FactoryMiner-ProjectModel");

        // Create control for error messages, but make it invisible until errors are determined
        this.msgList = new sas.hc.m.List({
            noDataText: " ",
            visible: false
        }).addStyleClass("ProjectDataView-warning").addStyleClass("projectSettingsTabs-vLayout");


        this.newTemplateFromAlgorithmBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.NEWSTARBURST,
            tooltip: sas.modelfactory.RB.getText("projectSettingsModel.tooltip.newTemplateFromAlgorithm.txt"),
            enabled: true,
            press: [oController.onNewTemplateFromAlgorithmPress, oController, this]
        }).addStyleClass("segBtn-action");

        this.newTemplateFromGlobalTemplateBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.ADD,
            //text: sas.modelfactory.RB.getText("projectSettingsModel.Button.newTemplateFromGlobalTemplate.txt"),
            tooltip: sas.modelfactory.RB.getText("projectSettingsModel.tooltip.newTemplateFromGlobalTemplate.txt"),
            enabled: true,
            press: [oController.onAddTemplateFromGlobalTemplatePress, oController, this]
        }).addStyleClass("segBtn-action");

        this.viewPropBtn = new sas.hc.m.Button({
            //text: sas.modelfactory.RB.getText("projectSettingsModel.button.viewProp.txt"),
            icon: sas.icons.HC.EDIT,
            tooltip: sas.modelfactory.RB.getText("projectSettingsModel.tooltip.viewProp.txt"),
            enabled: false,
            press: [oController.onViewPropertiesPress, oController, this]
        }).addStyleClass("segBtn-action");

        this.removeModelTemplate = new sas.hc.m.Button({
            //text: sas.modelfactory.RB.getText("projectSettingsModel.button.viewProp.txt"),
            icon: sas.icons.HC.DELETE,
            tooltip: sas.modelfactory.RB.getText("projectSettingsModel.tooltip.removeModelTemplate.txt"),
            enabled: false,
            press: [oController.onRemoveModelTemplatePress, oController, this]
        }).addStyleClass("segBtn-action");

        this.filter = new sas.hc.m.SearchField({
            type: sap.m.InputType.Text,
            placeholder: sas.modelfactory.RB.getText("projectProfileView.filter.placeholder.txt")
        }).addStyleClass("modelFactoryMain-filter");

        var buttonHBox = new sap.ui.layout.HorizontalLayout(this.createId("ProjectModelButtonLayout"), {
            content: [
                this.newTemplateFromAlgorithmBtn,
                this.newTemplateFromGlobalTemplateBtn,
                this.viewPropBtn,
                this.removeModelTemplate
            ]/*,
             width: "100%"*/
        });

        var horizontalLayout = new sap.ui.layout.HorizontalLayout(this.createId("ProjectModelHorizontalLayout"), {
            content: [
                this.filter,
                buttonHBox
            ],
            width: "100%"
        }).addStyleClass("modelFactoryMain-tableHBox");

        this.table = new sas.hc.ui.table.Table({
            id: this.createId("ProjectModelTemplateTable"),
            editable: false,
            //visibleRowCount: 15,
            visibleRowCountMode: "Auto",
            minAutoRowCount: 1,
            selectionBehavior: sap.ui.table.SelectionBehavior.Row,
            selectionMode: sap.ui.table.SelectionMode.Multi,
            rowSelectionChange: [oController.handleMetadataRowSelectionChange, oController],
            columnMove: [oController.onColumnMove, oController]
        });

        var nameCol = new sas.hc.ui.table.Column(this.createId("modTemp_name"), {
            label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectSettingsModel.column.name.txt")}),
            /*template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                path:"displayName",
                formatter:function(name) {
                    return sas.modelfactory.RB.getText(name);
                }
            }),*/
            template: new sas.hc.m.Link({
                text: "{displayName}",
                target: "_self",
                press: function(evt) {
                    oController.onLinkPress.call(oController, this);
                }
            }).bindProperty("enabled",{
                path: "miningAlgorithm/internalName",
                formatter: function(name) {
                    return name !== "Baseline";
                }
            }).data("id", "{id}"),
            sortProperty: "displayName",
            filterProperty: "displayName"});
        this.table.addColumn(nameCol);
        this.table.addColumn(new sas.hc.ui.table.Column(this.createId("modTemp_algorithm"), {
            label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectSettingsModel.column.algorithm.txt")}),
            template: new sas.hc.ui.commons.TextView().bindProperty("text", {
                path:"miningAlgorithm/displayName"
//                formatter:function(algorithm) {
//                    return sas.modelfactory.RB.getText(algorithm);
//                }
            }),
            sortProperty: "miningAlgorithm/displayName",
            filterProperty: "miningAlgorithm/displayName"}));
        this.table.addColumn(new sas.hc.ui.table.Column(this.createId("modTemp_owner"), {
            label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectSettingsModel.column.owner.txt")}),
            template: new sas.hc.ui.commons.TextView({text: "{createdBy}"})}));
        this.table.addColumn(new sas.hc.ui.table.Column(this.createId("modTemp_modified"), {
            label: new sas.hc.m.Text({text: sas.modelfactory.RB.getText("projectSettingsModel.column.modified.txt")}),
            template: new sas.hc.ui.commons.TextView({
                text: {
                    path: "modifiedTimestamp",
                    formatter: function(value) {
                        return sas.modelfactory.util.FormatUtil.getElapsedTimeLabel(value);
                    }
                }
            })}));

        this.filter.attachLiveChange(function(oEvent) {
            this.table.filter(nameCol, this.filter.getValue());
        }, this);

        this.pageLayout = new sas.hc.ui.commons.layout.BorderLayout({
            width: "100%",
            height: "100%",
            top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                width: "100%",
                size: "48px",
                content: [this.msgList,horizontalLayout]
            }),
            center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                width: "100%",
                size: "100%",
                content: [this.table]
            })
        }).addStyleClass("projectDataView-borderLayout");

        return this.pageLayout;
    },

    setAndBindModel: function(model, bindPath) {
        this.table.setModel(model);
        this.table.bindRows(bindPath);

        //Initially sort the table
        this.table.sort(this.table.getColumns()[0]);
    }
});