/**
 * Created by rayoun on 6/11/14.
 */

(function(){

    jQuery.sap.declare("sas.modelfactory.proxies.AbstractProxy");

    jQuery.sap.require(
        "sas.hc.m.MessageBox",
        "sas.modelfactory.util.LogUtil"
    );

    abstractProxy = {

        makeRequest: function(options, successCallback, errorCallback, completeCallback){
            options.cache = false;
            if (options.sas !== undefined){         // This is a work-around as laid out in S1161879 for S1166960.
                options.sas.authenticate = true;
            } else {
                options.sas = {authenticate: true};
            }
            options.success = function(){
                if (successCallback){
                    successCallback.apply(this, arguments);
                }
            };
            options.error = function(xhr, err, msg) {

                // S1177414: Sense when a user doesn't have "Factory Miner: User" authority
                if (xhr.status === 403) {
                    var json = xhr.responseJSON;
                    var errMsg = sas.modelfactory.RB.getText("login.not.authorized.txt");
                    if (json !== undefined && json && json.message){
                        errMsg = json.message;
                    }
                    sas.hc.m.MessageBox.error(errMsg, sas.modelfactory.RB.getText("login.not.authorized.title.txt"),
                        sas.modelfactory.RB.getText("login.not.authorized.signout.txt"),
                        "confirmWaitDialog",
                        function(oAction) {
                            // Force the user to sign off
                            sas.applicationSwitcher.logoff();
                        }
                    );
                } else {
                    if (errorCallback){
                        errorCallback.apply(this, arguments);
                    } else {
                        // Default error message
                        var title = sas.modelfactory.RB.getText("projectView.errorMsg.title.txt");
                        var json = xhr.responseJSON;

                        // We may have received json as text.  Test it to see.
                        if (json === undefined){
                            try {
                                if (xhr.responseText){
                                    json = JSON.parse(xhr.responseText);
                                }
                            }
                            catch(err) {
                                logUtil.log("WARNING! abstractProxy trying to parse response that is not JSON");
                            }
                        }
                        var moreMsg = "";
                        if (json !== undefined){
                            moreMsg = sas.modelfactory.RB.getText("error.errorCode.txt") + " " + (json.errorCode?json.errorCode:"")  + "\n" +
                                sas.modelfactory.RB.getText("error.errorMsg.txt") + " " + (json.message?json.message:"")  + "\n" +
                                sas.modelfactory.RB.getText("error.errorDetails.txt") + " " + (json.details?json.details:"") + "\n";
                        }
                        var errMsg = (xhr.status?(xhr.status + ":  "):"") + msg + "\n\n" +
                            sas.modelfactory.RB.getText("error.errorMore.txt") + "\n" +
                            (json!==undefined?moreMsg:"") +
                            sas.modelfactory.RB.getText("error.errorUrl.txt") + " " + this.url;
                        sas.hc.m.MessageBox.error(errMsg, title);
                    }
                }
            };
            options.complete = function(){
                if (completeCallback){
                    completeCallback.apply(this, arguments);
                }
            };

            sas.ajax(options);
        }
    };
})();