/**
 * Created by rayoun on 5/29/14.
 */

jQuery.sap.require(
    "sas.modelfactory.Config",
    "sas.modelfactory.proxies.AbstractProxy",
    "sas.modelfactory.util.LogUtil"
);

(function(){

    jQuery.sap.declare('sas.modelfactory.proxies.DatasourceProxy');

    datasourceProxy = {
        getDatasources: function(onSuccess, onError, complete) {

            var url = sas.modelfactory.Config.serverUrl + "/datasources";
            logUtil.log(url);

            var options = {
                url:  url,
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        createDatasource: function(datasource, onSuccess, onError, complete) {

            var url = sas.modelfactory.Config.serverUrl + "/datasources";

            var options = {
                url:  url,
                type: 'POST',
                headers: {'Content-Type': 'application/vnd.sas.factoryminer.datasource.library.import.options+json' },
                data: datasource,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        deleteDatasource: function(dsId, onSuccess, onError, complete) {

            var url = sas.modelfactory.Config.serverUrl + "/datasources/" + dsId;

            var options = {
                url:  url,
                type: 'DELETE',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        getLibrary: function(tableId, onSuccess, onError, complete) {
            // Note that the object type value for Table content is 32.
            var id = tableId.replace("/", "-");

            var tableServiceUrl = sas.registry.wip.getTableService().url;
            // For S1183379, use an alternate URL to avoids the 303 redirect
            var newURL = tableServiceUrl.replace("{object-id}", id);
            var url = newURL.replace("32", "tables");
            url = url + "/library";

            var options = {
                url:  url,
                type: 'GET',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        getTable: function(tableId, onSuccess, onError, complete) {
            // Note that the object type value for Table content is 32.
            var id = tableId.replace("/", "-");

            var tableServiceUrl = sas.registry.wip.getTableService().url;
            var url = tableServiceUrl.replace("{object-id}", id);

            var options = {
                url:  url,
                type: 'GET',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        getDatasourceVariables: function(id, onSuccess, onError, complete) {

            var url = sas.modelfactory.Config.serverUrl + "/datasources/" + id + "/variables";

            var options = {
                url:  url,
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        }
    };
})();