/**
 * Created by rayoun on 7/8/14.
 */

jQuery.sap.require(
    "sas.modelfactory.Config",
    "sas.modelfactory.proxies.AbstractProxy",
    "sas.modelfactory.util.LogUtil"
);

(function(){

    jQuery.sap.declare('sas.modelfactory.proxies.PreferencesProxy');

    prefProxy = {
        getPreferences: function(prefs, onSuccess, onError, onComplete) {
            var modelfactoryConfig = sas.config.get("modelfactory");
            var userid = modelfactoryConfig.userId;

            var prefServiceUrl = sas.registry.wip.getPreferenceService().url;
            var url = prefServiceUrl.replace("{user-id}", encodeURIComponent(userid));

            var options = {
                url: url,
                type: 'GET',
                headers: {'Accept': 'application/json' },
                dataType: 'json',
                data: prefs
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        setPreferences: function (prefs, onSuccess, onError, onComplete) {
            var modelfactoryConfig = sas.config.get("modelfactory");
            var userid = modelfactoryConfig.userId;

            var prefServiceUrl = sas.registry.wip.getPreferenceService().url;
            var url = prefServiceUrl.replace("{user-id}", encodeURIComponent(userid));

            var options = {
                url: url,
                type: 'PUT',
                headers: {'Accept': 'application/json' },
                data: prefs,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)

        },

        getTablePreferences: function(projId, onSuccess, onError, onComplete) {
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/preferences",
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        setTablePreference: function (projId, preferenceId, prefs, onSuccess, onError, onComplete) {
            var options;
            if (preferenceId !== null) {
                options = {
                    url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/preferences/" + preferenceId,
                    type: 'PUT',
                    headers: {'Accept': 'application/json', 'Content-Type': 'application/json' },
                    data: prefs,
                    dataType: 'json'
                };
            }
            else {
                options = {
                    url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/preferences",
                    type: 'POST',
                    headers: {'Content-Type': 'application/json' },
                    data: prefs,
                    dataType: 'json'
                };
            }

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        deleteAllTablePreferences: function(projId, key, onSuccess, onError, onComplete) {
            logUtil.log("Delete column preferences for projectId:", projId, "on key:", key, "for all users.");
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/preferences?key=" + key + "&forAllUsers=true",
                type: 'DELETE',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        deleteTablePreference: function(projId, prefId, onSuccess, onError, onComplete) {
            logUtil.log("Delete column preference:", prefId);
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/preferences/" + prefId,
                type: 'DELETE',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        }
    };
})();