jQuery.sap.require(
    "sas.modelfactory.Config",
    "sas.modelfactory.proxies.AbstractProxy"
);

/**
 * Created by rayoun on 4/29/14.
 */

(function(){

    jQuery.sap.declare('sas.modelfactory.proxies.ProjectProxy');

    projectProxy = {
        getProjects: function(onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects",
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        createProject: function(params, onSuccess, onError, onComplete){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects",
                headers: {'Content-Type': 'application/vnd.sas.factoryminer.project+json' },
                type: 'POST',
                data: params,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        getProject: function(projId, onSuccess, onError, onComplete){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId,
                headers: {'Accept': 'application/vnd.sas.factoryminer.project+json' },
                type: 'GET',
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        updateProject: function(projId, params, onSuccess, onError, onComplete){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId,
                headers: {'Content': 'application/vnd.sas.factoryminer.project+json' },
                type: 'PUT',
                data: params,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        deleteProject: function(projId, onSuccess, onError, onComplete){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId,
                headers: {'Accept': 'application/json' },
                type: 'DELETE',
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        lockProject: function(projId, onSuccess, onError, onComplete, error){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/lock",
                headers: {'Accept': 'application/json', 'Content-Type': 'application/json' },
                type: 'PUT',
                data: "true",
                dataType: 'text'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete, error)
        },

        unlockProject: function(projId, onSuccess, onError, onComplete){
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/lock",
                headers: {'Accept': 'application/json', 'Content-Type': 'application/json' },
                type: 'PUT',
                data: "false",
                dataType: 'text'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        breakLock: function(projId, onSuccess, onError, onComplete){

            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/lock",
                headers: {'Accept': 'application/json', 'Content-Type': 'application/json' },
                type: 'PUT',
                data: "override",
                dataType: 'text'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        createProjectDataVariables: function(projId, onSuccess, onError, onComplete) {

            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/variables",
                type: 'POST',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        getProjectDataVariables: function(projId, onSuccess, onError, onComplete) {

            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/variables",
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        updateProjectDataVariables: function(projId, varList, onSuccess, onError, onComplete) {
            var json = JSON.stringify(varList);
            //logUtil.log(json);

            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/variables",
                type: 'PUT',
                headers: {'Content-Type': 'application/json'},
                data: json,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        updateProjectDataVariable: function(projId, varId, variable, onSuccess, onError, onComplete) {
            var json = JSON.stringify(variable);

            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/variables/" + varId,
                type: 'PUT',
                headers: {'Accept': 'application/json', 'Content-Type': 'application/vnd.sas.factoryminer.project.variable+json'},
                data: json,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        updateProjectProfileExcludeList: function(projId, profileId, excludeList, onSuccess, onError, onComplete) {
            var json = JSON.stringify(excludeList);
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/profile/" + profileId + "/segmentExcludeList",
                type: 'PUT',
                headers: {'Accept': 'application/json', 'Content-Type': 'application/vnd.sas.collection+json'},
                data: json,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        getProjectProfiles: function(projId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/profile",
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        getProjectProfile: function(projId, profileId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/profile/" + profileId,
                type: 'GET',
                headers: sas.modelfactory.Config.headersAccept,
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        buildProjectProfile: function(projId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/profile",
                type: 'POST',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        runProjectModels: function (projId, onSuccess, onError, onComplete) {
            var options = {
                //url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/models/results",
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/jobs",
                type: 'POST',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete)
        },

        getProjectJobs: function (projId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projId + "/jobs",
                type: 'GET',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, complete)
        },

        retrainProject: function (projectId, params, onSuccess, onError, onComplete) {
            // http://{{host}}/{{uri}}/projects/{{projectId}}/retrain?libname=fmweblig&memberName=hmeq_2
            // params = libname, memberName
            var options = {
                url:  sas.modelfactory.Config.serverUrl + "/projects/" + projectId + "/retrain?libname=" + params.libname + "&memberName=" + params.memberName,
                headers: {'Content': 'application/vnd.sas.factoryminer.project+json' },
                type: 'PUT',
                data: params,
                dataType: 'json'
            };

            abstractProxy.makeRequest(options, onSuccess, onError, onComplete);
        },

        registerProject: function( projectId, params, onSuccess, onError, onComplete) {
            var options = {
                //url: sas.modelfactory.Config.serverUrl + "/projects/" + projectId + "/registrationStatus?mock=true",
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projectId + "/registrationStatus",
                type: 'PUT',
                data: params,
                headers: {'Accept': 'application/vnd.sas.modelmanager.folder+json',
                         'Content': 'application/json'},
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete);
        },

        cancelRegisterProject: function( projectId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projectId + "/registrationStatus",
                type: 'DELETE'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete);
        },

        getProjectStatus: function( projectId, onSuccess, onError, onComplete) {
            var options = {
                url: sas.modelfactory.Config.serverUrl + "/projects/" + projectId + "/status",
                type: 'GET',
                headers: {'Accept': 'application/json' },
                dataType: 'json'
            };
            abstractProxy.makeRequest(options, onSuccess, onError, onComplete);
        }
    };
})();