jQuery.sap.require(
    "sas.ltjs.ltjsLoader",
    "sas.modelfactory.util.ModelUtil",
    "sas.modelfactory.util.SpinnerUtil"
);

sap.ui.core.mvc.Controller.extend("sas.modelfactory.report.BaseGraphController", {

//    updateCalled: 1,
    defaultSortedOrderByAlgorithm: {
        "Forest": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "ITERATIONPLOT", "ROC", "MISCLASSIFICATION", "VARIABLEIMPORTANCE", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "Regression": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "TVALUES", "ROC", "MISCLASSIFICATION", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "NeuralNetwork": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "NETWORKDIAGRAM", "ROC", "ITERATIONPLOT", "MISCLASSIFICATION", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "DecisionTree": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "TREEPLOT", "ROC", "RULES", "MISCLASSIFICATION", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "GLM": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "TVALUES", "ROC", "MISCLASSIFICATION", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "SVM": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "ROC", "MISCLASSIFICATION", "SVMFITSTATISTICS", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "Boost": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "SUBSERIESPLOT", "ROC", "MISCLASSIFICATION", "VARIABLEIMPORTANCE", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "BayesianNetwork": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "ROC", "MISCLASSIFICATION", "NETWORKSUMMARY", "VARIABLESELECTION", "SCOREINPUTS", "FITSTATISTICS"
        ],
        "Baseline": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "MISCLASSIFICATION", "ROC", "FITSTATISTICS"
        ],
        "Comparison": [
            "LIFT", "PREDICTED", "PREDICTEDBYDEPTH", "ROC", "SCOREINPUTS", "FITSTATISTICS", "FLOW"
        ]
    },

    onInit: function() {
        this.getView().setModel(new sap.ui.model.json.JSONModel());
    },

    setReports: function(reports, algorithmId) {
        var view = this.getView();
        var controller = this;

        // setup model
        var sortedReports = [];

        if (reports && reports.items && reports.items.length > 0) {

            // Don't bother sorting the reports if there is just one
            if (reports.items.length === 1) {
                sortedReports.push(reports.items[0]);
            }
            else {
                // Order the list of reports according to intelligent defaults by algorithm name
                if (algorithmId !== undefined) {
                    var algorithmName = sas.modelfactory.util.ModelUtil.getAlgorithmName(algorithmId);
                    reports.items = controller.reorderReports(algorithmName, reports.items);
                }

                // Add reports to model
                for (var i = 0; i < reports.items.length; i++) {
                    var item = reports.items[i];
                    item.name = item.description;
                    sortedReports.push(item);
                }
            }
        }

        // Remove the spinner because it is colliding with other spinners (i.e. cancel run request).
        //busySpinner.show(true, sas.modelfactory.RB.getText("projectView.busyDialog.initGraphs.txt"), "");

        sas.ltjs.ltjsLoader.runWhenInitialized(function () {

            controller.updateReports(view, sortedReports);

            //busySpinner.show(false);
        });
    },

    updateReports: function(view, sortedReports) {

//        console.log("** updateReports call # " + this.updateCalled + " / HEAP = " + sas.ltjs.ltjs.HEAP16.byteLength);
//        this.updateCalled++;
//        var size = sas.ltjs.ltjs.HEAP16.length;
//        for (; size >= 0; size--) {
//            if (sas.ltjs.ltjs.HEAP16[size-1] !== 0)
//                break;
//        }
//
//		var graphTypes = [];
//        for (var i = 0; i < sortedReports.length; i++) {
//			if (sortedReports[i].type !== "Table")
//				graphTypes.push(sortedReports[i].type);
//		}
//
//        console.log("** updateReports - Adding graphs: " + graphTypes);
//        console.log("** updateReports - High water BEFORE update: " + size);

        this.destroyTiles(view, sortedReports);

        // Set the "reports" binding on the model to nothing first, if this is a single report
        // Otherwise, if the report is the same as last time, it doesn't get rerendered
        if (sortedReports.length === 1)
            view.getModel().setProperty("/reports", []);

        view.getModel().setProperty("/reports", sortedReports);
    },

    destroyTiles: function(view, sortedReports) {
        // Destroy all old tiles that exist for this controller
        if (view.graphGrid !== undefined) {
            var tiles = view.graphGrid.getContent();

            // If tiles exist, destroy them
            if (tiles.length > 0) {
                view.graphGrid.removeAllContent();
                for (var i = 0; i < tiles.length; i++) {
                    tiles[i].destroy(this);
                }

                // S1172731 - Explicitly call invalidate on the view to signal graph code to redraw graphs.  Also
                // added check != 1, as always calling invalidate breaks graph selector on project history dialog.
                if (sortedReports.length != 1)
                    view.invalidate();
            }
        }
    },

    reorderReports: function(algorithmName, reports) {
        var reportMap = {};
        var unkeyedArr = [];

        // Iterate over the reports, creating a keyed map entry to each report
        for (var i = 0; i < reports.length; i++) {
            var key = this.getReportKey(reports[i]);
            if (key !== null) {

                // Make sure all reportKeys are unique
                var j = 2;
                while (reportMap[key] !== undefined) {
                    key = key+j;
                    j++;
                }

                reportMap[key] = reports[i];
            }
            // Report w/o keys get put here for later
            else
                unkeyedArr.push(reports[i]);
        }

        var newReports = [];
        var arr = this.defaultSortedOrderByAlgorithm[algorithmName];

        // If we found a sorted list of report keys for this particular algorithm
        if (arr) {
            // Order the reports by key from the sorted list
            for (var i = 0; i < arr.length; i++) {
                var report = reportMap[arr[i]];
                if (report !== undefined)
                    newReports.push(report);

                // Delete each report from the map, so we can tell if we have leftovers
                delete reportMap[arr[i]];
            }

            // Add additional reports that don't have keys at the bottom of the list
            for (var i = 0; i < unkeyedArr.length; i++) {
                newReports.push(unkeyedArr[i]);
            }

            // And just in case we got reports with keys that we weren't expecting, add them last
            for (var unknownReport in reportMap) {
                newReports.push(reportMap[unknownReport])
            }
        }

        // Otherwise, the order remains the same
        else
            newReports = reports;

        return newReports;
    },

    getReportKey: function(report) {
        var parm = report.parameterMap["reportKey"];
        if (parm !== undefined) {
            return parm.values[0];
        }
        else
            return null;
    },

    onOpenItem: function(evt, item) {
        item.minimize = false;

        jQuery.sap.require("sas.modelfactory.report.ExpandedGraphDialog");

        var dialog = new sas.modelfactory.report.ExpandedGraphDialog({
            content: sap.ui.view({
                type: sap.ui.core.mvc.ViewType.JS,
                viewName: "sas.modelfactory.report.ExpandedGraph",
                viewData: {items: [item]}
            }),
            afterClose: function() {
                dialog.destroy();
            },
            contentWidth: "100%",
            contentHeight: "100%"
        });

        var title = item.description;
        if (!title || title.title === 0)
            title = item.name;
        dialog.setTitle(title);

        // Due to a timing issue with Chrome, delay the dialog open.  This is to correct an issue
        // with the column headings in the tables not being aligned properly with the columns.
        setTimeout(function(){dialog.open();}, 0);
    }

});