jQuery.sap.declare("sas.modelfactory.report.ReportNetworkDiagram");

(function() {
    jQuery.sap.require("sas.modelfactory.report.Report");
    jQuery.sap.require("sas.modelfactory.report.ReportParameter");

    sas.modelfactory.report.Report.extend("sas.modelfactory.report.ReportNetworkDiagram", {

        nodeId: null,
        nodeLabel: null,
        x: null,
        y: null,

        linkId: null,
        fromId: null,
        toId: null,
        linkColorResponse: null,

        nodeArrangement: null,
        nodeTip: null,

        type: null,

        constructor: function(jsonObject) {
            var Report = sas.modelfactory.report.Report;
            var ReportParameter = sas.modelfactory.report.ReportParameter;
            Report.prototype.constructor.apply(this, arguments);

            // Sort the data points according to the provided rowNumber value
            var dataSets = this.getReportDataSets();
            if (dataSets) {
                for (var i = 0; i < dataSets.length; i++) {
                    var dataSet = dataSets[i];
                    if (dataSet.data.length > 0 && dataSet.data[0].rowNumber)
                        dataSet.data.sort(this.getSortMethod("rowNumber"));
                }
            }

            this.linkId = this.getParameterByName("linkId");
            this.fromId = this.getParameterByName("fromId");
            this.toId = this.getParameterByName("toId");
            this.linkColorResponse = this.getParameterByName("linkColorResponse");

            this.nodeId = this.getParameterByName("nodeId");
            this.nodeLabel = this.getParameterByName("nodeLabel");
            this.x = this.getParameterByName("x");
            this.y = this.getParameterByName("y");

            this.nodeArrangement = this.getParameterByName("nodeArrangement");
            this.nodeTip = this.getParameterByName("nodeTip");

            // Need to dummy the type parameter since it is not passed
            var typeObject = { parameter: "type", type: "String", values: ["dataType"], label: null};
            this.type = new ReportParameter(typeObject);
        },

        getLinkId: function() {
            return this.linkId;
        },

        getFromId: function() {
            return this.fromId;
        },

        getToId: function() {
            return this.toId;
        },

        getLinkColorResponse: function() {
            return this.linkColorResponse;
        },

        getNodeId: function() {
            return this.nodeId;
        },

        getNodeLabel: function() {
            return this.nodeLabel;
        },

        getX: function() {
            return this.x;
        },

        getY: function() {
            return this.y;
        },

        getNodeArrangement: function() {
            return this.nodeArrangement;
        },

        getNodeTip: function() {
            return this.nodeTip;
        },

        getType: function() {
            return this.type;
        },

        getSortMethod: function(){
            var _args = Array.prototype.slice.call(arguments);
            return function(a, b) {
                for (var x in _args) {
                    var ax = a[_args[x]];
                    var bx = b[_args[x]];

                    ax = ax / 1;
                    bx = bx / 1;

                    if (ax !== bx) {
                        return ax < bx ? -1 : 1;
                    }
                }
            }
        }

    });
})();