jQuery.sap.declare("sas.modelfactory.report.ReportTable");

(function() {
    jQuery.sap.require("sas.modelfactory.report.Report");

    sas.modelfactory.report.Report.extend("sas.modelfactory.report.ReportTable", {

        constructor: function(jsonObject) {
            var Report = sas.modelfactory.report.Report;
            Report.prototype.constructor.apply(this, arguments);

            // Sort the data points according to the provided rowNumber value
            var dataSets = this.getReportDataSets();
            if (dataSets) {
                for (var i = 0; i < dataSets.length; i++) {
                    var dataSet = dataSets[i];
                    if (dataSet.data.length > 0 && dataSet.data[0].rowNumber)
                        dataSet.data.sort(this.getSortMethod("rowNumber"));
                }
            }

            // Sort the parameters according to the provided order value
            var parameters = this.getReportParameters();
            if (parameters.length > 0 && parameters[0].order) {
                parameters.sort(this.getSortMethod("order"));
            }
        },

        // Parameter Estimates Table
        getTvalue: function() {
            return this.getParameterByName("tvalue");
        },

        getProbChiSq: function() {
            return this.getParameterByName("ProbChiSq");
        },

        getEstimate: function() {
            return this.getParameterByName("Estimate");
        },

        getSign: function() {
            return this.getParameterByName("Sign");
        },

        getParameter: function() {
            return this.getParameterByName("Parameter");
        },

        getEffect: function() {
            return this.getParameterByName("Effect");
        },

        getAbsEstimate: function() {
            return this.getParameterByName("AbsEstimate");
        },

        // Scoring Inputs Table
        getRelativeImportance: function() {
            return this.getParameterByName("RelativeImportance");
        },

        getRank: function() {
            return this.getParameterByName("Rank");
        },

        getName: function() {
            return this.getParameterByName("Name");
        },

        // Fit Statistics Table
        getTarget: function() {
            return this.getParameterByName("TARGET");
        },

        getValidate: function() {
            return this.getParameterByName("VALIDATE");
        },

        getTrain: function() {
            return this.getParameterByName("TRAIN");
        },

        getTest: function() {
            return this.getParameterByName("TEST");
        },

        getLabel: function() {
            return this.getParameterByName("LABEL");
        },

        getStat: function() {
            return this.getParameterByName("STAT");
        },

        // Flow Table
        getComponent: function() {
            return this.getParameterByName("Component");
        },

        getPropertyName: function() {
            return this.getParameterByName("PropertyName");
        },

        getPropertyValue: function() {
            return this.getParameterByName("PropertyValue");
        },

        // Variable Importantance Table
        getGini: function() {
            return this.getParameterByName("Gini");
        },

        getGiniOOB: function() {
            return this.getParameterByName("GiniOOB");
        },

        getGiniValid: function() {
            return this.getParameterByName("GiniValid");
        },

        getMargin: function() {
            return this.getParameterByName("Margin");
        },

        getMarginOOB: function() {
            return this.getParameterByName("MarginOOB");
        },

        getMarginValid: function() {
            return this.getParameterByName("MarginValid");
        },

        getNRules: function() {
            return this.getParameterByName("NRules");
        },

        getVariable: function() {
            return this.getParameterByName("Variable");
        },

        // Model Comparison
        getComparison: function() {
            return this.getParameterByName("comparison");
        },

        getSortMethod: function(){
            var _args = Array.prototype.slice.call(arguments);
            return function(a, b) {
                for (var x in _args) {
                    var ax = a[_args[x]];
                    var bx = b[_args[x]];

                    ax = ax / 1;
                    bx = bx / 1;

                    if (ax !== bx) {
                        return ax < bx ? -1 : 1;
                    }
                }
            }
        }
    });
})();