sap.ui.core.Control.extend("sas.modelfactory.report.ReportTile", {

    // --------------------------------------------------------------------
    //    IMPORTANT: this class assumes that the items in the view
    //    are populated via data binding.
    // --------------------------------------------------------------------

    metadata: {
        properties: {
            "showExpandButton": {type: "boolean", defaultValue: false},
            "showMenuButton": {type: "boolean", defaultValue: false},
            "showHeader": {type: "boolean", defaultValue: true},
            "fillWidth": {type: "boolean", defaultValue: true},
            "menu": {type: "object", defaultValue: null},
            "name": {type: "string", defaultValue: null}
        },
        aggregations: {
            "image": {type: "sap.ui.core.Control", multiple: false},
            "_header": {type: "sap.ui.core.Control", multiple: false, visibility: "hidden"}
        }
    },

    init: function() {
        this.addStyleClass("fm_reportTile");

        this.nameText = new sas.hc.m.Label({
            width: "100%",
            layoutData: new sap.m.FlexItemData({growFactor: 1})
        }).addStyleClass("fm_reportTile_nameText");

        this.menuButton = new sas.hc.ui.commons.MenuButton({
            visible: false,
            icon: sas.icons.HC.SHOWACTIONSBUTTON,
            tooltip: sas.modelfactory.RB.getText("projectHistory.selector.tip.txt")
        });

        this.expandButton = new sas.hc.m.Button({
            visible: false,
            icon: sas.icons.HC.MAXIMIZEVIEW,
            tooltip: sas.modelfactory.RB.getText("projectResults.report.expand.txt")
        }).addStyleClass("sapUiMenuButton");

        this.header = new sas.hc.m.HBox({
            fitContainer: true,
            items: [this.nameText, this.expandButton, this.menuButton]
        });
        this.setAggregation("_header", this.header);
    },

    destroy: function(oController) {
        // Destroy the image and remove the aggregation
        if (this.getImage && this.getImage()) {
            var image = this.getImage();
            this.setAggregation("image", null);
            image.destroy();
        }

        // Detach the expand button handler
        if (oController) {
            var expandButton = this.getExpandButton()
            if (expandButton) {
                expandButton.detachPress(oController.onOpenItem);
                expandButton.destroy();
            }
            this.expandButton = null;

            this.menuButton.setMenu(null);
            if (this.header) {
                this.setAggregation("_header", null);
                this.header.destroy();
                this.header = null;
            }
        }
    },

    setShowHeader: function(value) {
        this.setProperty("showHeader", value);
        if (!value) {
            this.removeStyleClass("fm_reportTile");
            this.header.destroy();
            this.header = null;
        }
    },

    setShowExpandButton: function(value) {
        this.setProperty("showExpandButton", value);
        this.expandButton.setVisible(value);
    },

    getExpandButton: function() {
        return this.expandButton;
    },

    setShowMenuButton: function(value) {
        this.setProperty("showMenuButton", value);
        this.menuButton.setVisible(value);
    },

    setMenu: function(value) {
        this.setProperty("menu", value);
        this.menuButton.setMenu(value);
    },

    getMenu: function() {
        return this.getProperty("menu");
    },

    setName: function(value) {
        this.setProperty("name", value);
        this.nameText.setText(value);
    }

});
