jQuery.sap.declare("sas.modelfactory.report.TableReport");
(function() {

    jQuery.sap.require("sap.m.FlexBox");
    jQuery.sap.require("sap.m.Text");
    jQuery.sap.require("sas.modelfactory.report.RelativeImportance");
//    jQuery.sap.require("sas.modelfactory.report.RelativeImportanceGraph");
    jQuery.sap.require("sap.ui.table.VisibleRowCountMode");
    jQuery.sap.require("sas.hc.ui.table.Column");
    jQuery.sap.require("sas.hc.ui.table.Table");
    jQuery.sap.require("sas.modelfactory.report.ReportTable");

    var FlexBox = sap.m.FlexBox;
    var RelativeImportance = sas.modelfactory.report.RelativeImportance;
//    var RelativeImportanceGraph = sas.modelfactory.report.RelativeImportanceGraph;
    var Column = sas.hc.ui.table.Column;
    var Table = sas.hc.ui.table.Table;
    var ReportTable = sas.modelfactory.report.ReportTable;

    FlexBox.extend("sas.modelfactory.report.TableReport", {

        reportTable: null,
        haveTestingData: false,
        haveValidationData: false,
        haveTrainingData: false,
        modelComparisonScoringInputs: false,

        constructor: function(jsonObject) {
            this.reportTable = new ReportTable(jsonObject);

            FlexBox.call(this);
        },

        renderer: "sap.m.FlexBoxRenderer",

        init: function () {
            FlexBox.prototype.init.apply(this);

            var modelComparison	= false;
            var comparison = this.reportTable.getComparison();
            if (comparison && comparison.values[0] !== null && comparison.values[0].length > 0)
                modelComparison	= true;

            // Initialization
            var dgcArrayCol = [];
            var modelData = [];
            var sortField = null;
            var propertyNameLookup = {};
            var collectionOfModelObjectsForModelComparison = [];
            var updatedCollectionOfModelObjects = [];
            var table = new Table();
            table.setShowTableOptions(false);
            this.modelComparisonScoringInputs = false;

            // Loop through the data points keeping track of the selected models and manipulating the data based on which type of table we are generating
            var reportDataSets = this.reportTable.getReportDataSets();
            for (var i = 0; i < reportDataSets.length; i++) {
                var reportData = reportDataSets[i].data;

                if (reportData !== undefined && reportData && reportData.length > 0){
                    // For the first data point of a Fit Statistics table, determine what kind of data we have to work with
                    var stat = this.reportTable.getStat();
                    if (stat && stat.values[0] !== null) {
                        this.determineDataForDataPoint(reportData[0]);
                    }

                    // If it is not a Model Comparison, just add all the data as is
                    if (!modelComparison) {
                        modelData = reportData;
                    }
                    // For a Model Comparison, we need to keep track of the number of selected models and manipulate the data based on which table we are generating for each data point
                    else {
                        for (var j = 0; j < reportData.length; j++) {
                            var reportDataPoint = reportData[j];

                            // Create some objects to keep track of the number of selected models and for use when displaying the columns
                            var modelId = reportDataPoint[comparison.values[0]].replace(" ", "_");
                            var modelObject = {"modelId": modelId, "modelName": reportDataPoint[comparison.values[0]]};
                            var needToAddModelName = true;

                            // Add the first selected model
                            if (updatedCollectionOfModelObjects.length === 0) {
                                updatedCollectionOfModelObjects.push(modelObject);
                            }
                            // If this is not the first selected model, see if it exists in the collection already and if not, add it
                            else {
                                for (var k = 0; k < collectionOfModelObjectsForModelComparison.length; k++) {
                                    var collectionModel = updatedCollectionOfModelObjects[k];
                                    if (collectionModel.modelId === modelId) {
                                        needToAddModelName = false;
                                        break;
                                    }
                                }
                                if (needToAddModelName)
                                    updatedCollectionOfModelObjects.push(modelObject);
                            }
                            collectionOfModelObjectsForModelComparison = updatedCollectionOfModelObjects; // Update the main collection

                            // Depending on which type of table we are generating, manipulate the data so we can display the columns correctly
                            if (stat && stat.values[0] !== null) {

                                // Create a Fit Statistics data model
                                var fitStatModelComparePropName = reportDataPoint[stat.values[0]];

                                // If the Dictionary does not have the property, add it along with its data
                                if (!propertyNameLookup.hasOwnProperty(fitStatModelComparePropName)) {
                                    var tempFitDataObject = {};
                                    tempFitDataObject.LABEL = reportDataPoint[this.reportTable.getLabel().values[0]];
                                    tempFitDataObject.STAT = reportDataPoint[stat.values[0]];
                                    tempFitDataObject.TARGET = reportDataPoint[this.reportTable.getTarget().values[0]];

                                    // This should always be true
                                    if (this.haveTrainingData)
                                        tempFitDataObject["TRAIN_" + modelId] = reportDataPoint[this.reportTable.getTrain().values[0]];

                                    // Only display Vaidation data if we have it
                                    if (this.haveValidationData)
                                        tempFitDataObject["VALIDATE_" + modelId] = reportDataPoint[this.reportTable.getValidate().values[0]];

                                    // Only display Testing data if we have it
                                    if (this.haveTestingData)
                                        tempFitDataObject["TEST_" + modelId] = reportDataPoint[this.reportTable.getTest().values[0]];

                                    propertyNameLookup[fitStatModelComparePropName] = tempFitDataObject;
                                    modelData.push(tempFitDataObject);
                                }
                                else {
                                    // Object exists in the Dictionary but we need to add more data to it, so grab it and add appropriate data
                                    var moreFitDataObject = propertyNameLookup[fitStatModelComparePropName];

                                    // This should always be true
                                    if (this.haveTrainingData)
                                        moreFitDataObject["TRAIN_" + modelId] = reportDataPoint[this.reportTable.getTrain().values[0]];

                                    // Only display Vaidation data if we have it
                                    if (this.haveValidationData)
                                        moreFitDataObject["VALIDATE_" + modelId] = reportDataPoint[this.reportTable.getValidate().values[0]];

                                    // Only display Testing data if we have it
                                    if (this.haveTestingData)
                                        moreFitDataObject["TEST_" + modelId] = reportDataPoint[this.reportTable.getTest().values[0]];
                                }
                            }
                            else if (this.reportTable.getName() && this.reportTable.getName().values[0] !== null) {
                                // Create a Scoring Inputs data model
                                var scoreInputsModelComparePropertyName = reportDataPoint[this.reportTable.getName().values[0]];
                                this.modelComparisonScoringInputs = true;
                                var scoreDataObject;

                                // If the Dictionary does not have the property, add it along with its data
                                if (!propertyNameLookup.hasOwnProperty(scoreInputsModelComparePropertyName)) {
                                    scoreDataObject = {};
                                    scoreDataObject.Name = reportDataPoint[this.reportTable.getName().values[0]];
                                    scoreDataObject["SCORE_" + modelId] = reportDataPoint[this.reportTable.getRelativeImportance().values[0]];
                                    propertyNameLookup[scoreInputsModelComparePropertyName] = scoreDataObject;
                                    modelData.push(scoreDataObject);
                                }
                                else {
                                    // Object exists in the Dictionary but we need to add more data to it, so grab it and add appropriate data
                                    scoreDataObject = propertyNameLookup[scoreInputsModelComparePropertyName];
                                    scoreDataObject["SCORE_" + modelId] = reportDataPoint[this.reportTable.getRelativeImportance().values[0]];
                                }
                            }
                            else if (this.reportTable.getComponent() && this.reportTable.getComponent().values[0] !== null) {
                                var propertyName = this.reportTable.getPropertyName().values[0];
                                var component = this.reportTable.getComponent().values[0];
                                var flowModelComparePropertyName = reportDataPoint[this.reportTable.getComponent().values[0]] + "_" + reportDataPoint[propertyName];;

                                var flowDataObject;

                                // If the Dictionary does not have the property, add it along with its data
                                if (!propertyNameLookup.hasOwnProperty(flowModelComparePropertyName)) {
                                    flowDataObject = {};
                                    flowDataObject[component] = reportDataPoint[component];
                                    flowDataObject[propertyName] = reportDataPoint[propertyName];
                                    flowDataObject[modelId] = reportDataPoint[this.reportTable.getPropertyValue().values[0]];
                                    propertyNameLookup[flowModelComparePropertyName] = flowDataObject;
                                    modelData.push(flowDataObject);
                                }
                                else {
                                    // Object exists in the Dictionary but we need to add more data to it, so grab it and add appropriate data
                                    flowDataObject = propertyNameLookup[flowModelComparePropertyName];
                                    flowDataObject[modelId] = reportDataPoint[this.reportTable.getPropertyValue().values[0]];
                                }
                            }
                        }
                    }
                }
            }

            // If a model comparison scoring inputs table...
            if (this.modelComparisonScoringInputs === true) {

                // ROP - Because we allow sorting by model values in this table, make sure there are no sparse cells.
                // If sparse cells are present, the model-specific sort gets messed up!  Put a 0 in the sparse cells.
                for (var i = 0; i < modelData.length; i++) {
                    var node = modelData[i];
                    for (var j = 0; j < collectionOfModelObjectsForModelComparison.length; j++) {
                        var id = collectionOfModelObjectsForModelComparison[j].modelId;
                        if (!node.hasOwnProperty("SCORE_" + id))
                            node["SCORE_" + id] = 0;
                    }
                }
            }

            // Depending on which parameters are present, build that type of DataGrid/Table

            var parameter = this.reportTable.getParameter();
            var label = this.reportTable.getLabel();
            if (parameter && parameter.values[0] !== null && parameter.values[0].length > 0) {
                // Build a Parameter Estimates table and sort the data by parameter value
                this.sortField = parameter.values[0];
                dgcArrayCol = this.buildParameterEstimatesTable();
            }
            else if ((label && label.values[0] !== null && label.values[0].length > 0) && (this.reportTable.getTarget() !== null)) {
                // Build a Fit Statistics table and sort the data according to Statistic name/label
                this.sortField = this.reportTable.getLabel().values[0];

                // Build a different table based on whether this is a Model Comparison or not
                if (modelComparison)
                    dgcArrayCol = this.buildModelComparisonFitStatisticsTable(collectionOfModelObjectsForModelComparison);
                else
                    dgcArrayCol = this.buildFitStatisticsTable();
            }
            else if (this.reportTable.getComponent()) {
                // Build a Flow table
                this.sortField = this.reportTable.getComponent().values[0];

                // Build a different table based on whether this is a Model Comparison or not
                if (modelComparison)
                    dgcArrayCol = this.buildModelComparisonFlowTable(collectionOfModelObjectsForModelComparison);
                else
                    dgcArrayCol = this.buildFlowTable();
            }
            else if (this.reportTable.getRelativeImportance())  {
                var sortCol;

                // Build a different table based on whether this is a Model Comparison or not
                if (modelComparison) {
                    // Sort by Name
                    sortCol = this.reportTable.getName();
                    dgcArrayCol = this.buildModelComparisonScoringInputsTable(collectionOfModelObjectsForModelComparison);
                }
                else {
                    // Sort by Rank
                    sortCol = this.reportTable.getRank();
                    dgcArrayCol = this.buildScoringInputsTable();
                }

                if (sortCol !== null)
                    this.sortField = sortCol.values[0];
            }
            // Otherwise, build this generic table according to the passed values
            else {
                dgcArrayCol = this.buildGenericTable();
            }

            // Sort the data according to which type of table it is
            if (this.sortField)
                modelData.sort(this.getSortMethod(this.sortField));

            for (var i = 0; i < dgcArrayCol.length; i++) {
                table.addColumn(dgcArrayCol[i]);
            }

            // Set the visible row count to Auto.  This gives horizontal scrollbars in the table tile.  And it
            // causes the table to expand to fill the dialog in expanded mode
            table.setVisibleRowCountMode(sap.ui.table.VisibleRowCountMode.Auto);

            var model = new sap.ui.model.json.JSONModel();
            model.setData({modelData: modelData});

            table.setModel(model);
            table.bindRows("/modelData");

            this.addItem(table);
        },

        getDescription: function() {
            var desc = this.reportTable.getDescription();
            if (this.modelComparisonScoringInputs === true)
                desc = desc + ": " + sas.modelfactory.RB.getText("relative.importance.txt");
            return desc;
        },

        getSortMethod: function(){
            var _args = Array.prototype.slice.call(arguments);
            return function(a, b) {
                for (var x in _args) {
                    var ax = a[_args[x]];
                    var bx = b[_args[x]];

                    // Handle character sort in locale independent way
                    if (typeof ax === "string") {
                        if (ax !== bx)
                            return ax.localeCompare(bx);
                    }
                    // Otherwise, handle numeric sort
                    else {
                        ax = ax / 1;
                        bx = bx / 1;

                        if (ax !== bx) {
                            return ax < bx ? -1 : 1;
                        }
                    }
                }
            }
        },

        getReportName: function() {
            return this.getDescription();
        },

        /**
         * Create the parameter estimates data grid for a given model's data
         *
         * Returns: A collection of the DataGrid Columns to build a Parameter Estimates table
         */
        buildParameterEstimatesTable: function() {
            var petArrayCol = [];

            // Columns are: Parameter; tvalue; Effect; Sign; Estimate; AbsEstimate; ProbChiSq
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getParameter()));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getTvalue(), this.format4));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getEffect()));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getSign(), null, "50px"));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getEstimate(), this.format4));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getAbsEstimate(), this.format4));
            petArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getProbChiSq(), this.format4));

            return petArrayCol;
        },

        /**
         * Create the model fit statistics data grid for a given model's data
         *
         * Returns: A collection of the DataGrid Columns to build a Fit Statistics table
         */
        buildFitStatisticsTable: function() {
            var fitArrayCol = [];

            // Columns are: Target; label; train (if exists); validate (if exists), test (if exists)
            fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getTarget(), null, "100px"));
            fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getLabel(), null, "150px"));

            // TODO: Check with Dan & Jared to see if we should follow model of showing Testing, then Validation, then fall back to Training
            if (this.haveTrainingData)
                fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getTrain(), this.format4, "100px"));

            if (this.haveValidationData)
                fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getValidate(), this.format4, "100px"));

            if (this.haveTestingData)
                fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getTest(), this.format4, "100px"));

            return fitArrayCol;
        },

        /**
         * Create the model fit statistics data grid for model comparison's data
         *
         * @param collectionOfModelObjects ArrayList A list of the model objects for the model comparison
         *
         * Returns: A collection of the DataGrid Columns to build a Fit Statistics table for Model Comparison
         */
        buildModelComparisonFitStatisticsTable: function(collectionOfModelObjects) {
            var fitArrayCol = [];

            // Create the two main columns of target and label
            fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getTarget(), null, "100px"));
            fitArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getLabel(), null, "150px"));

            // Display Training and Validation data if present during Model Comparison
            for (var i = 0; i < collectionOfModelObjects.length; i++) {
                var model = collectionOfModelObjects[i];
                if (this.haveTrainingData)
                    fitArrayCol.push(this.buildDataGridColumn("TRAIN_" + model.modelId, "num", sas.modelfactory.RB.getText("trainView.columnPrefixTrain.txt") + " " + model.modelName, this.format4, "100px"));
                if (this.haveValidationData)
                    fitArrayCol.push(this.buildDataGridColumn("VALIDATE_" + model.modelId, "num", sas.modelfactory.RB.getText("trainView.columnPrefixValid.txt") + " " + model.modelName, this.format4, "100px"));
            }

            return fitArrayCol;
        },

        /**
         * Create the scoring inputs data grid for a given model's data
         *
         * Returns: An array of the DataGrid Columns to build a Scoring Inputs table
         */
        buildScoringInputsTable: function() {
            var siArrayCol = [];

            // Columns are: name; rank; relativeImportance
            siArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getName(), null, "150px", true));
            siArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getRank(), this.formatInteger, "75px", true));
            siArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getRelativeImportance(), this.format4, "150px", true));

            return siArrayCol;
        },

        /**
         * Create the model scoring inputs data grid for model comparison's data
         *
         * @param collectionOfModelObjects ArrayList A list of the model objects for the model comparison
         *
         * Returns: An array of the DataGrid Columns to build a Scoring Inputs table for Model Comparison
         */
        buildModelComparisonScoringInputsTable: function(collectionOfModelObjects) {
            var siArrayCol = [];

            // Create the main name column
            siArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getName(), null, "150px", true));

            // Create a column of scoring data for each model in the Model Comparison
            for (var i = 0; i < collectionOfModelObjects.length; i++) {
                var model = collectionOfModelObjects[i];
                siArrayCol.push(this.buildDataGridColumn("SCORE_" + model.modelId, "num", model.modelName, null, "200px", true, RelativeImportance));
            }

            return siArrayCol;
        },

        /**
         * Create the flow data grid for a given model's data
         *
         * Returns: A collection of the DataGrid Columns to build a Flow table
         */
        buildFlowTable: function() {
            var flowArrayCol = [];

            // Columns are: Target; label; train (if exists); validate (if exists), test (if exists)
            flowArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getComponent(), null, "100px"));
            flowArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getPropertyName(), null, "150px"));
            flowArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getPropertyValue(), null, "100px"));

            return flowArrayCol;
        },

        /**
         * Create the flow data grid for model comparison's data
         *
         * @param collectionOfModelObjects ArrayList A list of the model objects for the model comparison
         *
         * Returns: A collection of the DataGrid Columns to build a Flow table for Model Comparison
         */
        buildModelComparisonFlowTable: function(collectionOfModelObjects) {
            var flowArrayCol = [];

            // Create the two main columns of Component and label
            flowArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getComponent(), null, "100px"));
            flowArrayCol.push(this.buildDataGridColumnFromParameter(this.reportTable.getPropertyName(), null, "100px"));

            for (var i = 0; i < collectionOfModelObjects.length; i++) {
                var model = collectionOfModelObjects[i];
                flowArrayCol.push(this.buildDataGridColumn(model.modelId, "char", model.modelName, null, "100px"));
            }

            return flowArrayCol;
        },

        /**
         * Create a generic data grid for a given model's data, using the system supplied order
         *
         * Returns: An array of the DataGrid Columns to build a generic table
         */
        buildGenericTable: function() {
            var viArrayCol = [];

            // Columns are plucked from the pre-sorted parameters
            var columns = this.reportTable.getReportParameters();
            for (var i = 0; i < columns.length; i++) {
                viArrayCol.push(this.buildDataGridColumnFromParameter(columns[i], null, "90px", true));
            }

            return viArrayCol;
        },

        buildDataGridColumnFromParameter: function(parameter, labelFunction, width, sortable, itemRenderer) {
            // If not preformatted, numeric, and no formatting function passed in, default to format4
            if (parameter.preformatted === false && parameter.type === "num" && labelFunction === null)
                labelFunction = this.format4;
            // If preformatted, numeric, and no formatting function passed in, format to locale only
            else if (parameter.preformatted === true && parameter.type === "num" && labelFunction === null)
                labelFunction = this.formatToLocale;
            var dgc = this.buildDataGridColumn(parameter.values[0], parameter.type, parameter.label, labelFunction, width, sortable, itemRenderer);

            return dgc;
        },

        /**
         * Build a DataGrid column based on the parameters passed in
         *
         * @param dataField - String The data field for the column
         * @param headerText - String The header text for the column
         * @param labelFunction - Function The function to render the label; Default is null
         * @param width Number - The width of the column; Default is 100
         * @param sortable - Boolean Is the column sortable; Default is true
         * @param itemRenderer - ClassFactory A renderer for the item in the column
         *
         * Returns: DataGridColumn A new DataGridColumn based on the parameters
         */
        buildDataGridColumn: function(dataField, type, headerText, labelFunction, width, sortable, itemRenderer) {
            var dgc = new Column();

            if (itemRenderer) {
                var importance = new itemRenderer().bindProperty("percentValue", {
                    path: dataField,
                    formatter: this.formatImportancePercent
                }).bindProperty("displayValue",{
                    path: dataField,
                    formatter: this.format4
                });

                dgc.setTemplate(importance);
            }
            else {
                var tv = new sas.hc.m.Text().bindProperty("text", {
                    path: dataField,
                    formatter: labelFunction
                });
                dgc.setTemplate(tv);

                if (width !== undefined)
                    dgc.setWidth(width);
            }

            // Make the column label the column name if the label is null or blank
            if (headerText === null || headerText.length === 0)
                headerText = dataField;

            var align;
            if (type === "num") {
                align = sap.ui.core.TextAlign.End ;
                var tv = dgc.getTemplate();
                if (tv !== null && tv.setTextAlign)
                    tv.setTextAlign("End");
                dgc.setStyleClass("fmColumnTextAlignEnd");
            }
            else
                align = sap.ui.core.TextAlign.Begin ;

            dgc.setLabel(new sap.m.Text({textAlign: align, /* wrapping: true, */ text: headerText}));

            if (sortable !== undefined && sortable == true)
                dgc.setSortProperty(dataField);

            return dgc;
        },

        /**
         * Determine the type of data we have to work with and set the global variables appropriately
         *
         * @param dataPoint - Object which represents the data point
         *
         * Returns: nothing
         */
        determineDataForDataPoint: function(dataPoint) {
            var trainNumber = NaN;
            var validateNumber = NaN;
            var testNumber = NaN;

            if (this.reportTable.getTrain() && this.reportTable.getTrain().values[0] !== null) {
                trainNumber = dataPoint[this.reportTable.getTrain().values[0]];
                if (!isNaN(trainNumber))
                    this.haveTrainingData = true;
            }

            if (this.reportTable.getValidate() && this.reportTable.getValidate().values[0] !== null) {
                validateNumber = dataPoint[this.reportTable.getValidate().values[0]];
                if (!isNaN(validateNumber))
                    this.haveValidationData = true;
            }

            if (this.reportTable.getTest() && this.reportTable.getTest().values[0] !== null) {
                testNumber = dataPoint[this.reportTable.getTest().values[0]];
                if (!isNaN(testNumber))
                    this.haveTestingData = true;
            }
        },

        /**
         * Formats a column's numeric data to four decimal places.
         *
         * @returns the formatted numeric value as a String
         *
         **/
        format4: function(value) {
            if (value === undefined || value === null)
                return null;
            else if (isNaN(value))
                return null;

            var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
            if (localeStr === null)
                localeStr = "en_US";

            var floatFmt = sap.ui.core.format.NumberFormat.getFloatInstance({
                minFractionDigits: 4,
                maxFractionDigits: 4,
                groupingEnabled: true
            }, new sap.ui.core.Locale(localeStr));

            var retVal = floatFmt.format(value);

            // If the value is positive, but less than 0.0000, return it in scientific notation to 4 digits
            if (value > 0 && retVal === 0)
                retVal = value.toExponential(4);

            return retVal;
        },

        /**
         * Formats a column's numeric data to locale only.
         *
         * @returns the formatted numeric value as a String
         *
         **/
        formatToLocale: function(value) {
            if (value === undefined || value === null)
                return null;
            else if (isNaN(value))
                return null;

            var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
            if (localeStr === null)
                localeStr = "en_US";

            var floatFmt = sap.ui.core.format.NumberFormat.getFloatInstance({
                groupingEnabled: true
            }, new sap.ui.core.Locale(localeStr));

            var trimmed = value.trim(); // Remove any leading/trailing whitespace
            var retVal = floatFmt.format(trimmed);

            return retVal;
        },

        formatInteger: function(value) {
            if (value === undefined || value === null)
                return null;
            else if (isNaN(value))
                return null;

            var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
            if (localeStr === null)
                localeStr = "en_US";

            var integerFmt = sap.ui.core.format.NumberFormat.getIntegerInstance({
            }, new sap.ui.core.Locale(localeStr));

            var retVal = integerFmt.format(value);

            return retVal;
        },

        /**
         * Formats a column's numeric data to four decimal places and multiplied by 100 for percentage.
         *
         * @returns the formatted numeric value as a String
         *
         **/
        formatImportancePercent: function(value) {
            if (value === undefined || value === null)
                return null;
            return Math.round(value.toFixed(4) * 100);
        }

    });
})();