jQuery.sap.require(
    "sap.m.MessageToast",
    "sas.hc.m.MessageBox",
    "sas.modelfactory.proxies.ModelProxy",
    "sas.modelfactory.proxies.TemplateProxy",
    "sas.modelfactory.proxies.TrainProxy",
    "sas.modelfactory.proxies.JobProxy",
    "sas.modelfactory.proxies.ScoreCodeProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.report.BaseGraphController",
    "sas.modelfactory.util.PreferencesUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sas.modelfactory.report.BaseGraphController.extend("sas.modelfactory.segment.train.Train", {

    project: null,
    projectId: null,
    segmentId: null,
    depth: null,
    currentModelId: -1,
    usePartition: false,
    selectedRows: null,
    isPolling: false,
    stopPolling: false,
    dataLoaded: false,
    resultsLoaded: false,
    //dialog: null,
    newModelDialog: null,
    CONST_POLL_TIME: 2500, // 2.5 secs.
    me: null,
    fragColData: null,
    blankModel: null,
    isNewModel: false,
    statusMap: null,
    pendingRequestsArray: [],
    preferenceId: null,
    currentSelectedRows: [],
    resultsCached: false,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.modelfactory.segment.train.Train
     */
    onInit: function() {

        this.me = this;
        var controller = this;
        sas.modelfactory.report.BaseGraphController.prototype.onInit.apply(this);

        // subscribe to event bus
        sas.eventBus.subscribe("ModelTemplateEvent", "dataLoaded", this.modelDataLoadedHandler, this);

        this.getView().addEventDelegate({
            onBeforeFirstShow: function(evt) {
                // Get the current open project
                var model = sap.ui.getCore().getModel("openProject");
                var project = null;
                if (model){
                    project = model.getProperty("/project");
                    if (project){
                        controller.project = project;
                        controller.usePartition = project.usePartition;
                    }
                }

                var objectContentPage = sap.ui.getCore().byId(controller.getView().data("objectContentPageId"));
                if (objectContentPage) {
                    var model = objectContentPage.getItem();
                    if (model) {
                        controller.projectId = model.projId;
                        controller.segmentId = model.segId;
                        controller.depth = model.depth;
                    }

                    // Place the segment information button onto the custom header
                    objectContentPage.getCustomHeader().insertContentRight(controller.getView().segmentInfoButton);

                    var view = controller.getView();
                    if (view) {
                        view.usePartition = controller.usePartition;
                    }
                }

                controller.statusMap = {};
                var enumModel = sap.ui.getCore().getModel("enums");
                if (enumModel){
                    controller.statusMap = enumModel.getProperty("/runStatus");

                    if (controller.statusMap === undefined){
                        logUtil.error("Enumeration Error: enum map missing", controller.statusMap);
                    }
                }

                // Get the models for the selected segment
                if (controller.projectId && controller.segmentId){
                    controller.getModels(controller.projectId, controller.segmentId);
                }
            },

            onBeforeShow: function(evt) {
                controller.stopPolling = false;
            },

            onBeforeHide: function() {
                logUtil.log("Train onBeforeHide hit");
                controller.stopPolling = true;
            }
        });

        this.getView().table.attachAfterColumnsDialogClose(function(oEvent) {
            // If OK was pressed, persist the column manager changes
            if (oEvent.getParameter("acceptedChange"))
                controller.onColumnManagerClose();
        });
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.segment.train.Train
     */
    onBeforeRendering: function () {

    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.segment.train.Train
     */
    onAfterRendering: function ()
    {
        jQuery(".results-hLayout").css("height", "600px");
        jQuery(".actionButton-floatRight").parent().css("float", "right");
    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.segment.train.Train
     */
    onExit: function() {
        logUtil.log("sas.modelfactory.segment.train.Train: onExit called");

        sas.eventBus.publish("ProjectEvent", "reloadResults", {} );
        if (this.project){
            var links = this.project.links;
            var projectUri = messageProxy.findUri(links, "self", "GET");
            if (projectUri) {
                // publish to event bus
                sas.eventBus.publish("ProjectEvent", "reload", { projectUri: projectUri});
            }
        }

        // Stop any polling
        this.stopPolling = true;

        this.destroyTiles(this.getView(), []);
    },

    processControls: function(indices){
        var view = this.oView;

        var anyRunningStatus = function() {
            var selection = view.table.getModel().getProperty("/models");
            for(var i=0;i<selection.length;i++){
                var item = selection[i];
                if (item.status.key === "RUNNING" || item.status.key === "SCHEDULED"){
                    return true;
                }
            }
            return false;
        };

        var runningStatus = function(selection) {
            for(var i=0;i<selection.length;i++){
                var context = view.table.getContextByIndex(selection[i]);
                if (context){
                    var item = context.getObject();
                    if (item.status.key === "RUNNING" || item.status.key === "SCHEDULED"){
                        return true;
                    }
                }
            }
            return false;
        };

        var needsRunning = function(selection) {
            for(var i=0;i<selection.length;i++){
                var context = view.table.getContextByIndex(selection[i]);
                if (context){
                    var item = context.getObject();
                    if (item.status.key === "RUN_NEEDED" || item.status.key === "CANCELED" || item.status.key === "ERROR" || item.status.key === "FAILED"){
                        return true;
                    }
                }
            }
            return false;
        };

        var allowDelete = function(selection) {
            // To allow delete,
            // a. One of the selections must not be baseline
            var foundBaseline = false;
            for(var i=0;i<selection.length;i++){
                var context = view.table.getContextByIndex(selection[i]);
                if (context){
                    var item = context.getObject();
                    if (item.miningAlgorithm.internalName === "Baseline"){
                        foundBaseline = true;
                    }
                }
            }

            return (selection.length>1 && foundBaseline) || !foundBaseline;
        };

        this.oView.addModelFromGlobalTemplateBtn.setEnabled(!anyRunningStatus());
        this.oView.newModelBtn.setEnabled(!anyRunningStatus());

        // Enable/disable the actions button
        if (indices.length == 0) {
            this.oView.actionButton.setEnabled(false);
            this.byId("runBtn").setEnabled(false);
            this.byId("stopBtn").setEnabled(false);
            this.byId("compareBtn").setEnabled(false);
            this.byId("deleteBtn").setEnabled(false);
            this.byId("duplicateBtn").setEnabled(false);
            this.byId("championBtn").setEnabled(false);
            this.byId("unChampionBtn").setEnabled(false);
            this.byId("renameBtn").setEnabled(false);
            this.byId("reportBtn").setEnabled(false);
            this.byId("logBtn").setEnabled(false);
            this.byId("scoreBtn").setEnabled(false);
            this.oView.editBtn.setEnabled(false);
        } else if (indices.length == 1) {
            var index = indices[0];
            var context = this.oView.table.getContextByIndex(index);
            var selectedModel = null;
            var allowEditProperties = false;
            if (context) {
                var selectedModel = context.getObject();
                allowEditProperties = selectedModel.miningAlgorithm.internalName !== "Baseline" && !anyRunningStatus();
            }

            this.oView.actionButton.setEnabled(true);
            this.byId("runBtn").setEnabled(needsRunning(indices));
            this.byId("stopBtn").setEnabled(runningStatus(indices));
            this.byId("compareBtn").setEnabled(false);
            this.byId("deleteBtn").setEnabled(allowDelete(indices));
            this.byId("duplicateBtn").setEnabled(true);

            // Handle the enabling/disabling of the Mark/Unmark as Champion actions
            if (selectedModel) {
                var isModelStatusCompleted = selectedModel.status && (selectedModel.status.key === "COMPLETED");
                this.byId("championBtn").setEnabled(!(selectedModel.champion) && isModelStatusCompleted);
                this.byId("unChampionBtn").setEnabled(selectedModel.champion && selectedModel.championOverridden);
            } else {
                this.byId("championBtn").setEnabled(false);
                this.byId("unChampionBtn").setEnabled(false);
            }

            this.byId("renameBtn").setEnabled(true);
            this.byId("reportBtn").setEnabled(true);
            this.byId("logBtn").setEnabled(true);
            this.byId("scoreBtn").setEnabled(true);
            this.oView.editBtn.setEnabled(allowEditProperties);
        } else if (indices.length > 1) {
            this.oView.actionButton.setEnabled(true);
            this.byId("runBtn").setEnabled(needsRunning(indices));
            this.byId("stopBtn").setEnabled(runningStatus(indices));
            this.byId("compareBtn").setEnabled(true);
            this.byId("deleteBtn").setEnabled(allowDelete(indices));
            this.byId("duplicateBtn").setEnabled(false);
            this.byId("championBtn").setEnabled(false);
            this.byId("unChampionBtn").setEnabled(false);
            this.byId("renameBtn").setEnabled(false);
            this.byId("reportBtn").setEnabled(true);
            this.byId("logBtn").setEnabled(false);
            this.byId("scoreBtn").setEnabled(true);
            this.oView.editBtn.setEnabled(false);
        }
    },

    storeSelectedRows: function(indices) {
        var controller = this;
        var view = this.getView();
        var selArray = [];
        if (indices.length > 0) {
            // Walk the list and store off the selected row's ids
            for(var i=0; i<indices.length; i++){
                var context = view.table.getContextByIndex(indices[i]);
                if (context) {
                    var selectedModel = context.getObject();
                    selArray.push(selectedModel.id);
                    logUtil.log("storeSelectedRows: row", indices[i], "for", selectedModel.name);
                }
            }

            // Store the selected ids
            controller.currentSelectedRows = selArray;
            //logUtil.log("storeSelectedRows:", controller.currentSelectedRows);
        }
    },

    getIndexForId: function(id) {
        var controller = this;
        var view = this.getView();
        var models = view.table.getModel().getProperty("/models");
        if (models.length > 0) {
            for(var j=0; j<models.length; j++) {
                var context = view.table.getContextByIndex(j);
                if (context) {
                    if (context.getObject().id === id) {
                        //logUtil.log("getIndexForId: id-", id, "index-", j, "name-", context.getObject().name);
                        return j;
                        break;
                    }
                }
            }
        }
        return -1;
    },

    setSelectedRows: function() {
        var controller = this;
        var view = this.getView();
        var models = view.table.getModel().getProperty("/models");
        var selIndices = [];
        if(controller.currentSelectedRows.length > 0){
            if (models.length > 0) {
                for(var i=0; i<controller.currentSelectedRows.length; i++){
                    var index = controller.getIndexForId(controller.currentSelectedRows[i]);
                    selIndices.push(index);
                }

                logUtil.log("setSelectedRows:", selIndices);
                for (var j=0;j<selIndices.length;j++){
                    if (j === 0)
                        view.table.setSelectionInterval(selIndices[j], selIndices[j]);
                    else
                        view.table.addSelectionInterval(selIndices[j], selIndices[j]);
                }
            }
        }
    },

    onRowSelectionClick: function (evt)
    {
        var view = this.oView;
        var controller = this;
        var indices = this.oView.table.getSelectedIndices();

        // If the user is about to deselect the last remaining selected row, keep that row selected (S1138813)
        // This keeps the graphs from blanking out when no model in the table is selected
        if (indices.length === 0 && this.selectedRows) {
            view.table.setSelectedIndex(evt.getParameter("rowIndex"));
            return;
        }

        controller.storeSelectedRows(indices);
        logUtil.log("rows selected: " + indices);

        controller.processControls(indices);

        // If a selection is currently made, update the graphs
        if (indices.length > 0) {

            // If more than one model selected, show only the first.
            var index = indices[0];
            var context = this.oView.table.getContextByIndex(index);
            if (context) {
                var selectedModel = context.getObject();

                // Get the results
                if (this.currentModelId !== selectedModel.id) {
                    this.resultsLoaded = false;
                    this.getModelResults(selectedModel);
                }
            }
        }
        // Otherwise, no selections so have to remove the graphs
        else {
            // Store current id
            this.currentModelId = -1;

            // No selection. Remove the last reports
            var reports = view.graphGrid.getContent();
            if (reports){
                view.graphGrid.removeAllContent();
                view.getModel().setProperty("/reports", null);
            }
        }
    },

    getModelResults: function(selectedModel){
        var controller = this;
        var view = this.oView;

        var clearReports = function() {
            // Remove the last reports
            var reports = view.graphGrid.getContent();
            if (reports){
                view.graphGrid.removeAllContent();
                view.graphGrid.rerender();
                view.getModel().setProperty("/reports", null);

                // Explicitly destroy all previous graphs before obtaining new reports
                for (var i = 0; i < reports.length; i++) {
                    reports[i].destroy(controller);
                }
            }
        };

        var onSuccess = function(reports){
            controller.resultsLoaded = true;
            busySpinner.show(false);
            clearReports();

            // setup model
            controller.setReports(reports, selectedModel.miningAlgorithm.id);

            // Update the cache
            if (reports){
                var model = view.table.getModel();
                models = model.getProperty("/models");
                var reportModel = view.getModel("results");
                var uri = messageProxy.findUri(reports.links, "self", "GET");
                var id = "";
                if (uri){
                    id = uri.slice(8, uri.indexOf("/results"));
                    logUtil.log("Model id:", id);
                    if (id){
                        for (var j=0;j<models.length;j++){
                            if (models[j].id === id){
                                if (reportModel === undefined){
                                    reportModel = new sap.ui.model.json.JSONModel();
                                    reportModel.setData({"reports": []});
                                }
                                // Update the model
                                reportModel.setProperty("/reports/"+id, reports);
                                logUtil.log("Update model:", reportModel.getProperty("/reports/"+id));
                            }
                        }
                    }
                }
+
                view.setModel(reportModel, "results");
            }

            var uri = this.url;
            var id = "";
            if (uri){
                id = uri.slice(uri.indexOf("models/")+7, uri.indexOf("/results"));
                logUtil.log("Model id:", id);
                if (id){
                    controller.removePendingRequest(id);
                }
            }
        };

        var onError = function(xhr, err, msg){
            var self = this;
            busySpinner.show(false);

            var json = xhr.responseJSON;
            if (json){
                var errMsg = (xhr.status?(xhr.status + ":  "):"") + msg + "\n\n" +
                    sas.modelfactory.RB.getText("error.errorMore.txt") + "\n" +
                    sas.modelfactory.RB.getText("error.errorCode.txt") + " " + (json!==undefined && json.errorCode?json.errorCode:"")  + "\n" +
                    sas.modelfactory.RB.getText("error.errorMsg.txt") + " " + (json!==undefined && json.message?json.message:"")  + "\n" +
                    sas.modelfactory.RB.getText("error.errorDetails.txt") + " " + (json!==undefined && json.details?json.details:"") + "\n" +
                    sas.modelfactory.RB.getText("error.errorUrl.txt") + " " + this.url;
                //(sMessage, sTitle, vAction, sDialogId, fnCallback)
                var title = sas.modelfactory.RB.getText("trainView.errorMsg.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title, sas.hc.m.MessageBox.Action.CLOSE, "modelResultsDialog", function(evt){
                    var uri = self.url;
                    var id = "";
                    if (uri){
                        id = uri.slice(uri.indexOf("models/")+7, uri.indexOf("/results"));
                        logUtil.log("Model id:", id);
                        if (id){
                            if (controller.isPolling){
                                var req = controller.getPendingRequest(id);
                                controller.updatePendingRequest(id, req.hasCallback, true);
                            } else {
                                controller.removePendingRequest(id);
                            }
                        }
                    }
                });
            }
        };

        var onComplete = function(oJson){
            busySpinner.show(false);
        };

        // Store current id
        this.currentModelId = selectedModel.id;

        // Check the cache first
        var cachedResults = null;
        var reportsModel = view.getModel("results");
        if (reportsModel){
            cachedResults = reportsModel.getProperty("/reports/"+selectedModel.id);
        }
        if (selectedModel.status && selectedModel.status.key === "COMPLETED" && cachedResults){
            logUtil.log("Loading results for", selectedModel.name, "from cache");
            busySpinner.show(false);
            if (!controller.resultsLoaded){
                controller.resultsLoaded = true;
                clearReports();

                // setup model
                controller.setReports(cachedResults, selectedModel.miningAlgorithm.id);
            }
        } else if (selectedModel.status && selectedModel.status.key !== "COMPLETED") {
            clearReports();
        } else {
            clearReports();

            // Get the model results
            if (selectedModel.status && selectedModel.status.key === "COMPLETED"){
                if (!controller.isPending(selectedModel.id)){
                    busySpinner.show(true, sas.modelfactory.RB.getText("trainView.busy.title.txt"));
                    controller.addPendingRequest(selectedModel.id, true);
                    modelProxy.getModelResults(selectedModel.id, onSuccess, onError, onComplete);
                } else {
                    // Is a pending request.  Does it have a callback?
                    if (!controller.hasCallback(selectedModel.id)){
                        busySpinner.show(true, sas.modelfactory.RB.getText("trainView.busy.title.txt"));
                        controller.updatePendingRequest(selectedModel.id, true);
                    }
                }
            }
        }
    },

    onSegmentPropertiesPress: function(evt) {
        var controller = this;
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.SegmentPropertiesDialog", controller);

//        var data = [{name: "Target", value: "Bad"}, {name: "Segment", value: "Reason"}, {name: "Observations", value: "4582"}];
        var model = new sap.ui.model.json.JSONModel();

        model.setData({projectId: controller.projectId, segmentId: controller.segmentId});

        dialog.setModel(model);
        dialog.open();
    },

    onNewPress: function (evt) {
        var controller = this;
        controller.isNewModel = true;
        var onSuccess = function (oJson) {
            controller.newModelDialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewModelTemplateDialog", controller);

            // fetch the blank template- The algorithms are the in the properties where groupName = "Model"
            // compile a unique list
            var templates = oJson.items;

            var onBlankTemplateSuccess = function (oJson) {
                var algorithms = sap.ui.getCore().getModel("algorithms").getData().algorithmsForDisplay;
                controller.blankModel = oJson;

                var model = new sap.ui.model.json.JSONModel();
                model.setData({algorithms: algorithms});
                controller.newModelDialog.setModel(model);
                controller.newModelDialog.open();

                sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: oJson, saveEnabled: true});
            };

            var onBlankTemplateError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("Loading template");
                var title = sas.modelfactory.RB.getText("Loading template");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            for (var i=0; i<templates.length; i++) {
                logUtil.log("name: = " + templates[i].name);
                if (templates[i].name === "Blank Template") {
                    var links = templates[i].links;
                    messageProxy.request(links, "self", "GET", "json", null, onBlankTemplateSuccess, onBlankTemplateError);
                    break;
                }
            }
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.modelTemplate.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        }

        var onComplete = function () {
            sas.eventBus.unsubscribe("ModelTemplateEvent", "dataLoaded", this.modelDataLoadedHandler, this);
        }


        templateProxy.getTemplates(onSuccess, onError, onComplete);
    },

    onNewModelTemplateCreate: function(name, dialog) {

        var selectedItem = dialog.modelAlgorithm.getSelectedItem();
        logUtil.log("onNewModelTemplateCreate: ", name.getValue());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getText());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getKey());

        var controller = this;

        // Find the selected algorithm index.
        var algorithms = selectedItem.getModel().getData().algorithms;
        var algorithmIndex = -1;
        for (var i=0; i< algorithms.length; i++){
            if (algorithms[i].id === selectedItem.getKey()){
                algorithmIndex = i;
                break;
            }
        }

        // initialize the blankModel and "use" the respective model properties component
        var newModel = controller.blankModel;
        newModel.name = name.getValue();
        newModel.displayName = name.getValue();
        newModel.createdBy = null;
        newModel.modifiedBy = null;
        newModel.createdTimestamp = null;
        newModel.modifiedTimestamp = null;
        newModel.links = null;
        newModel.description = "User created model";
//        newModel.algorithmName = algorithms[algorithmIndex].displayName;
        newModel.miningAlgorithm = {id: algorithms[algorithmIndex].id, internalName: algorithms[algorithmIndex].internalName};
        newModel.favorite = dialog.defaultCheckBox.getSelected();
        newModel.makeGlobalTemplate = dialog.availCheckBox.getSelected();
        newModel.hidden = false;
        newModel.sasonly = false;

        for (var i=0; i<newModel.properties.length; i++) {
            if (newModel.properties[i].groupName === 'Model') {
                if (newModel.properties[i].componentName === algorithms[algorithmIndex].internalName) {
                    if (newModel.properties[i].propertyName === '_use') {
                        newModel.properties[i].propertyValue = 'Y';
                        newModel.properties[i].position = 1;
                    }
                }
            }

        }

        // open the model dialog
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", this);
        var title =  name.getValue();
        dialog.setTitle(title);
        dialog.addSaveButton();
        dialog.open();
        sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: newModel, saveEnabled: true});

    },

    onAddTemplateFromGlobalTemplatePress: function(evt) {

        var controller = this;
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.AddModelTemplateDialog", controller);
        var model = new sap.ui.model.json.JSONModel();

        logUtil.log("onAddTemplateFromGlobalTemplatePress");

        dialog.open();
        // load the available templates
        var onSuccess = function (oJson) {

            // This code removes hidden templates such as the blank template
            var i = 0;
            while ( i<oJson.items.length) {
                if (oJson.items[i].hidden) {
                    oJson.items.splice(i,1);
                } else {
                    i++;
                }
            }
            var model = new sap.ui.model.json.JSONModel();
            model.setData({templateData: oJson.items});

            sas.eventBus.publish("newModel", "addModelToProject", {templateData:  oJson.items});
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.modelTemplate.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        }

        var onComplete = function () {

        }

        var rootModel = sap.ui.getCore().getModel("root");
        if (rootModel === undefined) {
            logUtil.error("Root Link Error: Root model undefined.");
        } else {
            var links = rootModel.getProperty("/root");
            // Get the global template list
            messageProxy.request(links, "templates", "GET", "json", null, onSuccess);
        }
    },


    onAddModelTemplateCreate: function(selectedTemplates) {
        logUtil.log("AddModelTemplateCreate: ", selectedTemplates);
        var controller = this;

        var createTemplate = function(items) {
            if (items && items.length > 0){
                var selectedTemplate = items.pop();

                var onGetTemplateSuccess = function(oJson) {
                    var template = oJson;
                    var onCreateSuccess = function(oJson) {
                        logUtil.log("Success");

                        if (items.length > 0){
                            createTemplate(items);
                        }
                    };

                    var onCreateError = function(xhr, err, msg){
                        var errResponse = jQuery.parseJSON(xhr.responseText);
                        var title = sas.modelfactory.RB.getText("trainView.error.addModel.title.txt");
                        sas.hc.m.MessageBox.error(errResponse.message, title);
                    };

                    var onCreateComplete = function(oJson) {

                        var setSelectedModel = function() {
                            var selectedId = oJson.responseJSON.id;

                            // Attempt to set the currently created model as the selected model in the table
                            if (selectedId) {
                                var foundIndex = -1;
                                foundIndex = controller.getIndexForId(selectedId);
                                if (foundIndex !== -1){
                                    controller.oView.table.setSelectedIndex(foundIndex);
                                    controller.oView.table.setFirstVisibleRow(foundIndex);
                                }
                            }
                        }

                        logUtil.log("Complete");
                        if (items.length === 0){
                            controller.oView.setBusy(false);
                            controller.getModels(controller.projectId, controller.segmentId, setSelectedModel);
                        }
                    };
                    modelProxy.createModelForSegment(controller.project.id, controller.segmentId, template, onCreateSuccess, onCreateError, onCreateComplete);
                };

                var onGetTemplateError = function(xhr, err, msg){

                };

                var onGetTemplateComplete = function(oJson) {

                };

                if (selectedTemplate){
                    logUtil.log("Add template:", selectedTemplate.name);
                    templateProxy.getTemplate(selectedTemplate.id, onGetTemplateSuccess, onGetTemplateError, onGetTemplateComplete);
                }
            }
        };

        if (selectedTemplates.length > 0){
            createTemplate(selectedTemplates);
            controller.oView.setBusy(true);
        }

    },


    onTemplateSave: function(dialog) {
        var controller = this;
        var model = dialog.getModel();
        logUtil.log("Save model template:", JSON.stringify(model.oData.template));

        var index = this.oView.table.getSelectedIndex();
        var context = this.oView.table.getContextByIndex(index);
        if (context) {
            var selectedData = context.getObject();
            logUtil.log(selectedData);

            var onSuccess = function (oJson)
            {
                // Templated updated, refresh the model list.
                if (controller.projectId && controller.segmentId) {
                    controller.getModels(controller.projectId, controller.segmentId);
                }
            };

            var onError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("trainView.error.updateModelTemplate.txt");
                var title = sas.modelfactory.RB.getText("trainView.errorMsg.title.txt");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            // Update the model template
            if (controller.isNewModel) {
                var makeGlobalTemplate = model.oData.template.makeGlobalTemplate;
                delete model.oData.template['makeGlobalTemplate'];
                modelProxy.createModelForSegment(controller.project.id, controller.segmentId, model.oData.template, onSuccess);
                if (makeGlobalTemplate) {
                    var onGlobalSuccess = function (oJson) {
                        logUtil.log("Create global template")
                    };
                    var onGlobalError = function(xhr, err, msg){
                        logUtil.log("create global template failed");
                        var json = xhr.responseJSON;
                        var errMsg = sas.modelfactory.RB.getText("projectModelView.error.createGlobalTemplate.txt") +
                            "\n\n" + sas.modelfactory.RB.getText("projectModelView.error.status.txt") + xhr.status +
                            "\n\n " + json.message;
                        sas.hc.m.MessageBox.error(errMsg);
                    };
                    var onGlobalComplete = function () {
                        logUtil.log("Create global template completed");
                    };
                    templateProxy.createGlobalModelTemplate (model.oData.template, onGlobalSuccess, onGlobalError, onGlobalComplete);
                }

            } else {
                if (selectedData && selectedData.links) {
                    var links = selectedData.links;
                    messageProxy.request(links, "template", "PUT", "json", model.oData.template, onSuccess);
                }
            }
        }
    },

    onTemplateSaveAs: function(dialog) {
        var controller = this;
        var model = dialog.getModel().oData.template;
        logUtil.log("Save model template:", JSON.stringify(model));

        // Update or create the model template

        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
        dialog.renameHandler = this.saveAsTemplateRenameHandler;

        var dialogModel = new sap.ui.model.json.JSONModel();
        dialogModel.setData({template: model,
            templateData: controller.oView.table.getModel().oData.models});
        dialog.bindElement("/template");
        dialog.uniqueNamePath = "/templateData";
        dialog.setNameProperty("name");
        dialog.setModel(dialogModel);

        dialog.open();
    },

    saveAsTemplateRenameHandler: function(dialog) {

        var view = this.getView();
        var controller = this;
        var model = dialog.getModel();
        model.oData.template.name = model.oData.template.displayName;
        controller.oView.setBusy(true);

        var onSuccess = function (oJson) {
            // Templated updated, refresh the template list.
            if (controller.project) {
                controller.getModels(controller.projectId, controller.segmentId);
            }
        };
        var onError = function(xhr, err, msg){
            var errResponse = jQuery.parseJSON(xhr.responseText);
            var msg;
            if (errResponse) {
                msg = errResponse.message;
            }
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.saveAsModelTemplate.txt" + "\n\n" + msg);
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            view.setBusy(false);
        };

        var newModel = model.oData.template;
        model.name = model.displayName;
        var makeGlobalTemplate = model.makeGlobalTemplate;
        delete newModel['makeGlobalTemplate'];
        newModel.id = null;
        newModel.modelId = null;

        var dupModel = {
            name :  model.oData.template.name,
            id : model.oData.template.modelId
        };

        modelProxy.createModelForSegment(controller.project.id, controller.segmentId, newModel, onSuccess, onError, onComplete);
    },

    resetButton: function() {},

    onEditPress: function (evt) {
        var index = this.oView.table.getSelectedIndex();
        var context = this.oView.table.getContextByIndex(index);
        this.isNewModel = false;
        if (context) {
            var selectedData = context.getObject();
            logUtil.log(selectedData);

            // Get the model template data and open the dialog to edit
            this.getModelTemplate(selectedData);
        }
    },

    getModelTemplate: function (model)
    {
        var controller = this;
        var view = this.oView;

        var onSuccess = function (oJson)
        {
            logUtil.log("Model Template properties: ", oJson);

            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", controller);
            var title = sas.modelfactory.RB.getText(oJson.displayName);
            dialog.setTitle(title);
            dialog.addSaveButton();
            dialog.addSaveAsButton();
            dialog.open();
            // publish to event bus
            sas.eventBus.publish("ModelTemplateEvent", "editTemplate", {
                template: oJson});

        };


        // Get the model template data
        //modelProxy.getModelTemplate(modelId, templateId, onSuccess, onError);
        if (model && model.links) {
            var links = model.links;
            messageProxy.request(links, "template", "GET", "json", null, onSuccess, null);
        }
    },


    onTrainAction: function (evt)
    {
        var controller = this;
        var view = this.oView;
        var id = evt.getParameter("id");
        var menuItem = id.substring(id.lastIndexOf("--")+2, id.length);
        logUtil.log("onTrainAction menuItem: ", id);

        var tempStr = "";
        var action;
        var indices = this.oView.table.getSelectedIndices();

        switch (menuItem)
        {
            case "runBtn":
                var count = indices.length;
                var skipCount = 0;

                for (var i=0; i<indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                    {
                        var selectedData = context.getObject();

                        var onRunSuccess = function(oJson){
                            // Have jobs started?
                            var jobs = oJson.jobs;
                            if (jobs && jobs.length === 0){
                                // Model has no jobs to run.
                                skipCount++;
                            }
                        };

                        var onRunError = function(xhr, err, msg){
                            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.runModel.txt");
                            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
                            sas.hc.m.MessageBox.error(errMsg, title);
                        };

                        var onRunComplete = function(oJson){
                            count--;
                            logUtil.log("Count:", count);
                            if (count === 0 && controller.projectId && controller.segmentId){
                                busySpinner.show(false);

                                setTimeout(function(){sas.eventBus.publish("ProjectEvent", "running", {});}, 2000);

                                // Update the model list
                                controller.getModels(controller.projectId, controller.segmentId);
                            }
                        };

                        // Run the specific model
                        modelProxy.runModel(selectedData.id, onRunSuccess, onRunError, onRunComplete);

                        busySpinner.show(true,
                            sas.modelfactory.RB.getText("projectView.busyDialog.title.txt"),
                            sas.modelfactory.RB.getText("projectView.busyDialog.scheduleRun.txt")
                        );

                        // Update the status locally
                        var path = "/models[" + indices[i] + "]/status";
                        view.table.getModel().setProperty(path, controller.statusMap["SCHEDULED"]);

                        // Clear the result cache
                        var reportModel = view.getModel("results");
                        if (reportModel){
                            reportModel.setProperty("/reports/"+selectedData.id, null);
                        }

                        // Remove the last reports
                        var reports = view.graphGrid.getContent();
                        if (reports){
                            controller.resultsLoaded = false;
                            view.graphGrid.removeAllContent();
                            view.getModel().setProperty("/reports", null);
                        }
                    }
                }

                break;
            case "stopBtn":
                var count = indices.length;
                busySpinner.show(true,
                    sas.modelfactory.RB.getText("projectView.busyDialog.cancel.title.txt"),
                    sas.modelfactory.RB.getText("projectView.busyMsg.cancelJobs.txt")
                );
                for (var i=0; i<indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    var selectedData = context.getObject();
                    if (context)
                    {
                        var onCancelSuccess = function(oJson){

                        };

                        var onCancelError = function(xhr, err, msg){
                            var errMsg = sas.modelfactory.RB.getText("projectResults.error.stopRun.txt");
                            var title = sas.modelfactory.RB.getText("projectResults.error.title.txt");
                            sas.hc.m.MessageBox.error(errMsg, title);
                        };

                        var onCancelComplete = function(oJson){
                            count--;
                            if (count === 0 && controller.project){
                                busySpinner.show(false);

                                if (controller.project && controller.segmentId){
                                    // Update the model list
                                    controller.getModels(controller.projectId, controller.segmentId);
                                }
                            }
                        };

                        // Cancel the job for the model
                        jobProxy.cancelJob(controller.project.id, controller.project.jobTicketId, null, selectedData.jobId, onCancelSuccess, onCancelError, onCancelComplete);
                    }
                }
                break;
            case "compareBtn":
                if (indices.length > 1) {
                    busySpinner.show(true,
                        sas.modelfactory.RB.getText("model.comparison.busyDialog.title.txt"),
                        sas.modelfactory.RB.getText("model.comparison.busyDialog.generatingCompare.txt")
                    );

                    var onModelCompare = function(oJson)
                    {
                        busySpinner.show(false);
                        // Create the comparison report
                        jQuery.sap.require("sas.modelfactory.report.ExpandedGraphDialog");

                        var dialog = new sas.modelfactory.report.ExpandedGraphDialog({
                            content: sap.ui.view({
                                type: sap.ui.core.mvc.ViewType.JS,
                                id: controller.createId("Comparison"),
                                viewName: "sas.modelfactory.report.Comparison",
                                viewData: oJson
                            })
                        });
                        dialog.closeButton.setText(sas.modelfactory.RB.getText("button.close.txt"));
                        dialog.closeButton.setIcon(null);
                        dialog.setVerticalScrolling(true);

                        // Build a list of selected model names for the title
                        var str = sas.modelfactory.RB.getText("model.comparison.title.txt");
                        for (var i = 0; i < indices.length; i++) {
                            var context = controller.oView.table.getContextByIndex(indices[i]);
                            if (context) {
                                if (i > 0)
                                    str = str + ",";
                                str = str + " " + context.getObject().name;
                            }
                        }

                        dialog.setTitle(str);

                        // Due to a timing issue with Chrome, delay the dialog open.  This is to correct an issue
                        // with the column headings in the tables not being aligned properly with the columns.
                        setTimeout(function(){dialog.open();}, 0);
                    };

                    var onError = function(xhr, err, msg){
                        busySpinner.show(false);
                        var errMsg = sas.modelfactory.RB.getText("trainView.errorMsg.modelCompare.txt");
                        var title = sas.modelfactory.RB.getText("trainView.errorMsg.title.txt");
                        sas.hc.m.MessageBox.error(errMsg, title);
                    };

                    // Build a list of selected modelIds
                    var modelIds = [];
                    for (var i=0; i < indices.length; i++) {
                        var context = this.oView.table.getContextByIndex(indices[i]);
                        if (context)
                            modelIds.push(context.getObject().id);
                    }

                    // Get comparison data on the selected models
                    modelProxy.getComparisonResults(controller.projectId, modelIds, onModelCompare, onError);

                }
                break;
            case "deleteBtn":
                this.deleteModel(indices);
                break;
            case "duplicateBtn":
                var context = this.oView.table.getContextByIndex(indices[0]);
                var selectedData = context.getObject();
                if (selectedData) {
                    this.duplicateModel(selectedData);
                }
                break;
            case "ensembleBtn":
                action = "Ensemble";
                break;
            case "championBtn":
            case "unChampionBtn":
                var context = this.oView.table.getContextByIndex(indices[0]);
                var selectedData = context.getObject();
                if (selectedData) {
                    this.championOverride(selectedData);
                }
                break;
            case "renameBtn":
                var context = this.oView.table.getContextByIndex(indices[0]);
                var selectedData = context.getObject();
                if (selectedData) {
                    var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
                    dialog.renameHandler = this.renameModel;
                    var model = new sap.ui.model.json.JSONModel();
                    model.setData({model: selectedData,
                        modelList: controller.oView.table.getModel().oData.models});
                    dialog.bindElement("/model");
                    dialog.uniqueNamePath = "/modelList";
                    dialog.setNameProperty("name");
                    dialog.setModel(model);
                    dialog.open();
                }
                break;
            case "reportBtn":
                // Build a list of selected model ids
                var modelIds = [];
                for (var i=0; i < indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                        modelIds.push(context.getObject().id);
                }
                logUtil.log("report for ids:", modelIds);
                if (modelIds.length  > 0){
                    // Prepare the data for a report
                    controller.reportFragData = {
                        projectId: controller.projectId,
                        segmentId: controller.segmentId,
                        reportType: "model",
                        items: modelIds
                    };

                    // Open the report dialog
                    var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ReportDialog", this);
                    dialog.open();
                }
                break;
            case "logBtn":
                var context = this.oView.table.getContextByIndex(indices[0]);
                if (context) {
                    var project = context.getObject();
                    if (project) {
                        var links = project.links;
                        if (links && links.length > 0) {
                            var url = messageProxy.findUrl(links, "log", "GET");
                            //window.open(url, "_blank");
                            var downloadFrame = document.createElement("iframe");
                            downloadFrame.setAttribute('src',url);
                            document.body.appendChild(downloadFrame);
                        }
                    }
                }
                break;
            case "scoreBtn":
                // Build a list of selected model ids
                var modelIds = [];
                for (var i=0; i < indices.length; i++) {
                    var context = this.oView.table.getContextByIndex(indices[i]);
                    if (context)
                        modelIds.push(context.getObject().id);
                }
                logUtil.log("score code for ids:", modelIds);

                // Build the report options
                var championModels = false;

                // Request download of the score code zip file
                scoreCodeProxy.downloadScoreCode(controller.projectId, modelIds, null, championModels);
                break;
        }

        if (action && action.length > 0)
            sap.m.MessageToast.show(action + " for " + indices);
    },

    renameModel: function(dialog) {
        var controller = this;
        if (dialog) {

            logUtil.log(dialog);
            var model = dialog.getModel();
            var m = model.getProperty("/model");

            var models = controller.oView.table.getModel().oData.models;
            busySpinner.show(true,
                sas.modelfactory.RB.getText("trainView.rename.busyDialog.title.txt"),
                sas.modelfactory.RB.getText("trainView.rename.busyDialog.description.txt")
            );

            if (m) {
                // controller.updateModel(m);
                var onSuccess = function(json) {
                    // Templated updated, refresh the model list.
                    if (controller.projectId && controller.segmentId) {
                        controller.getModels(controller.projectId, controller.segmentId);
                    }
                };
                var onError = function(xhr, err, msg){
                    // display an error
                    var errResponse = jQuery.parseJSON(xhr.responseText);
                    var title = sas.modelfactory.RB.getText("trainView.rename.error.title.txt");
                    sas.hc.m.MessageBox.error(errResponse.message, title);
                };

                var onComplete = function() {
                    busySpinner.show(false);
                };

                modelProxy.renameModel( m.id, m.displayName, onSuccess, onError, onComplete);
            }
        }
    },

    onYesConfirmDelete: function(selectedModelIndices) {

        var controller = this;

        var onRowSelectionChanged = function() {
            // Always revert to the first row (champion model) after a model delete
            controller.oView.table.setSelectedIndex(0);
            controller.onRowSelectionClick();
        }

        var onSuccess = function(json) {
            // Templated updated, refresh the model list.
            if (controller.projectId && controller.segmentId) {
                controller.getModels(controller.projectId, controller.segmentId, onRowSelectionChanged);
            }
        };
        var onError = function(xhr, err, msg){
            // display an error
            var errResponse = jQuery.parseJSON(xhr.responseText);
            var title = sas.modelfactory.RB.getText("trainView.error.delete.title.txt");
            sas.hc.m.MessageBox.error(errResponse.message, title);
        };
        var onComplete = function () {
            busySpinner.show(false);
        }

        busySpinner.show(true,
            sas.modelfactory.RB.getText("trainView.delete.busyDialog.description.txt")
        );

        var selectedModels = [];
        for (i = 0; i < selectedModelIndices.length; ++i) {
            var selectedModel = this.oView.table.getContextByIndex(selectedModelIndices[i]).getObject()
            if ((selectedModel && selectedModel.miningAlgorithm.internalName !== "Baseline")){
                selectedModels.push(this.oView.table.getContextByIndex(selectedModelIndices[i]).getObject());
            }
        }
        if (selectedModels.length > 0){
            modelProxy.deleteModels(selectedModels, onSuccess, onError, onComplete);
        }
    },

    onNoConfirmDelete: function (selectedModel) {
        return;
    },

    deleteModel: function(selectedModelIndices) {

        var controller = this;
        var promptMessage = null;
        var deletingChampion = false;
        var foundBaseline = false;
        var deleteModelsCount = 0;
        var selectedModel = null;
        for (var i=0; i < selectedModelIndices.length; i++) {
            var context = this.oView.table.getContextByIndex(selectedModelIndices[i]);
            if (context){
                selectedModel = context.getObject();
                if (selectedModel.champion){
                    deletingChampion = true;
                }

                if (selectedModel.miningAlgorithm.internalName === "Baseline"){
                    foundBaseline = true;
                }
            }
        }

        if (foundBaseline) {
            promptMessage = sas.modelfactory.RB.getText("trainView.cantDeleteBaseline.txt")
            deleteModelsCount = selectedModelIndices.length - 1 ;
            if ((deleteModelsCount === 1) && (selectedModel.miningAlgorithm.internalName === "Baseline")) {
                context = this.oView.table.getContextByIndex(selectedModelIndices[0]);
                selectedModel = context.getObject();
            }
        } else {
            deleteModelsCount = selectedModelIndices.length;
        }

        if (deletingChampion) {
            promptMessage = sas.modelfactory.RB.getText("trainView.confirmDeleteChampion.txt")  + (promptMessage ? promptMessage : "");
        };

        if (deleteModelsCount > 1) {
            promptMessage = (promptMessage? promptMessage : "") + sas.modelfactory.RB.getText("trainView.confirmDeleteMultipleModels.txt")+ " " + deleteModelsCount + " " +
                sas.modelfactory.RB.getText("trainView.confirmDeleteCount.txt");
        };

        if (deleteModelsCount === 1) {
            promptMessage = (promptMessage? promptMessage : "") + sas.modelfactory.RB.getText("trainView.confirmDeleteModel.txt")  + " " + selectedModel.name;
        };

        var confirmDialog =  sap.ui.jsfragment("sas.modelfactory.dialog.ConfirmDialog", this);
        confirmDialog.initDialog(sas.modelfactory.RB.getText("trainView.confirmDeleteChampion.title.txt"),
            promptMessage,
            controller.onYesConfirmDelete,
            controller.onNoConfirmDelete,
            selectedModelIndices);
        confirmDialog.setInitialFocus(confirmDialog.getEndButton());
        confirmDialog.open();
        return;
    },

    duplicateModel: function(selectedModel) {
        var controller = this;
        busySpinner.show(true,
            sas.modelfactory.RB.getText("trainView.duplicate.busyDialog.title.txt"),
            sas.modelfactory.RB.getText("trainView.duplicate.busyDialog.description.txt")
        );

        var onSuccess = function(json) {
            // Templated updated, refresh the model list.
            if (controller.projectId && controller.segmentId) {
                controller.getModels(controller.projectId, controller.segmentId);
            }
        };
        var onError = function(xhr, err, msg){
            // display an error
            var errResponse = jQuery.parseJSON(xhr.responseText);
            var title = sas.modelfactory.RB.getText("trainView.error.duplicateModel.title.txt");
            sas.hc.m.MessageBox.error(errResponse.message, title);
        };

        var onComplete = function () {
            busySpinner.show(false);
        }

        var dupModel = {
            name :  (sas.modelfactory.RB.getText("trainView.copyOf.txt") +" " + selectedModel.name).substring(0, 100),
            id : selectedModel.id
        };

        var models = this.oView.table.getModel().oData.models;
        // Test for 'Copy of <origname>'
        var nameCollision = false;
        for (var i=0; i<models.length; i++) {
            if (models[i].name === dupModel.name){
                nameCollision = true;
                break;
            }
        };

        // If there was a name collision ... try 'Copy (2) of <origName>'
        var nameCopyCounter = 2;
        while (nameCollision) {
            dupModel.name = (sas.modelfactory.RB.getText("trainView.copyCount.txt") + nameCopyCounter + ") " + selectedModel.name).substring(0, 100);
            nameCollision = false;
            for (var i=0; i<models.length; i++) {
                if (models[i].name === dupModel.name){
                    nameCollision = true;
                    break;
                }
            };
            nameCopyCounter ++;
        }

        modelProxy.duplicateModel(this.projectId, this.segmentId, dupModel, onSuccess, onError, onComplete);
    },

    championOverride: function(selectedModel) {
        var controller = this;
        onSuccess = function(oJson) {
            sas.eventBus.publish("ChampionOverrideChanged", controller.projectId, null);
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("trainView.errorMsg.championOverride.txt");
            var title = sas.modelfactory.RB.getText("trainView.errorMsg.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            // Update the model list
            if (controller.projectId && controller.segmentId){
                controller.getModels(controller.projectId, controller.segmentId);
            }
        };

        // This logic must parallel the button label logic
        if(selectedModel.champion && selectedModel.championOverridden) {
            modelProxy.championCalculate(selectedModel.id, onSuccess, onError, onComplete);
        } else {
            modelProxy.championOverride(selectedModel.id, onSuccess, onError, onComplete);
        }
    },

    updateModel: function(model) {
        var controller = this;
        if (model) {
            logUtil.log("Update model:", model);

            onSuccess = function(oJson) {

            },

                onError = function(xhr, err, msg){
                    var errMsg = sas.modelfactory.RB.getText("trainView.errorMsg.updateModel.txt");
                    var title = sas.modelfactory.RB.getText("trainView.errorMsg.title.txt");
                    sas.hc.m.MessageBox.error(errMsg, title);
                };

            var onComplete = function(oJson){
                // Update the model list
                if (controller.projectId && controller.segmentId){
                    controller.getModels(controller.projectId, controller.segmentId);
                }
            }

            //modelProxy.updateModel(onSuccess, onError, onComplete);
            logUtil.log("TODO: add updateModel handler");
        }
    },

    getModels: function (projectId, segmentId, onCompleteCallback)
    {
        var view = this.oView;
        var controller = this;

        var onSuccess = function (oJson)
        {
            if (!view.loaded){
                var model = new sap.ui.model.json.JSONModel();
                model.setData({models: oJson.items});
                view.setAndBindModel(model, "/models");

                // Update table with column preferences
                var key = "FactoryMiner.TrainColumnPreferences.";
                if (controller.getTargetType() === "INTERVAL")
                    key = key + "Numeric";
                else
                    key = key + "Class";

                var getPreferenceCallback = function(preferenceId) {
                    controller.preferenceId = preferenceId;
                }

                sas.modelfactory.util.PreferencesUtil.getColumnPref(controller.project.id, key, view.table, getPreferenceCallback);

            } else {
                // Update the data
                view.table.getModel().setProperty("/models", oJson.items);
                view.updateFitsLabels(oJson.items);
            }

            // Adjust table length
            var tableLen = oJson.items.length;
            if (tableLen > 8){
                tableLen = 8;
            }
            view.table.setVisibleRowCount(tableLen);

            // If not already started, poll model results
            if (!controller.isPolling) {
                controller.pollRunningModels(projectId, segmentId);
            }
        };

        var onComplete = function (oJson)
        {
            var view = controller.oView;
            var indices = view.table.getSelectedIndices();
            controller.processControls(indices);

            if (!controller.dataLoaded) {
                controller.dataLoaded = true;

                if (view.table.getModel().oData.models && view.table.getModel().oData.models.length > 0) {
                    // Select the first row in the table
                    view.table.setSelectedIndex(0);
                }
            } else {
                if (controller.currentSelectedRows && controller.currentSelectedRows.length > 0){
                    // Set the selection
                    controller.setSelectedRows();

                    // Only show the first row selected
                    var firstRow = controller.getIndexForId(controller.currentSelectedRows[0]);
                    logUtil.log("---------------------> first row:", firstRow);
                    var context = view.table.getContextByIndex(firstRow);
                    if (context){
                        var selectedModel = context.getObject();
                        if (selectedModel){
                            if (selectedModel.status && selectedModel.status.key === "COMPLETED" && !controller.resultsLoaded){
                                // Get the results
                                controller.getModelResults(selectedModel);
                            }
                        }
                    }
                }
            }

            // Cache model results
            if (!controller.resultsCached){
                controller.resultsCached = true;
                controller.cacheModelResults();
            }

            // Allow the caller of getModels() to know when the GET is complete
            if (onCompleteCallback)
                onCompleteCallback();
        };

        segmentsProxy.getModelsForSegment(projectId, segmentId, onSuccess, null, onComplete);
    },

    pollRunningModels: function(projectId, segmentId) {
        var view = this.oView;
        var controller = this;
        var modelList = view.table.getModel().oData.models;
        var isRunning = false;
        for(var i=0;i<modelList.length;i++){
            if (modelList[i].status !== undefined){
                if (modelList[i].status.key === "RUNNING" || modelList[i].status.key === "SCHEDULED"){
                    isRunning = true;
                    break;
                }
            }
        }

        // If any models are RUNNING, poll until finished
        if (isRunning && !controller.stopPolling){
            // Get the models for the selected segment
            logUtil.log("********** POLLING FOR RUNNING MODELS", Date(), "**********");
            controller.isPolling = true;
            if (projectId && segmentId){
                setTimeout(function(){
                    controller.isPolling = false;
                    controller.getModels(projectId, segmentId);
                }, controller.CONST_POLL_TIME);
            }
        } else {
            controller.isPolling = false;

            // Clear any pending results in error state
            for(var i=0;i<controller.pendingRequestsArray.length;i++){
                var req = controller.pendingRequestsArray[i];
                if (req !== undefined && req && req.inError){
                    delete controller.pendingRequestsArray[i];
                }
            }
        }
    },

    modelDataLoadedHandler: function (channelId, eventId, data) {
        var model = new sap.ui.model.json.JSONModel();
        model.setData({algorithms: data.template.items});
        this.newModelDialog.modelAlgorithm.setModel(model);
    },

    onColumnMove: function(evt) {
        logUtil.log("Model table column moved:", evt.getParameter("column").sId, "moved to", evt.getParameter("newPos"));

        // A column has moved
        var params = evt.getParameters();
        this.columnsMoved(params);
    },

    onColumnManagerClose: function() {
        var controller = this;
        var view = controller.getView();

        // Store column preferences
        var key = "FactoryMiner.TrainColumnPreferences.";
        if (controller.getTargetType() === "INTERVAL")
            key = key + "Numeric";
        else
            key = key + "Class";

        var storeSuccess = function(args) {
            controller.preferenceId = args.id;
        }

        sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "train_", view.table, controller.preferenceId, storeSuccess);
    },

    columnsMoved: function(params) {
        logUtil.log("Columns changed.  Do something with columns.")
        var view = this.getView();
        if (params) {
            var controller = this;
            var colMoved = params.column;
            var pos = params.newPos;
            var table = controller.getView().table;
            var cols = null;
            if (table){
                cols = table.getColumns();
            }

            // Update the columms
            if (table && cols){
                for(var i=0;i<cols.length;i++){
                    if (cols[i].getId() === colMoved.getId()){
                        cols.splice(i, 1);
                        break;
                    }
                }
                cols.splice(pos, 0, colMoved);
            }
            table.removeAllColumns();
            for(var j=0;j<cols.length;j++){
                table.addColumn(cols[j]);
            }

            // Store column preferences
            var key = "FactoryMiner.TrainColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";

            var storeSuccess = function(args) {
                controller.preferenceId = args.id;
            }

            sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "train_", view.table, controller.preferenceId, storeSuccess);
        }
    },

    getTargetType: function() {
        var targetType = "UNDEFINED";
        var targetModel = sap.ui.getCore().getModel("targetType");
        if (targetModel){
            var level = targetModel.getProperty("/level");
            if (level){
                targetType = level.key;
                logUtil.log("Project target type:", targetType);
            }
        }
        return targetType;
    },

    cacheModelResults: function(){
        var controller = this;
        var view = this.oView;
        var model = view.table.getModel();
        var models = null;
        if (model){
            models = model.getProperty("/models");
        }
        var reportModel = new sap.ui.model.json.JSONModel();
        reportModel.setData({"reports": []});

        var onSuccess = function(reports){
            if (reports){
                var uri = this.url;
                var id = "";
                if (uri){
                    id = uri.slice(uri.indexOf("models/")+7, uri.indexOf("/results"));
                    logUtil.log("Model id:", id);
                    if (id){
                        var modelUpdated = null;
                        for (var j=0;j<models.length;j++){
                            if (models[j].id === id){
                                // Update the model
                                modelUpdated = models[j];
                                reportModel.setProperty("/reports/"+id, reports);
                                view.setModel(reportModel, "results");
                                logUtil.log("Update model:", reportModel.getProperty("/reports/"+id));
                                break;
                            }
                        }

                        // If we have a callback, call it
                        if (controller.hasCallback(id) && modelUpdated){
                            controller.getModelResults(modelUpdated);
                            logUtil.log("cacheModelResults: callback called for", id);
                        }
                    }
                }

                view.setModel(reportModel, "results");
            }
        };

        var onError = function(xhr, err, msg){
        };

        var onComplete = function(oJson){
            var uri = this.url;
            var id = "";
            if (uri){
                id = uri.slice(uri.indexOf("models/")+7, uri.indexOf("/results"));
                logUtil.log("Model id:", id);
                if (id){
                    controller.removePendingRequest(id);
                }
            }
        };

        if (models && models.length > 0){
            for (var i=0;i<models.length;i++){
                // Get the model results
                var selectedModel = models[i];
                if (selectedModel.status && selectedModel.status.key === "COMPLETED"){
                    if (!controller.isPending(selectedModel.id)){
                        controller.addPendingRequest(selectedModel.id);
                        modelProxy.getModelResults(selectedModel.id, onSuccess, onError, onComplete);
                    }
                }
            }
        }
    },

    addPendingRequest: function(id, hasCallback) {
        if (!this.isPending(id)){
            logUtil.log("addPendingRequest:", id, "hasCallback:", hasCallback);
            var entry = {
                "id": id,
                "hasCallback": hasCallback!==undefined?hasCallback:false
            };
            this.pendingRequestsArray.push(entry);
        }
        logUtil.log("pendingRequestsArray:", this.pendingRequestsArray);
    },

    updatePendingRequest: function(id, hasCallback, error) {
        for(var i=0;i<this.pendingRequestsArray.length;i++){
            var req = this.pendingRequestsArray[i];
            if (req !== undefined && req && req.id === id){
                if (hasCallback !== undefined){
                    req.hasCallback = hasCallback;
                }
                if (error !== undefined){
                    req.inError = error;
                }
                logUtil.log("updatePendingRequest:", id, "hasCallback:", hasCallback, "inerror", error);
            }
        }
    },

    removePendingRequest: function(id) {
        logUtil.log("removePendingRequest:", id);
        for(var i=0;i<this.pendingRequestsArray.length;i++){
            var req = this.pendingRequestsArray[i];
            if (req !== undefined && req && req.id === id){
                delete this.pendingRequestsArray[i];
            }
        }
        //logUtil.log("pendingRequestsArray:", this.pendingRequestsArray);
    },

    isPending: function(id) {
        //logUtil.log("pendingRequestsArray:", this.pendingRequestsArray);
        for(var i=0;i<this.pendingRequestsArray.length;i++){
            var req = this.pendingRequestsArray[i];
            if (req !== undefined && req && req.id === id){
                logUtil.log("isPending true:", id);
                return true;
            }
        }
        logUtil.log("isPending false:", id);
        return false;
    },

    hasCallback: function(id) {
        //logUtil.log("pendingRequestsArray:", this.pendingRequestsArray);
        for(var i=0;i<this.pendingRequestsArray.length;i++){
            var req = this.pendingRequestsArray[i];
            if (req !== undefined && req && req.id === id && req.hasCallback){
                logUtil.log("pending request has callback - true:", id);
                return true;
            }
        }
        logUtil.log("pending request has callback - false:", id);
        return false;
    },

    getPendingRequest: function(id) {
        var req = null;
        for(var i=0;i<this.pendingRequestsArray.length;i++){
            req = this.pendingRequestsArray[i];
            if (req !== undefined && req && req.id === id){
                return req;
            }
        }
        logUtil.log("pending request returned:", req);
        return req;
    }
});