/**
 * Created by rayoun on 11/11/14.
 */

jQuery.sap.declare("sas.modelfactory.settings.ModelFactorySettingsProvider");
jQuery.sap.require(
    "sas.hc.ui.commons.settings.providers.AbstractSettingsProvider",
    "sas.hc.ui.commons.formControl.Field",
    "sas.hc.ui.commons.formControl.Group",
    "sas.modelfactory.proxies.ChampionProxy"
);


(function() {
    var Field = sas.hc.ui.commons.formControl.Field;

    sas.hc.ui.commons.settings.providers.AbstractSettingsProvider.extend(
        "sas.modelfactory.settings.ModelFactorySettingsProvider", {

            classChampionCriteriaList: null,
            numericChampionCriteriaList: null,
            defaultSettings: null,
            sampleCheckField: null,
            sampleMethField: null,
            samplePerField: null,
            samplePerValueField: null,
            sampleNobField: null,
            sampleNobValueField: null,
            partCheckField: null,
            partTrainField: null,
            partValField: null,
            depthField: null,

            /**
             *
             * @override
             * @returns {sas.hc.ui.commons.formControl.Group}
             */
            getPreferences: function() {
                return this.createGroup(
                    sas.modelfactory.RB.getText("settings.group.title.txt"),
                    [],
                    [
                        this._createSub1Group(),
                        this._createSub2Group(),
                        this._createSub3Group()
                    ]
                );
            },

            /**
             * @private
             */
            _createSub1Group: function() {
                return this.createGroup(sas.modelfactory.RB.getText("settings.subGroup1.title.txt"), [
                    this._createField1(),
                    this._createField2(),
                    this._createField3(),
                    this._createField3a(),
                    this._createField4(),
                    this._createField4a()
                ]);
            },

            /**
             * @private
             */
            _createSub2Group: function() {
                return this.createGroup(sas.modelfactory.RB.getText("settings.subGroup2.title.txt"), [
                    this._createField5(),
                    this._createField6(),
                    this._createField7()
                ]);
            },

            /**
             * @private
             */
            _createSub3Group: function() {
                return this.createGroup(sas.modelfactory.RB.getText("settings.subGroup3.title.txt"), [
                    this._createField10(),
                    this._createField11(),
                    this._createField8(),
                    this._createField8a(),
                    this._createField9()
                ]);
            },

            /**
             * Sample data settings
             */
            _createField1: function() {
                var me = this;
                this.sampleCheckField = this.createField(
                    "FACTORYMINER_SAMPLE_ENABLE_SAMPLING",
                    Field.BOOLEAN_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.sample.checkbox.txt"),
                    this.defaultSettings?this.defaultSettings.sampleDataSelected:false
                );
                this.sampleCheckField.setSelectionType("multiple");
                this.sampleCheckField.attachLiveChange(me, this.handleSampleChange);
                return this.sampleCheckField;
            },

            _createField2: function() {
                var sampleMethods = [
                    {text: sas.modelfactory.RB.getText("edit.partition.sampling.default.txt"), key: "DEFAULT"},
                    {text: sas.modelfactory.RB.getText("edit.partition.sampling.simplerandom.txt"), key: "RANDOM"},
                    //{text: sas.modelfactory.RB.getText("edit.partition.sampling.levelbased.txt"), key: "EVENT"},
                    {text: sas.modelfactory.RB.getText("edit.partition.sampling.stratified.txt"), key: "STRATIFIED"}
                ];
                this.sampleMethField = this.createField(
                    "FACTORYMINER_SAMPLE_METHOD",
                    Field.TEXT_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.sampling.label.txt"),
                    sampleMethods,
                    this.defaultSettings?this.defaultSettings.sampleDataMethod:"DEFAULT"
                );
                this.sampleMethField.setEnabled(this.sampleCheckField.getValue());
                return this.sampleMethField;
            },

            _createField3: function() {
                var self = this;
                this.samplePerField = this.createField(
                    "FACTORYMINER_SAMPLE_PERCENTAGE",
                    Field.BOOLEAN_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.percentage.radio.txt"),
                    this.defaultSettings?(this.defaultSettings.sampleUsingPercentageOrNObs==="PERCENT"):true
                );
                this.samplePerField.setName("FACTORYMINER_SAMPLE_TYPE_PER");
                this.samplePerField.setSelectionType("single");
                this.samplePerField.setEnabled(this.sampleCheckField.getValue());
                this.samplePerField.attachLiveChange(function(evt){
                    var value = evt.getParameter("newValue");
                    if (value !== undefined && self.sampleNobField !== undefined && self.sampleNobValueField !== undefined){
                        self.sampleNobField.setValue(!value);
                        self.sampleNobValueField.setEnabled(!value && self.sampleCheckField.getValue());
                    }
                });
                return this.samplePerField;
            },

            _createField3a: function() {
                this.samplePerValueField = this.createField(
                    "FACTORYMINER_SAMPLE_PERCENTAGE_VALUE",
                    Field.INTEGER_TYPE,
                    "",
                    this.defaultSettings?this.defaultSettings.sampleDataPercentage:20
                );
                this.samplePerValueField.setProperty("constraints", {min: 0, max: 100});
                this.samplePerValueField.setValidator({required: true});
                this.samplePerValueField.setEnabled(this.sampleCheckField.getValue());
                return this.samplePerValueField;
            },

            _createField4: function() {
                var self = this;
                this.sampleNobField = this.createField(
                    "FACTORYMINER_SAMPLE_NUMOBS",
                    Field.BOOLEAN_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.numobs.radio.txt"),
                    this.defaultSettings?(this.defaultSettings.sampleUsingPercentageOrNObs==="NOBS"):false
                );
                this.sampleNobField.setName("FACTORYMINER_SAMPLE_TYPE_NOBS");
                this.sampleNobField.setSelectionType("single");
                this.sampleNobField.setEnabled(this.sampleCheckField.getValue());
                this.sampleNobField.attachLiveChange(function(evt){
                    var value = evt.getParameter("newValue");
                    if (value !== undefined && self.samplePerField !== undefined && self.samplePerValueField !== undefined){
                        self.samplePerField.setValue(!value);
                        self.samplePerValueField.setEnabled(!value && self.sampleCheckField.getValue());
                    }
                });
                return this.sampleNobField;
            },

            _createField4a: function() {
                this.sampleNobValueField = this.createField(
                    "FACTORYMINER_SAMPLE_NUMOBS_VALUE",
                    Field.INTEGER_TYPE,
                    "",
                    this.defaultSettings?this.defaultSettings.sampleDataNObs:1
                );
                this.sampleNobValueField.setProperty("constraints", {min: 1});
                this.sampleNobValueField.setValidator({required: true});
                this.sampleNobValueField.setEnabled(this.sampleCheckField.getValue());
                return this.sampleNobValueField;
            },

            /**
             * Partition data settings
             */
            _createField5: function() {
                var me = this;
                this.partCheckField = this.createField(
                    "FACTORYMINER_PARTITION_ENABLE_PARTITION",
                    Field.BOOLEAN_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.data.checkbox.txt"),
                    this.defaultSettings?this.defaultSettings.partitionDataSelected:true
                );
                this.partCheckField.setSelectionType("multiple");
                this.partCheckField.attachLiveChange(me, this.handlePartChange);
                return this.partCheckField;
            },

            _createField6: function() {
                this.partTrainField = this.createField(
                    "FACTORYMINER_PARTITION_TRAIN",
                    Field.INTEGER_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.training.label.txt"),
                    this.defaultSettings?this.defaultSettings.partitionDataTrainingPart:70
                );
                this.partTrainField.setProperty("constraints", {min: 0, max: 100});
                this.partTrainField.setValidator({required: true});

                this.partTrainField.setEnabled(this.partCheckField.getValue());
                return this.partTrainField;
            },

            _createField7: function() {
                this.partValField = this.createField(
                    "FACTORYMINER_PARTITION_VALIDATE",
                    Field.INTEGER_TYPE,
                    sas.modelfactory.RB.getText("edit.partition.validation.label.txt"),
                    this.defaultSettings?this.defaultSettings.partitionDataValidatingPart:30
                );
                this.partValField.setProperty("constraints", {min: 0, max: 100});
                this.partValField.setValidator({required: true});
                this.partValField.setEnabled(this.partCheckField.getValue());
                return this.partValField;
            },

            /**
             * Champion Criterion settings
             */
            _createField8: function() {
                var me = this;
                var f = me.createField(
                    "FACTORYMINER_INTERVAL_CHAMPION_CRITERION",
                    Field.TEXT_TYPE,
                    sas.modelfactory.RB.getText("championCriteria.interval.txt"),
                    this.numericChampionCriteriaList,
                    this.defaultSettings?this.defaultSettings.intervalCriterion:"DEFAULT"    // Default
                );
                //f.attachLiveChange(me, this.handleCriterionChange);
                return f;
            },

            /**
             * Champion Criterion settings
             */
            _createField8a: function() {
                var me = this;
                var f = me.createField(
                    "FACTORYMINER_CHAMPION_CRITERION",
                    Field.TEXT_TYPE,
                    sas.modelfactory.RB.getText("championCriteria.class.txt"),
                    this.classChampionCriteriaList,
                    this.defaultSettings?this.defaultSettings.nominalCriterion:"DEFAULT"    // Default
                );
                f.attachLiveChange(me, this.handleCriterionChange);
                return f;
            },

            _createField9: function() {
                var championDepthData = [
                    {text: 5, key: 5},
                    {text: 10, key: 10},
                    {text: 15, key: 15},
                    {text: 20, key: 20}
                ];
                this.depthField = this.createField(
                    "FACTORYMINER_CHAMPION_DEPTH",
                    Field.INTEGER_TYPE,
                    sas.modelfactory.RB.getText("championCriteria.depth.txt"),
                    championDepthData,
                    this.defaultSettings?this.defaultSettings.depth:10
                );
                return this.depthField;
            },

            _createField10: function() {
                var f = this.createField(
                    "FACTORYMINER_MINIMUM_OBSERVATIONS",
                    Field.INTEGER_TYPE,
                    sas.modelfactory.RB.getText("rules.minObservations.txt"),
                    this.defaultSettings?this.defaultSettings.minNObs:100
                );
                f.setProperty("constraints", {min: 1});
                f.setValidator({required: true});
                return f;
            },

            _createField11: function() {
                var f = this.createField(
                    "FACTORYMINER_MINIMUM_EVENTRATE",
                    Field.DOUBLE_TYPE,
                    sas.modelfactory.RB.getText("rules.minEventRate.txt"),
                    this.defaultSettings?this.defaultSettings.minEventRate:0.1
                );
                f.setProperty("constraints", {min: 0.0001, max: 0.9999});
                f.setValidator({required: true});
                f._addValidationFunc(function(sValue){
                    var oConstraints = f.getConstraints();
                    var bValid = true;
                    if (!!oConstraints) {
                        if (!isNaN(oConstraints.min)) {
                            bValid = bValid && (sValue >= Number(oConstraints.min));
                        }
                        if (!isNaN(oConstraints.max)) {
                            bValid = bValid && (sValue <= Number(oConstraints.max));
                        }
                    }
                    return bValid;
                });
                return f;
            },

            setData: function(data) {
                if (data) {
                    var championCriterion = data.championCriterion;
                    this.classChampionCriteriaList = championCriterion.classChamps;
                    this.numericChampionCriteriaList = championCriterion.numericChamps;
                    this.defaultSettings = data.settings;
                }
            },

            handleSampleChange: function(evt, self) {
                var value = evt.getParameter("newValue");
                if (self){
                    self.sampleMethField.setEnabled(value);
                    self.samplePerField.setEnabled(value);
                    self.samplePerValueField.setEnabled(value && self.samplePerField.getValue());
                    self.sampleNobField.setEnabled(value);
                    self.sampleNobValueField.setEnabled(value && self.sampleNobField.getValue());
                }
            },

            handlePartChange: function(evt, self) {
                var value = evt.getParameter("newValue");
                if (self){
                    self.partTrainField.setEnabled(value);
                    self.partValField.setEnabled(value);
                }
            },

            handleCriterionChange: function(evt, self) {
                var selectedId;
                var selections =  this.getProperty("dataProvider");
                var showDepth = false;
                var selectedItem = evt.getParameter("newValue");

                if (typeof selectedItem === "object") {
                    selectedId = selectedItem.getProperty("key");
                } else {
                    selectedId = selectedItem;
                }

                for (var i = 0; i < selections.length; i++) {
                    if (selections[i].key === selectedId) {
                        showDepth = selections[i].isDepthSupported;
                        break;
                    }
                }

                self.depthField.setVisible(showDepth);
            }
        }); //end of class declaration
})();