jQuery.sap.require(
    "sap.m.MessageToast",
    "sas.modelfactory.proxies.TemplateProxy",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.controller("sas.modelfactory.templates.ModelTemplates", {

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
	onInit: function() {
        // subscribe to the event bus
        sas.eventBus.subscribe("ModelTemplateEvent", "editTemplate", this.templateEventHandler, this);
        sas.eventBus.subscribe("ModelTemplateEvent", "newModelTemplate", this.templateEventHandler, this);
	},

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
    onBeforeRendering: function ()
    {
        this.reset();
    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
    onAfterRendering: function ()
    {

    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
	onExit: function() {
        this.reset();

        sas.eventBus.unsubscribe("ModelTemplateEvent", "editTemplate", this.templateEventHandler, this);
        sas.eventBus.unsubscribe("ModelTemplateEvent", "newModelTemplate", this.templateEventHandler, this);
	},

    templateEventHandler : function(channelId, eventId, data) {
        var controller = this;
        var view = this.oView;
        if (data && data.template) {
            logUtil.log("ModelTemplates: data received ->", data);

            // Get the segments for the project
            var model = new sap.ui.model.json.JSONModel();
            model.setData({template: data.template});
            view.setAndBindModel(model, "/template");

            // Bind to the model changing
            var binding = new sap.ui.model.Binding(model, "/", model.getContext("/"));
            var dialog = view.getViewData();
            if (dialog){
                var saveButton = dialog.saveButton;
                if (saveButton && saveButton !== undefined) {
                    // Start with the Save button disabled
                    saveButton.setEnabled(false);
                    if (data.saveEnabled){
                        saveButton.setEnabled(data.saveEnabled);
                    }
                    binding.attachChange(function() {
                        // Enable the Save button on model changes
                        saveButton.setEnabled(true);
                    });
                }
            }
        }
    },

    newModelEventHandler : function(channelId, eventId, data) {
        var controller = this;
        var view = this.oView;
        if (data && data.template) {
            logUtil.log("ModelTemplates: data received ->", data);

            // Get the segments for the project
            var model = new sap.ui.model.json.JSONModel();
            model.setData({template: data.template});
            view.setAndBindModel(model, "/template");
        }
    },

    reset: function() {
        this.oView.useList.destroyItems();
        this.oView.modelList.destroyItems();
        this.oView.ignoreList.destroyItems();
    },

    onComponentPress: function(evt){
        var list = evt.getParameter("id");
        var key = evt.getParameter("listItem").sId;
        logUtil.log("onComponentPress: ",key);

        // Manage list selection between Use and Ignore
        if (list === "TemplateView--UseList") {
            this.byId("IgnoreList").removeSelections();
            this.byId("ModelList").removeSelections();
        } else if (list === "TemplateView--ModelList") {
            this.byId("IgnoreList").removeSelections();
            this.byId("UseList").removeSelections();
        } else {
            this.byId("UseList").removeSelections();
            this.byId("ModelList").removeSelections();
        }

        this.oView.populateProperties(key);
    }
});