jQuery.sap.require(
    "sas.hc.m.Label",
    "sas.hc.m.Select",
    "sas.hc.m.Slider",
    "sas.hc.ui.commons.Slider",
    "sas.hc.m.MessageBox",
    "sas.hc.m.CheckBox",
    "sas.hc.m.Input",
    "sas.hc.ui.commons.TextField",
    "sas.hc.m.Toolbar",
    "sas.hc.m.VBox",
    "sas.hc.m.List",
    "sap.ui.layout.GridData",
    "sas.hc.m.Panel",
    "sas.hc.ui.commons.RadioButtonGroup",
    "sas.modelfactory.control.rangeinput.RangeInput",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.jsview("sas.modelfactory.templates.ModelTemplates", {

    rightSide: null,
    useList: null,
    useListOrder: null,
    modelList: null,
    ignoreList: null,
    ignoreListOrder: null,
    componentList: null,
    isEditable: true,
    maxPrinPath: null,
    fixPrinPath: null,

    /** Specifies the Controller belonging to this View.
     * In the case that it is not implemented, or that "null" is returned, this View does not have a Controller.
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
    getControllerName : function() {
        return "sas.modelfactory.templates.ModelTemplates";
    },

    /** Is initially called once after the Controller has been instantiated. It is the place where the UI is constructed.
     * Since the Controller is given to this method, its event handlers can be attached right away.
     * @memberOf sas.modelfactory.templates.ModelTemplates
     */
    createContent : function(oController) {

        var view = this;

        // Create the left side
        this.useList = new sas.hc.m.List(this.createId("UseList"), {
            mode: "SingleSelectMaster",
            noDataText: " ",
            showNoData: false,
            headerToolbar: new sas.hc.m.Toolbar({
                content: [
                    new sas.hc.m.Label({
                        text: sas.modelfactory.RB.getText("modelTemplatesView.list.use.txt")
                    })
                ]
            }),
            selectionChange: [oController.onComponentPress, oController]
        }).addStyleClass("connectedSortable");

        this.modelList = new sas.hc.m.List(this.createId("ModelList"), {
            mode: "SingleSelectMaster",
            selectionChange: [oController.onComponentPress, oController]
        }).addStyleClass("ModelTemplateView-useList");

        this.ignoreList = new sas.hc.m.List(this.createId("IgnoreList"), {
            mode: "SingleSelectMaster",
            noDataText: " ",
            showNoData: false,
            headerToolbar: new sas.hc.m.Toolbar({
                content: [
                    new sas.hc.m.Label({
                        text: sas.modelfactory.RB.getText("modelTemplatesView.list.ignore.txt")
                    })
                ]
            }),
            selectionChange: [oController.onComponentPress, oController]
        }).addStyleClass("ModelTemplateView-ignoreList").addStyleClass("connectedSortable");

        var leftSide = new sas.hc.m.VBox(this.createId("LeftSide"), {
            items: [
                this.useList,
                this.modelList,
                this.ignoreList
            ],
            layoutData: new sap.ui.layout.GridData({
                span : "L4 M4 S12"
            })
        }).addStyleClass("ModelTemplateView-leftLayout");

        // Create the right side
        this.rightSide = new sas.hc.m.Panel(this.createId("RightSide"), {
            /*headerToolbar: new sas.hc.m.Toolbar({
             content: [
             new sas.hc.m.Label({
             text: sas.modelfactory.RB.getText("modelTemplatesView.panel.title.txt")
             })
             ]
             }),*/
            layoutData: new sap.ui.layout.GridData({
                span : "L8 M8 S12"
            }),
            width: "100%"
        });

        var grid = new sap.ui.layout.Grid(this.createId("Grid"), {
            hSpacing: 1,
            vSpacing: 1,
            content: [
                leftSide,
                this.rightSide
            ]
        });

        return grid;
    },

    setAndBindModel: function(model, bindPath) {
        this.setModel(model);

        var template = this.getModel().getProperty("/template");
        if (template){
            // Is this template editable
            this.isEditable = template.miningAlgorithm.internalName !== "Baseline";
        }

        // Populate the form
        this.populateComponents();
    },

    populateComponents: function() {
        var controller = this;
        this.componentList = {};
        var firstUseItem = null;
        var lastUseItem = null;
        var firstModelItem = null;
        var model = this.getModel();
        var data = this.getModel().oData;
        var modelItem;
        var tempUseList = {};
        for (var i=0; i<data.template.properties.length; i++) {
            var compItem = data.template.properties[i];
            if (compItem) {
                //logUtil.log("ModelTemplates view component:", compItem.groupName, compItem.componentName, compItem.propertyName, compItem.propertyValue, compItem.position);
                if (compItem.groupName === "Modify") {
                    if (compItem.propertyName === "_use") {
                        // Temporary!!!!
                        if (compItem.componentName != "Sample" &&
                            compItem.componentName != "Partition") {
                            // Find the components "_use" property.
                            var item = new sas.hc.m.StandardListItem({
                                title: sas.modelfactory.RB.getText(compItem.componentDisplayName),
                                icon: sas.icons.HC.GRABBERVERTICAL,
                                id: compItem.componentName
                            }).addStyleClass("ModelTemplateView-listItem");
                            item.setTooltip(sas.modelfactory.RB.getText(compItem.propertyDescription));
                            /*if (compItem.componentName == "Partition" ||
                             compItem.componentName == "Sample")
                             item.setVisible(false);*/

                            // If the "_use" flag is "Y" add to the Use list.
                            // If the "_use" flag is "N" add to the Ignore list.
                            var pos = compItem.position - 3; // Temporary!!!!
                            //var pos = compItem.position - 1; // convert to zero-based
                            if (compItem.propertyValue === "Y") {
                                //this.useList.insertItem(item, pos);
                                // Gather items in tempUseList to build the useList later
                                tempUseList[pos] = {"name": compItem.componentName, "item": item, "pos": pos};
                                logUtil.log("insert item in use list:", compItem.componentName, "pos:", pos);
                                logUtil.log("use list items:", this.useList.getItems());

                                // Remember the first component in the Use list.
                                if (pos === 0)
                                    firstUseItem = item;
                            } else {
                                this.ignoreList.addItem(item);
                                logUtil.log("ignore list items:", this.ignoreList.getItems());
                            }
                        }
                    } else {
                        // Add the component to the component list
                        var key = compItem.componentName;
                        var propName = compItem.propertyName;

                        if (this.componentList[key]) {
                            var propList = this.componentList[key];
                            propList[propName] = {data: compItem, index: i};
                        } else {
                            var propList = {};
                            propList[propName] = {data: compItem, index: i};
                            this.componentList[key] = propList;
                        }
                    }
                }

                // Get the algorithm and add it to the bottom of the Use list.
                if (compItem.groupName === "Model" || compItem.groupName === "Report") {
                    if ((compItem.propertyName === "_use") &&
                        (compItem.propertyValue === "Y")) {
                        modelItem = new sas.hc.m.StandardListItem({
                            title: sas.modelfactory.RB.getText(compItem.componentDisplayName),
                            id: compItem.componentName
                        });
                        modelItem.setTooltip(sas.modelfactory.RB.getText(compItem.propertyDescription));
                        // If this is Score, save it for the last spot in the list
                        if (compItem.componentName === "Score")
                            lastUseItem = modelItem;
                        else if (compItem.groupName === "Report"){
                            this.modelList.insertItem(modelItem, 0);
                        }
                        else {
                            this.modelList.insertItem(modelItem, 1);
                            firstModelItem = modelItem;
                        }
                    } else {
                        // Add the algorithm to the component list
                        var key = compItem.componentName;
                        var propName = compItem.propertyName;

                        if (this.componentList[key]) {
                            var propList = this.componentList[key];
                            propList[propName] = {data: compItem, index: i};
                        } else {
                            var propList = {};
                            propList[propName] = {data: compItem, index: i};
                            this.componentList[key] = propList;
                        }
                    }
                }
            }
        }

        // Use tempUseList to build the useList
        for(var index in tempUseList) {
            var attr = tempUseList[index];
            console.log(attr.name, ": position ", attr.pos);
            this.useList.insertItem(attr.item, attr.pos);
        }

        logUtil.log(this.componentList);

        // If we have Score, put it at the end of the Use list
        if (lastUseItem)
            this.modelList.insertItem(lastUseItem,99);

        // Select the first component in the Use list.
        if (firstUseItem){
            firstUseItem.setSelected(true);
            this.useList.fireSelectionChange({
                "listItem": firstUseItem,
                "selected": true
            });
        } else if (firstModelItem){
            firstModelItem.setSelected(true);
            this.modelList.fireSelectionChange({
                "listItem": firstModelItem,
                "selected": true
            });
        } else if (lastUseItem){
            lastUseItem.setSelected(true);
            this.modelList.fireSelectionChange({
                "listItem": lastUseItem,
                "selected": true
            });
        }

        if (this.isEditable){
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-core');
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-widget');
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-mouse');
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-draggable');
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-droppable');
            jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-sortable');

            var view = this;

            // Have to initialize useListOrder/ignoreListOrder in case the user hits Save without altering either list
            view.useListOrder = [];
            view.ignoreListOrder = [];

            var items = view.useList.getItems();
            if (items) {
                if (items.length === 0){
                    $("#TemplateView--UseList-listUl").addClass("sasDropZoneEmpty");
                }
                for (var i = 0; i < items.length; i++) {
                    this.useListOrder.push(items[i].sId);
                }
            }
            items = view.ignoreList.getItems();
            if (items) {
                if (items.length === 0){
                    $("#TemplateView--IgnoreList-listUl").addClass("sasDropZoneEmpty");
                }
                for (var i = 0; i < items.length; i++) {
                    this.ignoreListOrder.push(items[i].sId);
                }
            }

            setTimeout(function(){
                $("#TemplateView--UseList-listUl").sortable({
                    connectWith: "#TemplateView--IgnoreList-listUl",
                    axis: "y",
                    update: function(event, ui){
                        view.useListOrder = $("#TemplateView--UseList-listUl").sortable("toArray");
                        logUtil.log("update use list", view.useListOrder);
                        if (view.useListOrder && view.useListOrder.length === 0){
                            $("#TemplateView--UseList-listUl").addClass("sasDropZoneEmpty");
                        } else {
                            $("#TemplateView--UseList-listUl").removeClass("sasDropZoneEmpty");
                        }

                        var dialog = view.getViewData();
                        if (dialog){
                            var saveButton = dialog.saveButton;
                            if (saveButton && saveButton !== undefined) {
                                // Enable the Save button on use list changes
                                saveButton.setEnabled(true);
                            }
                        }
                    }
                    /*remove: function(event, ui){
                     logUtil.log("remove:", ui.item, $("#TemplateView--UseList-listUl").sortable("toArray"));
                     },
                     receive: function(event, ui){
                     logUtil.log("receive:", ui.item, $("#TemplateView--UseList-listUl").sortable("toArray"));
                     }*/
                });
                $("#TemplateView--IgnoreList-listUl").sortable({
                    connectWith: "#TemplateView--UseList-listUl",
                    axis: "y",
                    update: function(event, ui){
                        view.ignoreListOrder = $("#TemplateView--IgnoreList-listUl").sortable("toArray");
                        logUtil.log("update ignore list", view.ignoreListOrder);
                        if (view.ignoreListOrder && view.ignoreListOrder.length === 0){
                            $("#TemplateView--IgnoreList-listUl").addClass("sasDropZoneEmpty");
                        } else {
                            $("#TemplateView--IgnoreList-listUl").removeClass("sasDropZoneEmpty");
                        }
                    }
                    /*remove: function(event, ui){
                     logUtil.log("remove:", ui.item);
                     },
                     receive: function(event, ui){
                     logUtil.log("receive:", ui.item);
                     }*/
                });
            }, 0);
        }

        setTimeout(function(){
            var icons = $(".ModelTemplateView-listItem span");
            var titles = $(".ModelTemplateView-listItem .sapMSLIDiv");
            for (var i=0;i<icons.length;i++){
                $(icons[i]).insertAfter($(titles[i]));
                if (!controller.isEditable){
                    $(icons[i]).css({ opacity: 0.65 });
                } else {
                    $(icons[i]).css({ opacity:1 });
                }
            }
        }, 0);
    },

    findItem: function(itemId, itemList) {
        var item = null;
        for (var i=0;i<itemList.length;i++){
            if (itemList[i].getId() == itemId){
                item = itemList[i];
                break;
            }
        }
        return item;
    },

    populateProperties: function(key) {

        var comp = this.componentList[key];

        // Convert to an array and arrange by orderNumber
        var arr = [];
        Object.keys(comp).forEach(function (key) {
            arr.push(comp[key]);
        });
        arr.sort(function(a,b){return a.data.orderNumber - b.data.orderNumber});

        var panel = this.rightSide;
        if (comp) {
            logUtil.log("Component to display:", comp);

            // Clear the panel
            this.rightSide.destroyContent();

//            var vLayout = new sap.ui.layout.VerticalLayout(this.createId("ModelTemplateLayout"), {
//                width: "100%"
//            });

            // Create the form
            var formContainer = new sas.hc.ui.layout.form.FormContainer("F1C1", {
                //title: "Person data",
                //formElements: []
            });

            var title = sas.modelfactory.RB.getText("modelTemplatesView.panel.title.txt");
            var tooltip = sas.modelfactory.RB.getText("modelTemplatesView.panel.tooltip.txt");
            var item = null;
            if (this.useList && this.useList.getSelectedItem()){
                item = this.useList.getSelectedItem();
            } else if (this.ignoreList && this.ignoreList.getSelectedItem()){
                item = this.ignoreList.getSelectedItem();
            } else if (this.modelList && this.modelList.getSelectedItem()){
                item = this.modelList.getSelectedItem();
            }
            if (item){
                title = item.getTitle();
                tooltip = item.getTitle();
            }
            var propertyForm = new sas.hc.ui.layout.form.Form({
                title: new sap.ui.core.Title({
                    text: title,
                    tooltip: tooltip
                }),
                layout:  new sap.ui.layout.form.ResponsiveGridLayout(),
                formContainers: [
                    formContainer
                ]
            });

            for (var i=0;i<arr.length;i++) {
                // Process each property
                var prop = new sas.hc.m.Label({
                    text: arr[i].data.orderNumber + "-" + sas.modelfactory.RB.getText(arr[i].data.propertyDisplayName) + "-" + arr[i].data.propertyValue
                });

                // Get the control
                var element = this.getControl(arr[i]);

                // Add the properties
                //vLayout.addContent(prop);
                formContainer.addFormElement(element);
            };

            panel.addContent(propertyForm)
        }
    },

    formatValue: function(value, type) {
        if (value === "NaN"){
            return "";
        }
        else {
            if (type === "DOUBLE"){

                /*var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
                if (localeStr === null)
                    localeStr = "en_US";

                var floatFmt = sap.ui.core.format.NumberFormat.getFloatInstance({
                    minFractionDigits: 4,
                    maxFractionDigits: 4,
                    groupingEnabled: true
                }, new sap.ui.core.Locale(localeStr));

                // Double
                return floatFmt.format(value);*/
                return value;
            } else {
                // Integer
                return value;
            }
        }
    },

    numFormat: function(value){
        var numberFormat = sap.ui.core.format.NumberFormat.getFloatInstance({
                groupingEnabled: false,
                groupingSeparator: null
            },
            sap.ui.getCore().getConfiguration().getLocale());
        return numberFormat.format(value);
    },

    /**
     * Formats a column's numeric data to locale only.
     *
     * @returns the formatted numeric value as a String
     *
     **/
    formatToLocale: function(value) {
        if (value === undefined || value === null)
            return value;
        else if (isNaN(value))
            return value;

        var localeStr = sap.ui.getCore().getConfiguration().getLanguage();
        if (localeStr === null)
            localeStr = "en_US";

        var floatFmt = sap.ui.core.format.NumberFormat.getFloatInstance({
            groupingEnabled: true
        }, new sap.ui.core.Locale(localeStr));

        var trimmed = value.trim(); // Remove any leading/trailing whitespace
        var retVal = floatFmt.format(trimmed);

        return retVal;
    },

    getControl: function(entry) {
        var view = this;

        var property = entry.data;
        logUtil.log("Getting control for property: ", sas.modelfactory.RB.getText(property.propertyDisplayName), "-", property.propertyName);

        // Create the form element
        var formItem = new sap.ui.layout.form.FormElement();

        // Add the label
        var labelText = sas.modelfactory.RB.getText(property.propertyDisplayName) + ":";
        if (property.propertyValueType !== "BOOLEAN") {
            var label = new sap.ui.commons.Label({
                text: labelText,
                //layoutData:  new sap.ui.layout.GridData({span: "L5 M5 S12"})
                layoutData:  new sap.ui.layout.GridData({span: "L12 M12 S12"})    // Stack the labels
            });
            formItem.setLabel(label);
        }

        // Path into the model for the control value
        var path = "/template/properties/" + entry.index + "/propertyValue";

        if (property.constraint)
        {
            // Range or choiceList
            var constraint = property.constraint;
            var tip = "";
            switch(constraint.type)
            {
                case "choice":
                {
                    logUtil.log("       - choice property type");
                    logUtil.log("       - ", property.propertyValue);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");

                    var comboBox = new sas.hc.m.Select({
                        tooltip: property.propertyDescription,
                        //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                        layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                    }).addStyleClass("editMetadataDialog-dropdown");

                    // Add choices
                    var list = constraint.choices;
                    for (var i=0;i<list.length;i++) {
                        var oItem = new sas.hc.ui.core.ListItem({
                            key: list[i].name
                        });
                        var localizedItem = this.formatToLocale(sas.modelfactory.RB.getText(list[i].displayName));
                        oItem.setText(localizedItem);
                        comboBox.addItem(oItem);
                    }
                    comboBox.setSelectedKey(property.propertyValue);
                    comboBox.bindProperty("selectedKey", path);

                    formItem.addField(comboBox);

                    break;
                }

                case "range":
                {
                    logUtil.log("       - range property type");
                    logUtil.log("       - ", property.propertyValue);

                    var EXCLUDE_MIN_FLAG = 1;
                    var EXCLUDE_MAX_FLAG = 2;
                    var isInteger = property.propertyValueType === "INTEGER";
                    var hasMax = constraint.max != null;
                    var hasMin = constraint.min != null;
                    var flag = constraint.flags;
                    var excludeMin = (flag & EXCLUDE_MIN_FLAG) === EXCLUDE_MIN_FLAG;
                    var excludeMax = (flag & EXCLUDE_MAX_FLAG) === EXCLUDE_MAX_FLAG;
                    var max = isInteger ? parseInt(constraint.max) : parseFloat(constraint.max);
                    var min = isInteger ? parseInt(constraint.min) : parseFloat(constraint.min);

                    var value = 0;
                    if (isInteger)
                        value = parseInt(property.propertyValue);
                    else
                        value = parseFloat(property.propertyValue);

                    var stepSize = 1;
                    if (hasMax && hasMin && !isInteger){
                        // Is this a percentage?
                        if ((min >= 0 && min < 1) && max <= 1) {
                            stepSize = (max - min) / 10000; // 4 decimal places
                        }
                    }

                    logUtil.log("       - isInteger=", isInteger?"Y":"N");
                    logUtil.log("       - max=", max);
                    logUtil.log("       - min=", min);
                    logUtil.log("       - excludeMin=", excludeMin?"Y":"N");
                    logUtil.log("       - excludeMax=", excludeMax?"Y":"N");
                    logUtil.log("       - stepSize=", stepSize);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");

                    // Only present the slider if there is a min and max and the minimum value is zero, or is large
                    // enough to allow scrolling.
//                    if (hasMax && hasMin && (min === 0 || min > 0.001)) {
                    if ((hasMax && hasMin) &&
                        ((isInteger) || (!isInteger && min === 0 && max == 1))) {

                        // Slider doesn't support exclude boundary properties so do a calculation for actual boundaries
                        if (excludeMin) {
                            min = min + stepSize;
                        }
                        if (excludeMax) {
                            max = max - stepSize;
                        }

                        var slider = new sas.hc.ui.commons.Slider({
                            tooltip: property.propertyDescription,
                            //width: "95%",
                            min: min,
                            max: max,
                            value: value,
//                            totalUnits: 5,
                            smallStepWidth: stepSize,
                            //step: stepSize,
                            totalUnits: 4,
                            stepLabels : true,
                            liveChange : function(){
                                // This event called on key press
                                var valueStr = slider.getValue().toString();
                                var template = view.getModel().setProperty(path, valueStr);
                            },
                            change : function(){
                                // This event called on completion of edit: (focus exit and enter key)
                                    var valueStr = slider.getValue().toString();
                                    var template = view.getModel().setProperty(path, valueStr);
                            },
                            layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                        }).addStyleClass("editMetadataDialog-slider");

                        if (property.propertyName === "maxPrincomp" && view.maxPrinPath){
                            slider.bindProperty("enabled", view.maxPrinPath, function(bValue){
                                return bValue === "Y"?true:false;
                            });
                        } else if (property.propertyName === "fixPrincomp" && view.fixPrinPath){
                            slider.bindProperty("enabled", view.fixPrinPath, function(bValue){
                                return bValue === "Y"?true:false;
                            });
                        }

                        formItem.addField(slider);
                    } else {
                        // Otherwise, use a validation textbox for these numeric values
                        var numberFormat = sap.ui.core.format.NumberFormat.getFloatInstance({
                                groupingEnabled: false,
                                groupingSeparator: null
                            },
                            sap.ui.getCore().getConfiguration().getLocale());
                        var rangeInput = new sas.modelfactory.control.rangeinput.RangeInput({
                            //id : 'input1',
                            value: numberFormat.format(value),
                            type: sap.m.InputType.Number,
                            tooltip: sas.modelfactory.RB.getText(tip),
                            min: min,
                            max: max,
                            hasMin: hasMin,
                            hasMax: hasMax,
                            excludeMin: excludeMin,
                            excludeMax: excludeMax,
                            isInteger: isInteger,
                            enableGroupingSeparator: false,

                            liveChange : function(){
                                // This event called on key press
                                if (rangeInput.getValueState() === sap.ui.core.ValueState.None) {
                                    var rVal = numberFormat.parse(rangeInput.getValue());
                                    var valueStr = rVal.toString();
                                    var template = view.getModel().setProperty(path, valueStr);
                                }
                            },
                            change : function(){
                                // This event called on completion of edit: (focus exit and enter key)
                                if (rangeInput.getValueState() === sap.ui.core.ValueState.None) {
                                    var rVal = numberFormat.parse(rangeInput.getValue());
                                    var valueStr = rVal.toString();
                                    var template = view.getModel().setProperty(path, valueStr);
                                }
                            },
                            //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                            layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                        }).addStyleClass("editMetadataDialog-textInput");

                        //rangeInput.bindValue(path);

                        formItem.addField(rangeInput);
                    }

                    break;
                }

                default:
                {
                    sas.hc.m.MessageBox.error("Unknown property type!");
                    break;
                }
            }
        }
        else
        {
            switch(property.propertyValueType)
            {
                case "BOOLEAN":   // boolean
                {
                    logUtil.log("       - boolean property type");
                    logUtil.log("       - ", property.propertyValue);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");

                    // Hack to deal with dependant controls.  Remove this code when the properties
                    // support this relationship.
                    if (property.propertyName === "applyMaxPrincomp" || property.propertyName === "applyFixPrincomp"){
                        var radioButton = new sas.hc.m.RadioButton({
                            text: sas.modelfactory.RB.getText(property.propertyDisplayName),
                            tooltip: property.propertyDescription,
                            groupName: "groupA",
                            selected: property.propertyValue === "Y",
                            select: function(evt){
                                var model = view.getModel();
                                var isChecked = evt.getParameter("selected");
                                //model.oData.template.properties[entry.index].propertyValue = isChecked?"Y":"N";
                                var propPath = "/template/properties/" + entry.index + "/propertyValue";
                                model.setProperty(propPath, isChecked?"Y":"N");
                            },
                            layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})       // Stack the labels
                        });

                        if (property.propertyName === "applyMaxPrincomp"){
                            view.maxPrinPath = path;
                        } else if (property.propertyName === "applyFixPrincomp"){
                            view.fixPrinPath = path;
                        }

                        formItem.addField(radioButton);
                    } else {
                        var checkBox = new sas.hc.m.CheckBox({
                            text: sas.modelfactory.RB.getText(property.propertyDisplayName),
                            tooltip: property.propertyDescription,
                            selected: property.propertyValue === "Y",
                            select: function(evt){
                                var model = view.getModel();
                                var isChecked = evt.getParameter("selected");
                                //model.oData.template.properties[entry.index].propertyValue = isChecked?"Y":"N";
                                var propPath = "/template/properties/" + entry.index + "/propertyValue";
                                model.setProperty(propPath, isChecked?"Y":"N");
                            },
                            //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                            layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})       // Stack the labels
                        });

                        formItem.addField(checkBox);
                    }

                    break;
                }

                case "STRING":   // string
                {
                    logUtil.log("       - string property type");
                    logUtil.log("       - ", property.propertyValue);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");
                    var placeholder = sas.modelfactory.RB.getText("modelTemplatesView.prompt.txt") + " " + sas.modelfactory.RB.getText(property.propertyDisplayName);

                    var textInput = new sas.hc.m.Input({
                        value: property.propertyValue,
                        tooltip: property.propertyDescription,
                        placeholder: placeholder,
                        //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                        layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                    }).addStyleClass("editMetadataDialog-textInput");
                    textInput.bindValue(path);

                    formItem.addField(textInput);

                    break;
                }

                // TODO: S1222811 - As of 1/4/2016, propertyValueType of INTEGER is not being used for any model template property.
                //       When properties are added at a later date localization needs to be verified for these controls.
                case "INTEGER":   // integer
                {
                    logUtil.log("       - integer property type");
                    logUtil.log("       - ", property.propertyValue);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");
                    var placeholder = sas.modelfactory.RB.getText("modelTemplatesView.prompt.txt") + " " + sas.modelfactory.RB.getText(property.propertyDisplayName);

                    var intInput = new sas.hc.m.Input({
                        value: view.formatValue(property.propertyValue, property.propertyValueType),
                        type: sap.m.InputType.Number,
                        allowIntegerOnly: true,
                        tooltip: property.propertyDescription,
                        placeholder: placeholder,
                        //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                        layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                    }).addStyleClass("editMetadataDialog-textInput");
                    intInput.bindValue(path);

                    formItem.addField(intInput);

                    break;
                }

                // TODO: S1222811 - As of 1/4/2016, propertyValueType of INTEGER is not being used for any model template property.
                //       When properties are added at a later date localization needs to be verified for these controls.
                case "DOUBLE":   // double
                {
                    logUtil.log("       - double property type");
                    logUtil.log("       - ", property.propertyValue);

                    var tip = property.propertyDisplayName.replace(".txt", ".desc.txt");
                    var placeholder = sas.modelfactory.RB.getText("modelTemplatesView.prompt.txt") + " " + sas.modelfactory.RB.getText(property.propertyDisplayName);

                    var doubleInput = new sas.hc.m.Input({
                        value: view.formatValue(property.propertyValue, property.propertyValueType),
                        type: sap.m.InputType.Number,
                        tooltip: property.propertyDescription,
                        placeholder: placeholder,
                        //layoutData: new sap.ui.layout.GridData({span: "L7 M7 S12"})
                        layoutData: new sap.ui.layout.GridData({span: "L12 M12 S12"})     // Stack the labels
                    }).addStyleClass("editMetadataDialog-textInput");
                    doubleInput.bindValue(path);

                    formItem.addField(doubleInput);

                    break;
                }

                default:
                {
                    sas.hc.m.MessageBox.error("Unknown property type!");
                }
            }
        }

        if(formItem){
            var fields = formItem.getFields();
            if(fields && fields.length > 0){
                for(var i=0;i<fields.length;i++){
                    var comp = fields[i];
                    comp.setEnabled(this.isEditable);
                }
            }
        }

        return formItem;

    }
});